@@ -2,36 +2,41 @@
 # Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 An asynchronous mapping to U{DB-API 2.0<http://www.python.org/topics/database/DatabaseAPI-2.0.html>}.
 """
 
-from twisted.internet import defer, threads
+import sys
+
+from twisted.internet import threads
 from twisted.python import reflect, log
 from twisted.python.deprecate import deprecated
 from twisted.python.versions import Version
 
 
+
 class ConnectionLost(Exception):
-    """This exception means that a db connection has been lost.
-    Client code may try again."""
-    pass
-    
-    
+    """
+    This exception means that a db connection has been lost.  Client code may
+    try again.
+    """
+
+
+
 class Connection(object):
-    """A wrapper for a DB-API connection instance.
-    
+    """
+    A wrapper for a DB-API connection instance.
+
     The wrapper passes almost everything to the wrapped connection and so has
     the same API. However, the Connection knows about its pool and also
     handle reconnecting should when the real connection dies.
     """
-    
+
     def __init__(self, pool):
         self._pool = pool
         self._connection = None
         self.reconnect()
-        
+
     def close(self):
         # The way adbapi works right now means that closing a connection is
         # a really bad thing  as it leaves a dead connection associated with
@@ -41,7 +46,7 @@ class Connection(object):
         # rather than upsetting anyone by raising an exception, let's ignore
         # the request
         pass
-        
+
     def rollback(self):
         if not self._pool.reconnect:
             self._connection.rollback()
@@ -55,12 +60,12 @@ class Connection(object):
             self._connection.commit()
             return
         except:
-            pass
+            log.err(None, "Rollback failed")
 
         self._pool.disconnect(self._connection)
 
         if self._pool.noisy:
-            log.msg('Connection lost.')
+            log.msg("Connection lost.")
 
         raise ConnectionLost()
 
@@ -68,11 +73,11 @@ class Connection(object):
         if self._connection is not None:
             self._pool.disconnect(self._connection)
         self._connection = self._pool.connect()
-        
+
     def __getattr__(self, name):
         return getattr(self._connection, name)
-        
-        
+
+
 class Transaction:
     """A lightweight wrapper for a DB-API 'cursor' object.
 
@@ -103,6 +108,8 @@ class Transaction:
         except:
             if not self._pool.reconnect:
                 raise
+            else:
+                log.err(None, "Cursor creation failed")
 
         if self._pool.noisy:
             log.msg('Connection lost, reconnecting')
@@ -119,12 +126,20 @@ class Transaction:
 
 
 class ConnectionPool:
-    """I represent a pool of connections to a DB-API 2.0 compliant database.
+    """
+    Represent a pool of connections to a DB-API 2.0 compliant database.
+
+    @ivar connectionFactory: factory for connections, default to L{Connection}.
+    @type connectionFactory: any callable.
+
+    @ivar transactionFactory: factory for transactions, default to
+        L{Transaction}.
+    @type transactionFactory: any callable
     """
 
     CP_ARGS = "min max name noisy openfun reconnect good_sql".split()
 
-    noisy = True # if true, generate informational log messages
+    noisy = False # if true, generate informational log messages
     min = 3 # minimum number of connections in pool
     max = 5 # maximum number of connections in pool
     name = None # Name to assign to thread pool for debugging
@@ -133,6 +148,8 @@ class ConnectionPool:
     good_sql = 'select 1' # a query which should always succeed
 
     running = False # true when the pool is operating
+    connectionFactory = Connection
+    transactionFactory = Transaction
 
     def __init__(self, dbapiName, *connargs, **connkw):
         """Create a new ConnectionPool.
@@ -217,23 +234,53 @@ class ConnectionPool:
                                                             'shutdown',
                                                             self.finalClose)
             self.running = True
-            
+
+
     def runWithConnection(self, func, *args, **kw):
-        return self._deferToThread(self._runWithConnection,
-                                   func, *args, **kw)
+        """
+        Execute a function with a database connection and return the result.
+
+        @param func: A callable object of one argument which will be executed
+            in a thread with a connection from the pool.  It will be passed as
+            its first argument a L{Connection} instance (whose interface is
+            mostly identical to that of a connection object for your DB-API
+            module of choice), and its results will be returned as a Deferred.
+            If the method raises an exception the transaction will be rolled
+            back.  Otherwise, the transaction will be committed.  B{Note} that
+            this function is B{not} run in the main thread: it must be
+            threadsafe.
+
+        @param *args: positional arguments to be passed to func
+
+        @param **kw: keyword arguments to be passed to func
+
+        @return: a Deferred which will fire the return value of
+            C{func(Transaction(...), *args, **kw)}, or a Failure.
+        """
+        from twisted.internet import reactor
+        return threads.deferToThreadPool(reactor, self.threadpool,
+                                         self._runWithConnection,
+                                         func, *args, **kw)
+
 
     def _runWithConnection(self, func, *args, **kw):
-        conn = Connection(self)
+        conn = self.connectionFactory(self)
         try:
             result = func(conn, *args, **kw)
             conn.commit()
             return result
         except:
-            conn.rollback()
-            raise
-        
+            excType, excValue, excTraceback = sys.exc_info()
+            try:
+                conn.rollback()
+            except:
+                log.err(None, "Rollback failed")
+            raise excType, excValue, excTraceback
+
+
     def runInteraction(self, interaction, *args, **kw):
-        """Interact with the database and return the result.
+        """
+        Interact with the database and return the result.
 
         The 'interaction' is a callable object which will be executed
         in a thread using a pooled connection. It will be passed an
@@ -249,16 +296,22 @@ class ConnectionPool:
         function you pass to this if it tries to use non-local
         objects.
 
-        @param interaction: a callable object whose first argument is
-            L{adbapi.Transaction}. *args,**kw will be passed as
-            additional arguments.
+        @param interaction: a callable object whose first argument
+            is an L{adbapi.Transaction}.
+
+        @param *args: additional positional arguments to be passed
+            to interaction
+
+        @param **kw: keyword arguments to be passed to interaction
 
         @return: a Deferred which will fire the return value of
-            'interaction(Transaction(...))', or a Failure.
+            'interaction(Transaction(...), *args, **kw)', or a Failure.
         """
+        from twisted.internet import reactor
+        return threads.deferToThreadPool(reactor, self.threadpool,
+                                         self._runInteraction,
+                                         interaction, *args, **kw)
 
-        return self._deferToThread(self._runInteraction,
-                                   interaction, *args, **kw)
 
     def runQuery(self, *args, **kw):
         """Execute an SQL query and return the result.
@@ -277,9 +330,9 @@ class ConnectionPool:
         @return: a Deferred which will fire the return value of a DB-API
         cursor's 'fetchall' method, or a Failure.
         """
-
         return self.runInteraction(self._runQuery, *args, **kw)
 
+
     def runOperation(self, *args, **kw):
         """Execute an SQL query and return None.
 
@@ -296,9 +349,9 @@ class ConnectionPool:
 
         return: a Deferred which will fire None or a Failure.
         """
-
         return self.runInteraction(self._runOperation, *args, **kw)
 
+
     def close(self):
         """Close all pool connections and shutdown the pool."""
 
@@ -352,7 +405,6 @@ class ConnectionPool:
         called connect(). As with connect(), this function is not used
         in normal non-threaded twisted code.
         """
-
         tid = self.threadID()
         if conn is not self.connections.get(tid):
             raise Exception("wrong connection for thread")
@@ -360,25 +412,32 @@ class ConnectionPool:
             self._close(conn)
             del self.connections[tid]
 
+
     def _close(self, conn):
         if self.noisy:
             log.msg('adbapi closing: %s' % (self.dbapiName,))
         try:
             conn.close()
         except:
-            pass
+            log.err(None, "Connection close failed")
+
 
     def _runInteraction(self, interaction, *args, **kw):
-        conn = Connection(self)
-        trans = Transaction(self, conn)
+        conn = self.connectionFactory(self)
+        trans = self.transactionFactory(self, conn)
         try:
             result = interaction(trans, *args, **kw)
             trans.close()
             conn.commit()
             return result
         except:
-            conn.rollback()
-            raise
+            excType, excValue, excTraceback = sys.exc_info()
+            try:
+                conn.rollback()
+            except:
+                log.err(None, "Rollback failed")
+            raise excType, excValue, excTraceback
+
 
     def _runQuery(self, trans, *args, **kw):
         trans.execute(*args, **kw)
@@ -401,21 +460,10 @@ class ConnectionPool:
         self.__dict__ = state
         self.__init__(self.dbapiName, *self.connargs, **self.connkw)
 
-    def _deferToThread(self, f, *args, **kwargs):
-        """Internal function.
-
-        Call f in one of the connection pool's threads.
-        """
-
-        d = defer.Deferred()
-        self.threadpool.callInThread(threads._putResultInDeferred,
-                                     d, f, args, kwargs)
-        return d
-
 
 
 # Common deprecation decorator used for all deprecations.
-_unreleasedVersion = Version("Twisted", 2, 6, 0)
+_unreleasedVersion = Version("Twisted", 8, 0, 0)
 _unreleasedDeprecation = deprecated(_unreleasedVersion)
 
 
