@@ -1,20 +1,20 @@
-# Copyright (c) 2008 Twisted Matrix Laboratories.
+# Copyright (c) 2008-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 TCP support for IOCP reactor
 """
 
+import socket, operator, errno, struct
+
+from zope.interface import implements, directlyProvides
+
 from twisted.internet import interfaces, error, address, main, defer
 from twisted.internet.abstract import isIPAddress
 from twisted.internet.tcp import _SocketCloser, Connector as TCPConnector
 from twisted.persisted import styles
 from twisted.python import log, failure, reflect, util
 
-from zope.interface import implements
-import socket, operator, errno, struct
-
 from twisted.internet.iocpreactor import iocpsupport as _iocp, abstract
 from twisted.internet.iocpreactor.interfaces import IReadWriteHandle
 from twisted.internet.iocpreactor.const import ERROR_IO_PENDING
@@ -23,6 +23,14 @@ from twisted.internet.iocpreactor.const import SO_UPDATE_ACCEPT_CONTEXT
 from twisted.internet.iocpreactor.const import ERROR_CONNECTION_REFUSED
 from twisted.internet.iocpreactor.const import ERROR_NETWORK_UNREACHABLE
 
+try:
+    from twisted.protocols.tls import TLSMemoryBIOFactory, TLSMemoryBIOProtocol
+except ImportError:
+    TLSMemoryBIOProtocol = TLSMemoryBIOFactory = None
+    _extraInterfaces = ()
+else:
+    _extraInterfaces = (interfaces.ITLSTransport,)
+
 # ConnectEx returns these. XXX: find out what it does for timeout
 connectExErrors = {
         ERROR_CONNECTION_REFUSED: errno.WSAECONNREFUSED,
@@ -30,11 +38,54 @@ connectExErrors = {
         }
 
 
+class _BypassTLS(object):
+    """
+    L{_BypassTLS} is used as the transport object for the TLS protocol object
+    used to implement C{startTLS}.  Its methods skip any TLS logic which
+    C{startTLS} enables.
+
+    @ivar _connection: A L{Connection} which TLS has been started on which will
+        be proxied to by this object.  Any method which has its behavior
+        altered after C{startTLS} will be skipped in favor of the base class's
+        implementation.  This allows the TLS protocol object to have direct
+        access to the transport, necessary to actually implement TLS.
+    """
+    def __init__(self, connection):
+        self._connection = connection
+
+
+    def __getattr__(self, name):
+        return getattr(self._connection, name)
+
+
+    def write(self, data):
+        return abstract.FileHandle.write(self._connection, data)
+
+
+    def writeSequence(self, iovec):
+        return abstract.FileHandle.writeSequence(self._connection, iovec)
+
+
+    def loseConnection(self, reason=None):
+        return abstract.FileHandle.loseConnection(self._connection, reason)
+
+
 
 class Connection(abstract.FileHandle, _SocketCloser):
-    implements(IReadWriteHandle, interfaces.ITCPTransport,
-               interfaces.ISystemHandle)
+    """
+    @ivar _tls: C{False} to indicate the connection is in normal TCP mode,
+        C{True} to indicate that TLS has been started and that operations must
+        be routed through the L{TLSMemoryBIOProtocol} instance.
 
+    @ivar _tlsClientDefault: A flag which must be set by a subclass.  If set to
+        C{True}, L{startTLS} will default to initiating SSL as a client.  If
+        set to C{False}, L{startTLS} will default to initiating SSL as a
+        server.
+    """
+    implements(IReadWriteHandle, interfaces.ITCPTransport,
+               interfaces.ISystemHandle, *_extraInterfaces)
+
+    _tls = False
 
     def __init__(self, sock, proto, reactor=None):
         abstract.FileHandle.__init__(self, reactor)
@@ -117,16 +168,91 @@ class Connection(abstract.FileHandle, _SocketCloser):
         return operator.truth(self.socket.getsockopt(socket.SOL_SOCKET,
                                                      socket.SO_KEEPALIVE))
 
-        
+
     def setTcpKeepAlive(self, enabled):
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, enabled)
 
 
+    if TLSMemoryBIOFactory is not None:
+        def startTLS(self, contextFactory, normal=True):
+            """
+            @see: L{ITLSTransport.startTLS}
+            """
+            # Figure out which direction the SSL goes in.  If normal is True,
+            # we'll go in the direction indicated by the subclass.  Otherwise,
+            # we'll go the other way (client = not normal ^ _tlsClientDefault,
+            # in other words).
+            if normal:
+                client = self._tlsClientDefault
+            else:
+                client = not self._tlsClientDefault
+
+            tlsFactory = TLSMemoryBIOFactory(contextFactory, client, None)
+            tlsProtocol = TLSMemoryBIOProtocol(tlsFactory, self.protocol, False)
+            self.protocol = tlsProtocol
+
+            self.getHandle = tlsProtocol.getHandle
+            self.getPeerCertificate = tlsProtocol.getPeerCertificate
+
+            # Mark the transport as secure.
+            directlyProvides(self, interfaces.ISSLTransport)
+
+            # Remember we did this so that write and writeSequence can send the
+            # data to the right place.
+            self._tls = True
+
+            # Hook it up
+            self.protocol.makeConnection(_BypassTLS(self))
+
+
+    def write(self, data):
+        """
+        Write some data, either directly to the underlying handle or, if TLS
+        has been started, to the L{TLSMemoryBIOProtocol} for it to encrypt and
+        send.
+
+        @see: L{ITCPTransport.write}
+        """
+        if self._tls:
+            self.protocol.write(data)
+        else:
+            abstract.FileHandle.write(self, data)
+
+
+    def writeSequence(self, iovec):
+        """
+        Write some data, either directly to the underlying handle or, if TLS
+        has been started, to the L{TLSMemoryBIOProtocol} for it to encrypt and
+        send.
+
+        @see: L{ITCPTransport.writeSequence}
+        """
+        if self._tls:
+            self.protocol.writeSequence(iovec)
+        else:
+            abstract.FileHandle.writeSequence(self, iovec)
+
+
+    def loseConnection(self, reason=None):
+        """
+        Close the underlying handle or, if TLS has been started, first shut it
+        down.
+
+        @see: L{ITCPTransport.loseConnection}
+        """
+        if self._tls:
+            if self.connected and not self.disconnecting:
+                self.protocol.loseConnection()
+        else:
+            abstract.FileHandle.loseConnection(self, reason)
+
+
 
 class Client(Connection):
     addressFamily = socket.AF_INET
     socketType = socket.SOCK_STREAM
 
+    _tlsClientDefault = True
 
     def __init__(self, host, port, bindAddress, connector, reactor):
         self.connector = connector
@@ -147,7 +273,7 @@ class Client(Connection):
                 except socket.error, se:
                     raise error.ConnectBindError(se[0], se[1])
                 self.socket = skt
-                Connection.__init__(self, skt, None)
+                Connection.__init__(self, skt, None, reactor)
                 reactor.callLater(0, self.resolveAddress)
         except error.ConnectBindError, err:
             reactor.callLater(0, self.failIfNotConnected, err)
@@ -239,7 +365,7 @@ class Client(Connection):
 
         This indicates the address that I am connected to.
         """
-        return address.IPv4Address('TCP', *(self.addr + ('INET',)))
+        return address.IPv4Address('TCP', *(self.realAddress + ('INET',)))
 
 
     def __repr__(self):
@@ -265,8 +391,10 @@ class Server(Connection):
     accept() on a server.
     """
 
+    _tlsClientDefault = False
 
-    def __init__(self, sock, protocol, clientAddr, serverAddr, sessionno):
+
+    def __init__(self, sock, protocol, clientAddr, serverAddr, sessionno, reactor):
         """
         Server(sock, protocol, client, server, sessionno)
 
@@ -274,7 +402,7 @@ class Server(Connection):
         tuple of host, port describing the other end of the connection), an
         instance of Port, and a session number.
         """
-        Connection.__init__(self, sock, protocol)
+        Connection.__init__(self, sock, protocol, reactor)
         self.serverAddr = serverAddr
         self.clientAddr = clientAddr
         self.sessionno = sessionno
@@ -344,9 +472,6 @@ class Port(styles.Ephemeral, _SocketCloser):
         self.interface = interface
         self.reactor = reactor
 
-        skt = socket.socket(self.addressFamily, self.socketType)
-        self.addrLen = _iocp.maxAddrLen(skt.fileno())
-
 
     def __repr__(self):
         if self._realPortNumber is not None:
@@ -367,6 +492,8 @@ class Port(styles.Ephemeral, _SocketCloser):
         except socket.error, le:
             raise error.CannotListenError, (self.interface, self.port, le)
 
+        self.addrLen = _iocp.maxAddrLen(skt.fileno())
+
         # Make sure that if we listened on port 0, we update that to
         # reflect what the OS actually assigned us.
         self._realPortNumber = skt.getsockname()[1]
@@ -377,6 +504,7 @@ class Port(styles.Ephemeral, _SocketCloser):
         self.factory.doStart()
         skt.listen(self.backlog)
         self.connected = True
+        self.disconnected = False
         self.reactor.addActiveHandle(self)
         self.socket = skt
         self.getFileHandle = self.socket.fileno
@@ -402,20 +530,34 @@ class Port(styles.Ephemeral, _SocketCloser):
 
     def connectionLost(self, reason):
         """
-        Cleans up my socket.
+        Cleans up the socket.
         """
         log.msg('(Port %s Closed)' % self._realPortNumber)
         self._realPortNumber = None
+        d = None
+        if hasattr(self, "deferred"):
+            d = self.deferred
+            del self.deferred
+
         self.disconnected = True
         self.reactor.removeActiveHandle(self)
         self.connected = False
         self._closeSocket()
         del self.socket
         del self.getFileHandle
-        self.factory.doStop()
-        if hasattr(self, "deferred"):
-            self.deferred.callback(None)
-            del self.deferred
+
+        try:
+            self.factory.doStop()
+        except:
+            self.disconnecting = False
+            if d is not None:
+                d.errback(failure.Failure())
+            else:
+                raise
+        else:
+            self.disconnecting = False
+            if d is not None:
+                d.callback(None)
 
 
     def logPrefix(self):
@@ -468,7 +610,7 @@ class Port(styles.Ephemeral, _SocketCloser):
                 transport = Server(evt.newskt, protocol,
                         address.IPv4Address('TCP', rAddr[0], rAddr[1], 'INET'),
                         address.IPv4Address('TCP', lAddr[0], lAddr[1], 'INET'),
-                        s)
+                        s, self.reactor)
                 protocol.makeConnection(transport)
             return True
 
