@@ -1,6 +1,6 @@
 # -*- test-case-name: twisted.test.test_factories -*-
 #
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
@@ -10,7 +10,7 @@ Standard implementations of Twisted protocol-related interfaces.
 Start here if you are looking to write a new protocol implementation for
 Twisted.  The Protocol class contains some introductory material.
 
-Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
+Maintainer: Itamar Shtull-Trauring
 """
 
 import random
@@ -196,7 +196,8 @@ class ClientCreator:
 
 
 class ReconnectingClientFactory(ClientFactory):
-    """My clients auto-reconnect with an exponential back-off.
+    """
+    Factory which auto-reconnects clients with an exponential back-off.
 
     Note that clients should call my resetDelay method after they have
     connected successfully.
@@ -206,37 +207,47 @@ class ReconnectingClientFactory(ClientFactory):
     @ivar factor: a multiplicitive factor by which the delay grows
     @ivar jitter: percentage of randomness to introduce into the delay length
         to prevent stampeding.
+    @ivar clock: the clock used to schedule reconnection. It's mainly useful to
+        be parametrized in tests. If the factory is serialized, this attribute
+        will not be serialized, and the default value (the reactor) will be
+        restored when deserialized.
     """
     maxDelay = 3600
     initialDelay = 1.0
     # Note: These highly sensitive factors have been precisely measured by
     # the National Institute of Science and Technology.  Take extreme care
     # in altering them, or you may damage your Internet!
+    # (Seriously: <http://physics.nist.gov/cuu/Constants/index.html>)
     factor = 2.7182818284590451 # (math.e)
     # Phi = 1.6180339887498948 # (Phi is acceptable for use as a
     # factor if e is too large for your application.)
-    jitter = 0.11962656492 # molar Planck constant times c, Jule meter/mole
+    jitter = 0.11962656472 # molar Planck constant times c, joule meter/mole
 
     delay = initialDelay
     retries = 0
     maxRetries = None
     _callID = None
     connector = None
+    clock = None
 
     continueTrying = 1
 
+
     def clientConnectionFailed(self, connector, reason):
         if self.continueTrying:
             self.connector = connector
             self.retry()
 
+
     def clientConnectionLost(self, connector, unused_reason):
         if self.continueTrying:
             self.connector = connector
             self.retry()
 
+
     def retry(self, connector=None):
-        """Have this connector connect again, after a suitable delay.
+        """
+        Have this connector connect again, after a suitable delay.
         """
         if not self.continueTrying:
             if self.noisy:
@@ -263,15 +274,19 @@ class ReconnectingClientFactory(ClientFactory):
 
         if self.noisy:
             log.msg("%s will retry in %d seconds" % (connector, self.delay,))
-        from twisted.internet import reactor
 
         def reconnector():
             self._callID = None
             connector.connect()
-        self._callID = reactor.callLater(self.delay, reconnector)
+        if self.clock is None:
+            from twisted.internet import reactor
+            self.clock = reactor
+        self._callID = self.clock.callLater(self.delay, reconnector)
+
 
     def stopTrying(self):
-        """I put a stop to any attempt to reconnect in progress.
+        """
+        Put a stop to any attempt to reconnect in progress.
         """
         # ??? Is this function really stopFactory?
         if self._callID:
@@ -286,10 +301,11 @@ class ReconnectingClientFactory(ClientFactory):
                 pass
         self.continueTrying = 0
 
-    def resetDelay(self):
-        """Call me after a successful connection to reset.
 
-        I reset the delay and the retry counter.
+    def resetDelay(self):
+        """
+        Call this method after a successful connection: it resets the delay and
+        the retry counter.
         """
         self.delay = self.initialDelay
         self.retries = 0
@@ -306,7 +322,7 @@ class ReconnectingClientFactory(ClientFactory):
         """
         state = self.__dict__.copy()
         for key in ['connector', 'retries', 'delay',
-                    'continueTrying', '_callID']:
+                    'continueTrying', '_callID', 'clock']:
             if key in state:
                 del state[key]
         return state
@@ -476,6 +492,14 @@ class ProcessProtocol(BaseProtocol):
         """
 
 
+    def processExited(self, reason):
+        """
+        This will be called when the subprocess exits.
+
+        @type reason: L{twisted.python.failure.Failure}
+        """
+
+
     def processEnded(self, reason):
         """
         This will be called when the subprocess is finished.
@@ -484,6 +508,7 @@ class ProcessProtocol(BaseProtocol):
         """
 
 
+
 class AbstractDatagramProtocol:
     """
     Abstract protocol for datagram-oriented transports, e.g. IP, ICMP, ARP, UDP.
