@@ -1,13 +1,13 @@
 # -*- test-case-name: twisted.test.test_ftp -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 An FTP protocol implementation
 
-@author: U{Itamar Shtull-Trauring<mailto:itamarst@twistedmatrix.com>}
-@author: U{Jp Calderone<mailto:exarkun@divmod.com>}
-@author: U{Andrew Bennetts<mailto:spiv@twistedmatrix.com>}
+@author: Itamar Shtull-Trauring
+@author: Jp Calderone
+@author: Andrew Bennetts
 """
 
 # System Imports
@@ -18,6 +18,7 @@ import operator
 import stat
 import errno
 import fnmatch
+import warnings
 
 try:
     import pwd, grp
@@ -32,6 +33,7 @@ from twisted.internet import reactor, interfaces, protocol, error, defer
 from twisted.protocols import basic, policies
 
 from twisted.python import log, failure, filepath
+from twisted.python.compat import reduce
 
 from twisted.cred import error as cred_error, portal, credentials, checkers
 
@@ -483,18 +485,29 @@ class DTP(object, protocol.Protocol):
 
 class DTPFactory(protocol.ClientFactory):
     """
-    DTP protocol factory.
+    Client factory for I{data transfer process} protocols.
 
     @ivar peerCheck: perform checks to make sure the ftp-pi's peer is the same
         as the dtp's
     @ivar pi: a reference to this factory's protocol interpreter
+
+    @ivar _state: Indicates the current state of the DTPFactory.  Initially,
+        this is L{_IN_PROGRESS}.  If the connection fails or times out, it is
+        L{_FAILED}.  If the connection succeeds before the timeout, it is
+        L{_FINISHED}.
     """
 
+    _IN_PROGRESS = object()
+    _FAILED = object()
+    _FINISHED = object()
+
+    _state = _IN_PROGRESS
+
     # -- configuration variables --
     peerCheck = False
 
     # -- class variables --
-    def __init__(self, pi, peerHost=None):
+    def __init__(self, pi, peerHost=None, reactor=None):
         """Constructor
         @param pi: this factory's protocol interpreter
         @param peerHost: if peerCheck is True, this is the tuple that the
@@ -503,48 +516,63 @@ class DTPFactory(protocol.ClientFactory):
         self.pi = pi                        # the protocol interpreter that is using this factory
         self.peerHost = peerHost            # the from FTP.transport.peerHost()
         self.deferred = defer.Deferred()    # deferred will fire when instance is connected
-        self.deferred.addBoth(lambda ign: (delattr(self, 'deferred'), ign)[1])
         self.delayedCall = None
+        if reactor is None:
+            from twisted.internet import reactor
+        self._reactor = reactor
+
 
     def buildProtocol(self, addr):
         log.msg('DTPFactory.buildProtocol', debug=True)
+
+        if self._state is not self._IN_PROGRESS:
+            return None
+        self._state = self._FINISHED
+
         self.cancelTimeout()
-        if self.pi.dtpInstance:   # only create one instance
-            return
         p = DTP()
         p.factory = self
         p.pi = self.pi
         self.pi.dtpInstance = p
         return p
 
+
     def stopFactory(self):
         log.msg('dtpFactory.stopFactory', debug=True)
         self.cancelTimeout()
 
+
     def timeoutFactory(self):
         log.msg('timed out waiting for DTP connection')
-        if self.deferred:
-            d, self.deferred = self.deferred, None
+        if self._state is not self._IN_PROGRESS:
+            return
+        self._state = self._FAILED
 
-            # TODO: LEFT OFF HERE!
+        d = self.deferred
+        self.deferred = None
+        d.errback(
+            PortConnectionError(defer.TimeoutError("DTPFactory timeout")))
 
-            d.addErrback(debugDeferred, 'timeoutFactory firing errback')
-            d.errback(defer.TimeoutError())
-        self.stopFactory()
 
     def cancelTimeout(self):
-        if not self.delayedCall.called and not self.delayedCall.cancelled:
+        if self.delayedCall is not None and self.delayedCall.active():
             log.msg('cancelling DTP timeout', debug=True)
             self.delayedCall.cancel()
-            assert self.delayedCall.cancelled
-            log.msg('timeout has been cancelled', debug=True)
+
 
     def setTimeout(self, seconds):
         log.msg('DTPFactory.setTimeout set to %s seconds' % seconds)
-        self.delayedCall = reactor.callLater(seconds, self.timeoutFactory)
+        self.delayedCall = self._reactor.callLater(seconds, self.timeoutFactory)
+
 
     def clientConnectionFailed(self, connector, reason):
-        self.deferred.errback(PortConnectionError(reason))
+        if self._state is not self._IN_PROGRESS:
+            return
+        self._state = self._FAILED
+        d = self.deferred
+        self.deferred = None
+        d.errback(PortConnectionError(reason))
+
 
 # -- FTP-PI (Protocol Interpreter) --
 
@@ -898,9 +926,24 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
 
 
     def ftp_NLST(self, path):
-        # XXX: why is this check different to ftp_RETR/ftp_STOR?
+        """
+        This command causes a directory listing to be sent from the server to
+        the client. The pathname should specify a directory or other
+        system-specific file group descriptor. An empty path implies the current
+        working directory. If the path is non-existent, send nothing. If the
+        path is to a file, send only the file name.
+
+        @type path: C{str}
+        @param path: The path for which a directory listing should be returned.
+
+        @rtype: L{Deferred}
+        @return: a L{Deferred} which will be fired when the listing request
+            is finished.
+        """
+        # XXX: why is this check different from ftp_RETR/ftp_STOR? See #4180
         if self.dtpInstance is None or not self.dtpInstance.isConnected:
-            return defer.fail(BadCmdSequenceError('must send PORT or PASV before RETR'))
+            return defer.fail(
+                BadCmdSequenceError('must send PORT or PASV before RETR'))
 
         try:
             segments = toSegments(self.workingDirectory, path)
@@ -908,6 +951,18 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
             return defer.fail(FileNotFoundError(path))
 
         def cbList(results):
+            """
+            Send, line by line, each file in the directory listing, and then
+            close the connection.
+
+            @type results: A C{list} of C{tuple}. The first element of each
+                C{tuple} is a C{str} and the second element is a C{list}.
+            @param results: The names of the files in the directory.
+
+            @rtype: C{tuple}
+            @return: A C{tuple} containing the status code for a successful
+                transfer.
+            """
             self.reply(DATA_CNX_ALREADY_OPEN_START_XFR)
             for (name, ignored) in results:
                 self.dtpInstance.sendLine(name)
@@ -922,8 +977,24 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
             self.dtpInstance.transport.loseConnection()
             return (TXFR_COMPLETE_OK,)
 
-        # XXX Maybe this globbing is incomplete, but who cares.
-        # Stupid people probably.
+        def listErr(results):
+            """
+            RFC 959 specifies that an NLST request may only return directory
+            listings. Thus, send nothing and just close the connection.
+
+            @type results: L{Failure}
+            @param results: The L{Failure} wrapping a L{FileNotFoundError} that
+                occurred while trying to list the contents of a nonexistent
+                directory.
+
+            @rtype: C{tuple}
+            @returns: A C{tuple} containing the status code for a successful
+                transfer.
+            """
+            self.dtpInstance.transport.loseConnection()
+            return (TXFR_COMPLETE_OK,)
+
+        # XXX This globbing may be incomplete: see #4181
         if segments and (
             '*' in segments[-1] or '?' in segments[-1] or
             ('[' in segments[-1] and ']' in segments[-1])):
@@ -932,6 +1003,8 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         else:
             d = self.shell.list(segments)
             d.addCallback(cbList)
+            # self.shell.list will generate an error if the path is invalid
+            d.addErrback(listErr)
         return d
 
 
@@ -1246,7 +1319,7 @@ class FTPFactory(policies.LimitTotalConnectionsFactory):
 
     def __init__(self, portal=None, userAnonymous='anonymous'):
         self.portal = portal
-        self.userAnonymous = 'anonymous'
+        self.userAnonymous = userAnonymous
         self.instances = []
 
     def buildProtocol(self, addr):
@@ -1254,7 +1327,7 @@ class FTPFactory(policies.LimitTotalConnectionsFactory):
         if p is not None:
             p.wrappedProtocol.portal = self.portal
             p.wrappedProtocol.timeOut = self.timeOut
-            p.passivePortRange = self.passivePortRange
+            p.wrappedProtocol.passivePortRange = self.passivePortRange
         return p
 
     def stopFactory(self):
@@ -2177,9 +2250,21 @@ class _PassiveConnectionFactory(protocol.ClientFactory):
 
 class FTPClient(FTPClientBasic):
     """
-    A Twisted FTP Client
+    L{FTPClient} is a client implementation of the FTP protocol which
+    exposes FTP commands as methods which return L{Deferred}s.
 
-    Supports active and passive transfers.
+    Each command method returns a L{Deferred} which is called back when a
+    successful response code (2xx or 3xx) is received from the server or
+    which is error backed if an error response code (4xx or 5xx) is received
+    from the server or if a protocol violation occurs.  If an error response
+    code is received, the L{Deferred} fires with a L{Failure} wrapping a
+    L{CommandFailed} instance.  The L{CommandFailed} instance is created
+    with a list of the response lines received from the server.
+
+    See U{RFC 959<http://www.ietf.org/rfc/rfc959.txt>} for error code
+    definitions.
+
+    Both active and passive transfers are supported.
 
     @ivar passive: See description in __init__.
     """
@@ -2197,7 +2282,7 @@ class FTPClient(FTPClientBasic):
         @param password: FTP password
         @param passive: flag that controls if I use active or passive data
             connections.  You can also change this after construction by
-            assigning to self.passive.
+            assigning to C{self.passive}.
         """
         FTPClientBasic.__init__(self)
         self.queueLogin(username, password)
@@ -2216,11 +2301,11 @@ class FTPClient(FTPClientBasic):
         Retrieves a file or listing generated by the given command,
         feeding it to the given protocol.
 
-        @param command: list of strings of FTP commands to execute then receive
-            the results of (e.g. LIST, RETR)
-        @param protocol: A L{Protocol} *instance* e.g. an
+        @param commands: list of strings of FTP commands to execute then receive
+            the results of (e.g. C{LIST}, C{RETR})
+        @param protocol: A L{Protocol} B{instance} e.g. an
             L{FTPFileListProtocol}, or something that can be adapted to one.
-            Typically this will be an L{IConsumer} implemenation.
+            Typically this will be an L{IConsumer} implementation.
 
         @return: L{Deferred}.
         """
@@ -2404,6 +2489,56 @@ class FTPClient(FTPClientBasic):
 
     stor = storeFile
 
+
+    def rename(self, pathFrom, pathTo):
+        """
+        Rename a file.
+
+        This method issues the I{RNFR}/I{RNTO} command sequence to rename
+        C{pathFrom} to C{pathTo}.
+
+        @param: pathFrom: the absolute path to the file to be renamed
+        @type pathFrom: C{str}
+
+        @param: pathTo: the absolute path to rename the file to.
+        @type pathTo: C{str}
+
+        @return: A L{Deferred} which fires when the rename operation has
+            succeeded or failed.  If it succeeds, the L{Deferred} is called
+            back with a two-tuple of lists.  The first list contains the
+            responses to the I{RNFR} command.  The second list contains the
+            responses to the I{RNTO} command.  If either I{RNFR} or I{RNTO}
+            fails, the L{Deferred} is errbacked with L{CommandFailed} or
+            L{BadResponse}.
+        @rtype: L{Deferred}
+
+        @since: 8.2
+        """
+        renameFrom = self.queueStringCommand('RNFR ' + self.escapePath(pathFrom))
+        renameTo = self.queueStringCommand('RNTO ' + self.escapePath(pathTo))
+
+        fromResponse = []
+
+        # Use a separate Deferred for the ultimate result so that Deferred
+        # chaining can't interfere with its result.
+        result = defer.Deferred()
+        # Bundle up all the responses
+        result.addCallback(lambda toResponse: (fromResponse, toResponse))
+
+        def ebFrom(failure):
+            # Make sure the RNTO doesn't run if the RNFR failed.
+            self.popCommandQueue()
+            result.errback(failure)
+
+        # Save the RNFR response to pass to the result Deferred later
+        renameFrom.addCallbacks(fromResponse.extend, ebFrom)
+
+        # Hook up the RNTO to the result Deferred as well
+        renameTo.chainDeferred(result)
+
+        return result
+
+
     def list(self, path, protocol):
         """
         Retrieve a file listing into the given protocol instance.
@@ -2421,6 +2556,7 @@ class FTPClient(FTPClientBasic):
             path = ''
         return self.receiveFromConnection(['LIST ' + self.escapePath(path)], protocol)
 
+
     def nlst(self, path, protocol):
         """
         Retrieve a short file listing into the given protocol instance.
@@ -2436,6 +2572,7 @@ class FTPClient(FTPClientBasic):
             path = ''
         return self.receiveFromConnection(['NLST ' + self.escapePath(path)], protocol)
 
+
     def cwd(self, path):
         """
         Issues the CWD (Change Working Directory) command. It's also
@@ -2445,6 +2582,7 @@ class FTPClient(FTPClientBasic):
         """
         return self.queueStringCommand('CWD ' + self.escapePath(path))
 
+
     def changeDirectory(self, path):
         """
         Change the directory on the server and parse the result to determine
@@ -2454,18 +2592,62 @@ class FTPClient(FTPClientBasic):
         @param path: The path to which to change.
 
         @return: a L{Deferred} which will be called back when the directory
-            change has succeeded or and errbacked if an error occurrs.
+            change has succeeded or errbacked if an error occurrs.
         """
-        def cbParse(result):
-            try:
-                # The only valid code is 250
-                if int(result[0].split(' ', 1)[0]) == 250:
-                    return True
-                else:
-                    raise ValueError
-            except (IndexError, ValueError), e:
+        warnings.warn(
+            "FTPClient.changeDirectory is deprecated in Twisted 8.2 and "
+            "newer.  Use FTPClient.cwd instead.",
+            category=DeprecationWarning,
+            stacklevel=2)
+
+        def cbResult(result):
+            if result[-1][:3] != '250':
                 return failure.Failure(CommandFailed(result))
-        return self.cwd(path).addCallback(cbParse)
+            return True
+        return self.cwd(path).addCallback(cbResult)
+
+
+    def makeDirectory(self, path):
+        """
+        Make a directory
+
+        This method issues the MKD command.
+
+        @param path: The path to the directory to create.
+        @type path: C{str}
+
+        @return: A L{Deferred} which fires when the server responds.  If the
+            directory is created, the L{Deferred} is called back with the
+            server response.  If the server response indicates the directory
+            was not created, the L{Deferred} is errbacked with a L{Failure}
+            wrapping L{CommandFailed} or L{BadResponse}.
+        @rtype: L{Deferred}
+
+        @since: 8.2
+        """
+        return self.queueStringCommand('MKD ' + self.escapePath(path))
+
+
+    def removeFile(self, path):
+        """
+        Delete a file on the server.
+
+        L{removeFile} issues a I{DELE} command to the server to remove the
+        indicated file.  Note that this command cannot remove a directory.
+
+        @param path: The path to the file to delete. May be relative to the
+            current dir.
+        @type path: C{str}
+
+        @return: A L{Deferred} which fires when the server responds.  On error,
+            it is errbacked with either L{CommandFailed} or L{BadResponse}.  On
+            success, it is called back with a list of response lines.
+        @rtype: L{Deferred}
+
+        @since: 8.2
+        """
+        return self.queueStringCommand('DELE ' + self.escapePath(path))
+
 
     def cdup(self):
         """
@@ -2475,6 +2657,7 @@ class FTPClient(FTPClientBasic):
         """
         return self.queueStringCommand('CDUP')
 
+
     def pwd(self):
         """
         Issues the PWD (Print Working Directory) command.
@@ -2509,13 +2692,19 @@ class FTPClient(FTPClientBasic):
             return path
         return self.pwd().addCallback(cbParse)
 
+
     def quit(self):
         """
-        Issues the QUIT command.
+        Issues the I{QUIT} command.
+
+        @return: A L{Deferred} that fires when the server acknowledges the
+            I{QUIT} command.  The transport should not be disconnected until
+            this L{Deferred} fires.
         """
         return self.queueStringCommand('QUIT')
 
 
+
 class FTPFileListProtocol(basic.LineReceiver):
     """Parser for standard FTP file listings
 
