@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_memcache -*-
-# Copyright (c) 2007 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -130,8 +130,12 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
 
     @ivar _bufferLength: the total amount of bytes in C{_getBuffer}.
     @type _bufferLength: C{int}
+
+    @ivar _disconnected: indicate if the connectionLost has been called or not.
+    @type _disconnected: C{bool}
     """
     MAX_KEY_LENGTH = 250
+    _disconnected = False
 
     def __init__(self, timeOut=60):
         """
@@ -148,21 +152,38 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         self.persistentTimeOut = self.timeOut = timeOut
 
 
+    def _cancelCommands(self, reason):
+        """
+        Cancel all the outstanding commands, making them fail with C{reason}.
+        """
+        while self._current:
+            cmd = self._current.popleft()
+            cmd.fail(reason)
+
+
     def timeoutConnection(self):
         """
         Close the connection in case of timeout.
         """
-        for cmd in self._current:
-            cmd.fail(TimeoutError("Connection timeout"))
+        self._cancelCommands(TimeoutError("Connection timeout"))
         self.transport.loseConnection()
 
 
+    def connectionLost(self, reason):
+        """
+        Cause any outstanding commands to fail.
+        """
+        self._disconnected = True
+        self._cancelCommands(reason)
+        LineReceiver.connectionLost(self, reason)
+
+
     def sendLine(self, line):
         """
         Override sendLine to add a timeout to response.
         """
         if not self._current:
-           self.setTimeout(self.persistentTimeOut)
+            self.setTimeout(self.persistentTimeOut)
         LineReceiver.sendLine(self, line)
 
 
@@ -182,7 +203,11 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
             self._getBuffer = None
             self._bufferLength = None
             cmd = self._current[0]
-            cmd.value = val
+            if cmd.multiple:
+                flags, cas = cmd.values[cmd.currentKey]
+                cmd.values[cmd.currentKey] = (flags, cas, val)
+            else:
+                cmd.value = val
             self.setLineMode(rem)
 
 
@@ -207,9 +232,17 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         cmd = self._current.popleft()
         if cmd.command == "get":
-            cmd.success((cmd.flags, cmd.value))
+            if cmd.multiple:
+                values = dict([(key, val[::2]) for key, val in
+                               cmd.values.iteritems()])
+                cmd.success(values)
+            else:
+                cmd.success((cmd.flags, cmd.value))
         elif cmd.command == "gets":
-            cmd.success((cmd.flags, cmd.cas, cmd.value))
+            if cmd.multiple:
+                cmd.success(cmd.values)
+            else:
+                cmd.success((cmd.flags, cmd.cas, cmd.value))
         elif cmd.command == "stats":
             cmd.success(cmd.values)
 
@@ -234,11 +267,16 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         self._lenExpected = int(length)
         self._getBuffer = []
         self._bufferLength = 0
-        if cmd.key != key:
-            raise RuntimeError("Unexpected commands answer.")
-        cmd.flags = int(flags)
-        cmd.length = self._lenExpected
-        cmd.cas = cas
+        if cmd.multiple:
+            if key not in cmd.keys:
+                raise RuntimeError("Unexpected commands answer.")
+            cmd.currentKey = key
+            cmd.values[key] = [int(flags), cas]
+        else:
+            if cmd.key != key:
+                raise RuntimeError("Unexpected commands answer.")
+            cmd.flags = int(flags)
+            cmd.cas = cas
         self.setRawMode()
 
 
@@ -377,6 +415,8 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         Internal wrapper for incr/decr.
         """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
         if not isinstance(key, str):
             return fail(ClientError(
                 "Invalid type for key: %s, expecting a string" % (type(key),)))
@@ -494,6 +534,8 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         Internal wrapper for setting values.
         """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
         if not isinstance(key, str):
             return fail(ClientError(
                 "Invalid type for key: %s, expecting a string" % (type(key),)))
@@ -570,34 +612,88 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
 
         @return: A deferred that will fire with the tuple (flags, value) if
             C{withIdentifier} is C{False}, or (flags, cas identifier, value)
-            if C{True}.
+            if C{True}.  If the server indicates there is no value
+            associated with C{key}, the returned value will be C{None} and
+            the returned flags will be C{0}.
         @rtype: L{Deferred}
         """
-        if not isinstance(key, str):
-            return fail(ClientError(
-                "Invalid type for key: %s, expecting a string" % (type(key),)))
-        if len(key) > self.MAX_KEY_LENGTH:
-            return fail(ClientError("Key too long"))
+        return self._get([key], withIdentifier, False)
+
+
+    def getMultiple(self, keys, withIdentifier=False):
+        """
+        Get the given list of C{keys}.  If C{withIdentifier} is set to C{True},
+        the command issued is a C{gets}, that will return the identifiers
+        associated with each values. This identifier has to be used when
+        issuing C{checkAndSet} update later, using the corresponding method.
+
+        @param keys: The keys to retrieve.
+        @type keys: C{list} of C{str}
+
+        @param withIdentifier: If set to C{True}, retrieve the identifiers
+            along with the values and the flags.
+        @type withIdentifier: C{bool}
+
+        @return: A deferred that will fire with a dictionary with the elements
+            of C{keys} as keys and the tuples (flags, value) as values if
+            C{withIdentifier} is C{False}, or (flags, cas identifier, value) if
+            C{True}.  If the server indicates there is no value associated with
+            C{key}, the returned values will be C{None} and the returned flags
+            will be C{0}.
+        @rtype: L{Deferred}
+
+        @since: 9.0
+        """
+        return self._get(keys, withIdentifier, True)
+
+    def _get(self, keys, withIdentifier, multiple):
+        """
+        Helper method for C{get} and C{getMultiple}.
+        """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
+        for key in keys:
+            if not isinstance(key, str):
+                return fail(ClientError(
+                    "Invalid type for key: %s, expecting a string" % (type(key),)))
+            if len(key) > self.MAX_KEY_LENGTH:
+                return fail(ClientError("Key too long"))
         if withIdentifier:
             cmd = "gets"
         else:
             cmd = "get"
-        fullcmd = "%s %s" % (cmd, key)
+        fullcmd = "%s %s" % (cmd, " ".join(keys))
         self.sendLine(fullcmd)
-        cmdObj = Command(cmd, key=key, value=None, flags=0, cas="")
+        if multiple:
+            values = dict([(key, (0, "", None)) for key in keys])
+            cmdObj = Command(cmd, keys=keys, values=values, multiple=True)
+        else:
+            cmdObj = Command(cmd, key=keys[0], value=None, flags=0, cas="",
+                             multiple=False)
         self._current.append(cmdObj)
         return cmdObj._deferred
 
-
-    def stats(self):
+    def stats(self, arg=None):
         """
         Get some stats from the server. It will be available as a dict.
 
+        @param arg: An optional additional string which will be sent along
+            with the I{stats} command.  The interpretation of this value by
+            the server is left undefined by the memcache protocol
+            specification.
+        @type arg: L{NoneType} or L{str}
+
         @return: a deferred that will fire with a C{dict} of the available
             statistics.
         @rtype: L{Deferred}
         """
-        self.sendLine("stats")
+        if arg:
+            cmd = "stats " + arg
+        else:
+            cmd = "stats"
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
+        self.sendLine(cmd)
         cmdObj = Command("stats", values={})
         self._current.append(cmdObj)
         return cmdObj._deferred
@@ -611,6 +707,8 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
             version.
         @rtype: L{Deferred}
         """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
         self.sendLine("version")
         cmdObj = Command("version")
         self._current.append(cmdObj)
@@ -628,6 +726,8 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
             was successfully deleted, or C{False} if not.
         @rtype: L{Deferred}
         """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
         if not isinstance(key, str):
             return fail(ClientError(
                 "Invalid type for key: %s, expecting a string" % (type(key),)))
@@ -645,6 +745,8 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
             operation has succeeded.
         @rtype: L{Deferred}
         """
+        if self._disconnected:
+            return fail(RuntimeError("not connected"))
         self.sendLine("flush_all")
         cmdObj = Command("flush_all")
         self._current.append(cmdObj)
@@ -654,4 +756,3 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
 
 __all__ = ["MemCacheProtocol", "DEFAULT_PORT", "NoSuchCommand", "ClientError",
            "ServerError"]
-
