@@ -1,8 +1,9 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
+DEPRECATED since Twisted 8.0.
+
 Utility functions for reporting bytecode frequencies to Skip Montanaro's
 stat collector.
 
@@ -10,10 +11,17 @@ This module requires a version of Python build with DYNAMIC_EXCUTION_PROFILE,
 and optionally DXPAIRS, defined to be useful.
 """
 
-import sys, types, xmlrpclib
+import sys, types, xmlrpclib, warnings
+
+
+warnings.warn("twisted.python.dxprofile is deprecated since Twisted 8.0.",
+              category=DeprecationWarning)
+
 
 def rle(iterable):
-    """Run length encode a list"""
+    """
+    Run length encode a list.
+    """
     iterable = iter(iterable)
     runlen = 1
     result = []
@@ -37,6 +45,7 @@ def rle(iterable):
     return result
 
 
+
 def report(email, appname):
     """
     Send an RLE encoded version of sys.getdxp() off to our Top Men (tm)
