@@ -9,7 +9,6 @@ Object-oriented filesystem path representation.
 import os
 import errno
 import random
-import sha
 import base64
 
 from os.path import isabs, exists, normpath, abspath, splitext
@@ -25,6 +24,7 @@ from stat import S_ISREG, S_ISDIR
 # modified for inclusion in the standard library.  --glyph
 
 from twisted.python.runtime import platform
+from twisted.python.hashlib import sha1
 
 from twisted.python.win32 import ERROR_FILE_NOT_FOUND, ERROR_PATH_NOT_FOUND
 from twisted.python.win32 import ERROR_INVALID_NAME, ERROR_DIRECTORY
@@ -120,7 +120,7 @@ def _secureEnoughString():
     """
     Create a pseudorandom, 16-character string for use in secure filenames.
     """
-    return armor(sha.new(randomBytes(64)).digest())[:16]
+    return armor(sha1(randomBytes(64)).digest())[:16]
 
 
 
@@ -133,6 +133,21 @@ class _PathHelper:
     def getContent(self):
         return self.open().read()
 
+
+    def parents(self):
+        """
+        @return: an iterator of all the ancestors of this path, from the most
+        recent (its immediate parent) to the root of its filesystem.
+        """
+        path = self
+        parent = path.parent()
+        # root.parent() == root, so this means "are we the root"
+        while path != parent:
+            yield parent
+            path = parent
+            parent = parent.parent()
+
+
     def children(self):
         """
         List the chilren of this path object.
@@ -206,13 +221,19 @@ class _PathHelper:
         @return: a generator yielding FilePath-like objects.
         """
         yield self
-        if self.isdir() and (descend is None or descend(self)):
+        if self.isdir():
             for c in self.children():
-                for subc in c.walk(descend):
-                    if os.path.realpath(self.path).startswith(
-                        os.path.realpath(subc.path)):
-                        raise LinkError("Cycle in file graph.")
-                    yield subc
+                # we should first see if it's what we want, then we
+                # can walk through the directory
+                if (descend is None or descend(c)):
+                    for subc in c.walk(descend):
+                        if os.path.realpath(self.path).startswith(
+                            os.path.realpath(subc.path)):
+                            raise LinkError("Cycle in file graph.")
+                        yield subc
+                else:
+                    yield c
+
 
     def sibling(self, path):
         return self.parent().child(path)
