@@ -1,33 +1,23 @@
-# Copyright (c) 2007-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Tests for L{twisted.python.release} and L{twisted.python._release}.
+
+All of these tests are skipped on platforms other than Linux, as the release is
+only ever performed on Linux.
 """
 
+
 import warnings
 import operator
 import os, sys, signal
 from StringIO import StringIO
 import tarfile
-
-try:
-    from popen2 import Popen4
-except:
-    popen4Available = False
-else:
-    popen4Available = True
-    del Popen4
+from xml.dom import minidom as dom
 
 from datetime import date
 
-try:
-    import pydoctor.driver
-    # it might not be installed, or it might use syntax not available in
-    # this version of Python.
-except (ImportError, SyntaxError):
-    pydoctor = None
-
 from twisted.trial.unittest import TestCase
 
 from twisted.python.compat import set
@@ -46,14 +36,58 @@ from twisted.python._release import VERSION_OFFSET, DocBuilder, ManBuilder
 from twisted.python._release import NoDocumentsFound, filePathDelta
 from twisted.python._release import CommandFailed, BookBuilder
 from twisted.python._release import DistributionBuilder, APIBuilder
+from twisted.python._release import BuildAPIDocsScript
 from twisted.python._release import buildAllTarballs, runCommand
 from twisted.python._release import UncleanWorkingDirectory, NotWorkingDirectory
 from twisted.python._release import ChangeVersionsScript, BuildTarballsScript
+from twisted.python._release import NewsBuilder
 
+if sys.platform != 'linux2':
+    skip = "Release toolchain only supported on Linux."
+else:
+    skip = None
+
+
+# Check a bunch of dependencies to skip tests if necessary.
 try:
     from twisted.lore.scripts import lore
 except ImportError:
-    lore = None
+    loreSkip = "Lore is not present."
+else:
+    loreSkip = skip
+
+
+try:
+    from popen2 import Popen4
+except ImportError:
+    popen4Skip = "popen2.Popen4 is not available."
+else:
+    popen4Skip = skip
+
+
+try:
+    import pydoctor.driver
+    # it might not be installed, or it might use syntax not available in
+    # this version of Python.
+except (ImportError, SyntaxError):
+    pydoctorSkip = "Pydoctor is not present."
+else:
+    if getattr(pydoctor, "version_info", (0,)) < (0, 1):
+        pydoctorSkip = "Pydoctor is too old."
+    else:
+        pydoctorSkip = skip
+
+
+if which("latex") and which("dvips") and which("ps2pdf13"):
+    latexSkip = skip
+else:
+    latexSkip = "LaTeX is not available."
+
+
+if which("svn") and which("svnadmin"):
+    svnSkip = skip
+else:
+    svnSkip = "svn or svnadmin is not present."
 
 
 def genVersion(*args, **kwargs):
@@ -62,15 +96,9 @@ def genVersion(*args, **kwargs):
 
     @param args: Arguments to pass to L{Version}.
     @param kwargs: Keyword arguments to pass to L{Version}.
-    @type output: C{bool}
-    @param output: If passed as a keyword argument with a True value, replace
-        line endings with the platform's convention.
     """
-    output = kwargs.pop("output", False)
-    data = generateVersionFileData(Version(*args, **kwargs))
-    if output:
-        data = data.replace("\n", os.linesep)
-    return data
+    return generateVersionFileData(Version(*args, **kwargs))
+
 
 
 class StructureAssertingMixin(object):
@@ -83,13 +111,19 @@ class StructureAssertingMixin(object):
         Create a set of directories and files given a dict defining their
         structure.
 
-        @param root: The directory in which to create the structure.
+        @param root: The directory in which to create the structure.  It must
+            already exist.
         @type root: L{FilePath}
 
         @param dirDict: The dict defining the structure. Keys should be strings
             naming files, values should be strings describing file contents OR
-            dicts describing subdirectories. For example: C{{"foofile":
-            "foocontents", "bardir": {"barfile": "barcontents"}}}
+            dicts describing subdirectories.  All files are written in binary
+            mode.  Any string values are assumed to describe text files and
+            will have their newlines replaced with the platform-native newline
+            convention.  For example::
+
+                {"foofile": "foocontents",
+                 "bardir": {"barfile": "bar\ncontents"}}
         @type dirDict: C{dict}
         """
         for x in dirDict:
@@ -98,7 +132,7 @@ class StructureAssertingMixin(object):
                 child.createDirectory()
                 self.createStructure(child, dirDict[x])
             else:
-                child.setContent(dirDict[x])
+                child.setContent(dirDict[x].replace('\n', os.linesep))
 
     def assertStructure(self, root, dirDict):
         """
@@ -119,7 +153,7 @@ class StructureAssertingMixin(object):
                                 % (child.path,))
                 self.assertStructure(child, dirDict[x])
             else:
-                a = child.getContent()
+                a = child.getContent().replace(os.linesep, '\n')
                 self.assertEquals(a, dirDict[x], child.path)
             children.remove(x)
         if children:
@@ -216,11 +250,13 @@ class ChangeVersionTest(TestCase, StructureAssertingMixin):
     def test_changeAllProjectVersions(self):
         """
         L{changeAllProjectVersions} changes all version numbers in _version.py
-        and README files for all projects.
+        and README files for all projects as well as in the the top-level
+        README file.
         """
         root = FilePath(self.mktemp())
         root.createDirectory()
         structure = {
+            "README": "Hi this is 1.0.0.",
             "twisted": {
                 "topfiles": {
                     "README": "Hi this is 1.0.0"},
@@ -234,16 +270,16 @@ class ChangeVersionTest(TestCase, StructureAssertingMixin):
         self.createStructure(root, structure)
         changeAllProjectVersions(root, Version("lol", 1, 0, 2))
         outStructure = {
+            "README": "Hi this is 1.0.2.",
             "twisted": {
                 "topfiles": {
                     "README": "Hi this is 1.0.2"},
                 "_version.py":
-                    genVersion("twisted", 1, 0, 2, output=True),
+                    genVersion("twisted", 1, 0, 2),
                 "web": {
                     "topfiles": {
                         "README": "Hi this is 1.0.2"},
-                    "_version.py": genVersion("twisted.web", 1, 0, 2,
-                                              output=True)
+                    "_version.py": genVersion("twisted.web", 1, 0, 2),
                     }}}
         self.assertStructure(root, outStructure)
 
@@ -484,6 +520,15 @@ class BuilderTestsMixin(object):
         self.docCounter = 0
 
 
+    def assertXMLEqual(self, first, second):
+        """
+        Verify that two strings represent the same XML document.
+        """
+        self.assertEqual(
+            dom.parseString(first).toxml(),
+            dom.parseString(second).toxml())
+
+
     def getArbitraryOutput(self, version, counter, prefix="", apiBaseURL="%s"):
         """
         Get the correct HTML output for the arbitrary input returned by
@@ -494,16 +539,20 @@ class BuilderTestsMixin(object):
         @param counter: A counter to include in the output.
         @type counter: C{int}
         """
-        document = ('<?xml version="1.0"?><html><head>'
-                    '<title>Yo:Hi! Title: %(count)s</title></head>'
-                    '<body><div class="content">Hi! %(count)s'
-                    '<div class="API"><a href="%(foobarLink)s" title="foobar">'
-                    'foobar</a></div></div><a href="%(prefix)sindex.html">'
-                    'Index</a><span class="version">Version: %(version)s'
-                    '</span></body></html>')
-        return document % {"count": counter, "prefix": prefix,
-                           "version": version,
-                           "foobarLink": apiBaseURL % ("foobar",)}
+        document = """\
+<?xml version="1.0"?><html>
+    <head><title>Yo:Hi! Title: %(count)d</title></head>
+    <body>
+    <div class="content">Hi! %(count)d<div class="API"><a href="%(foobarLink)s" title="foobar">foobar</a></div></div>
+    <a href="%(prefix)sindex.html">Index</a>
+    <span class="version">Version: %(version)s</span>
+    </body>
+    </html>"""
+        # Try to normalize irrelevant whitespace.
+        return dom.parseString(
+            document % {"count": counter, "prefix": prefix,
+                        "version": version,
+                        "foobarLink": apiBaseURL % ("foobar",)}).toxml('utf-8')
 
 
     def getArbitraryLoreInput(self, counter):
@@ -564,15 +613,34 @@ with this."""
         Get an arbitrary lore input document which represents man-to-lore
         output based on the man page returned from L{getArbitraryManInput}
         """
-        return ("<html><head>\n<title>MANHOLE.1</title>"
-            "</head>\n<body>\n\n<h1>MANHOLE.1</h1>\n\n<h2>NAME</h2>\n\n"
-            "<p>manhole - Connect to a Twisted Manhole service\n</p>\n\n"
-            "<h2>SYNOPSIS</h2>\n\n<p><strong>manhole</strong> </p>\n\n"
-            "<h2>DESCRIPTION</h2>\n\n<p>manhole is a GTK interface to Twisted "
-            "Manhole services. You can execute python\ncode as if at an "
-            "interactive Python console inside a running Twisted process\nwith"
-            " this.</p>\n\n</body>\n</html>\n")
+        return """\
+<?xml version="1.0"?>
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html><head>
+<title>MANHOLE.1</title></head>
+<body>
 
+<h1>MANHOLE.1</h1>
+
+<h2>NAME</h2>
+
+<p>manhole - Connect to a Twisted Manhole service
+</p>
+
+<h2>SYNOPSIS</h2>
+
+<p><strong>manhole</strong> </p>
+
+<h2>DESCRIPTION</h2>
+
+<p>manhole is a GTK interface to Twisted Manhole services. You can execute python
+code as if at an interactive Python console inside a running Twisted process
+with this.</p>
+
+</body>
+</html>
+"""
 
     def getArbitraryManHTMLOutput(self, version, prefix=""):
         """
@@ -585,18 +653,36 @@ with this."""
         @param prefix: The prefix to include in the link to the index.
         @type prefix: C{str}
         """
-        return ('<?xml version="1.0"?><html><head>'
-            '<title>Yo:MANHOLE.1</title></head><body><div class="content">'
-            '<span></span><h2>NAME<a name="auto0"></a></h2><p>manhole - '
-            'Connect to a Twisted Manhole service\n</p><h2>SYNOPSIS<a '
-            'name="auto1"></a></h2><p><strong>manhole</strong></p><h2>'
-            'DESCRIPTION<a name="auto2"></a></h2><p>manhole is a GTK '
-            'interface to Twisted Manhole services. You can execute '
-            'python\ncode as if at an interactive Python console inside a '
-            'running Twisted process\nwith this.</p></div><a '
-            'href="%sindex.html">Index</a><span class="version">Version: '
-            '%s</span></body></html>' % (prefix, version))
+        # Try to normalize the XML a little bit.
+        return dom.parseString("""\
+<?xml version="1.0" ?><html>
+    <head><title>Yo:MANHOLE.1</title></head>
+    <body>
+    <div class="content">
 
+<span/>
+
+<h2>NAME<a name="auto0"/></h2>
+
+<p>manhole - Connect to a Twisted Manhole service
+</p>
+
+<h2>SYNOPSIS<a name="auto1"/></h2>
+
+<p><strong>manhole</strong> </p>
+
+<h2>DESCRIPTION<a name="auto2"/></h2>
+
+<p>manhole is a GTK interface to Twisted Manhole services. You can execute python
+code as if at an interactive Python console inside a running Twisted process
+with this.</p>
+
+</div>
+    <a href="%(prefix)sindex.html">Index</a>
+    <span class="version">Version: %(version)s</span>
+    </body>
+    </html>""" % {
+            'prefix': prefix, 'version': version}).toxml("utf-8")
 
 
 
@@ -610,6 +696,7 @@ class DocBuilderTestCase(TestCase, BuilderTestsMixin):
     forever. Feel free to update the tests when Lore changes, but please be
     careful.
     """
+    skip = loreSkip
 
     def setUp(self):
         """
@@ -646,8 +733,8 @@ class DocBuilderTestCase(TestCase, BuilderTestsMixin):
                            self.templateFile)
         out1 = self.howtoDir.child('one.html')
         out2 = self.howtoDir.child('two.html')
-        self.assertEquals(out1.getContent(), output1)
-        self.assertEquals(out2.getContent(), output2)
+        self.assertXMLEqual(out1.getContent(), output1)
+        self.assertXMLEqual(out2.getContent(), output2)
 
 
     def test_noDocumentsFound(self):
@@ -781,8 +868,7 @@ class APIBuilderTestCase(TestCase):
     """
     Tests for L{APIBuilder}.
     """
-    if pydoctor is None or getattr(pydoctor, "version_info", (0,)) < (0, 1):
-        skip = "APIBuilder requires Pydoctor 0.1 or newer"
+    skip = pydoctorSkip
 
     def test_build(self):
         """
@@ -832,23 +918,96 @@ class APIBuilderTestCase(TestCase):
         self.assertIn(
             '<a href="%s/%s">View Source</a>' % (sourceURL, packageName),
             quuxPath.getContent())
-        self.assertIn(
-            '<a href="%s/%s">View Source</a>' % (sourceURL, packageName),
-            quuxPath.getContent())
         self.assertIn(
             '<a href="%s/%s/__init__.py#L1" class="functionSourceLink">' % (
                 sourceURL, packageName),
             quuxPath.getContent())
         self.assertIn(privateDocstring, quuxPath.getContent())
 
+        # There should also be a page for the foo function in quux.
+        self.assertTrue(quuxPath.sibling('quux.foo.html').exists())
+
         self.assertEqual(stdout.getvalue(), '')
 
 
+    def test_buildWithPolicy(self):
+        """
+        L{BuildAPIDocsScript.buildAPIDocs} builds the API docs with values
+        appropriate for the Twisted project.
+        """
+        stdout = StringIO()
+        self.patch(sys, 'stdout', stdout)
+        docstring = "text in docstring"
+
+        projectRoot = FilePath(self.mktemp())
+        packagePath = projectRoot.child("twisted")
+        packagePath.makedirs()
+        packagePath.child("__init__.py").setContent(
+            "def foo():\n"
+            "    '%s'\n" % (docstring,))
+        packagePath.child("_version.py").setContent(
+            genVersion("twisted", 1, 0, 0))
+        outputPath = FilePath(self.mktemp())
+
+        script = BuildAPIDocsScript()
+        script.buildAPIDocs(projectRoot, outputPath)
+
+        indexPath = outputPath.child("index.html")
+        self.assertTrue(
+            indexPath.exists(),
+            "API index %r did not exist." % (outputPath.path,))
+        self.assertIn(
+            '<a href="http://twistedmatrix.com/">Twisted</a>',
+            indexPath.getContent(),
+            "Project name/location not in file contents.")
+
+        twistedPath = outputPath.child("twisted.html")
+        self.assertTrue(
+            twistedPath.exists(),
+            "Package documentation file %r did not exist."
+            % (twistedPath.path,))
+        self.assertIn(
+            docstring, twistedPath.getContent(),
+            "Docstring not in package documentation file.")
+        #Here we check that it figured out the correct version based on the
+        #source code.
+        self.assertIn(
+            '<a href="http://twistedmatrix.com/trac/browser/tags/releases/'
+            'twisted-1.0.0/twisted">View Source</a>',
+            twistedPath.getContent())
+
+        self.assertEqual(stdout.getvalue(), '')
+
+
+    def test_apiBuilderScriptMainRequiresTwoArguments(self):
+        """
+        SystemExit is raised when the incorrect number of command line
+        arguments are passed to the API building script.
+        """
+        script = BuildAPIDocsScript()
+        self.assertRaises(SystemExit, script.main, [])
+        self.assertRaises(SystemExit, script.main, ["foo"])
+        self.assertRaises(SystemExit, script.main, ["foo", "bar", "baz"])
+
+
+    def test_apiBuilderScriptMain(self):
+        """
+        The API building script invokes the same code that
+        L{test_buildWithPolicy} tests.
+        """
+        script = BuildAPIDocsScript()
+        calls = []
+        script.buildAPIDocs = lambda a, b: calls.append((a, b))
+        script.main(["hello", "there"])
+        self.assertEquals(calls, [(FilePath("hello"), FilePath("there"))])
+
+
 
 class ManBuilderTestCase(TestCase, BuilderTestsMixin):
     """
     Tests for L{ManBuilder}.
     """
+    skip = loreSkip
 
     def setUp(self):
         """
@@ -902,17 +1061,37 @@ class ManBuilderTestCase(TestCase, BuilderTestsMixin):
         docBuilder.build("1.2.3", self.manDir, self.manDir,
                          templateFile)
         output = self.manDir.child('test1-man.html').getContent()
-        self.assertEquals(output, '<?xml version="1.0"?><html><head>'
-            '<title>Yo:MANHOLE.1</title></head><body><div class="content">'
-            '<span></span><h2>NAME<a name="auto0"></a></h2><p>manhole - '
-            'Connect to a Twisted Manhole service\n</p><h2>SYNOPSIS<a '
-            'name="auto1"></a></h2><p><strong>manhole</strong></p><h2>'
-            'DESCRIPTION<a name="auto2"></a></h2><p>manhole is a GTK '
-            'interface to Twisted Manhole services. You can execute '
-            'python\ncode as if at an interactive Python console inside a '
-            'running Twisted process\nwith this.</p></div><a '
-            'href="index.html">Index</a><span class="version">Version: '
-            '1.2.3</span></body></html>')
+
+        self.assertXMLEqual(
+            output,
+            """\
+<?xml version="1.0" ?><html>
+    <head><title>Yo:MANHOLE.1</title></head>
+    <body>
+    <div class="content">
+
+<span/>
+
+<h2>NAME<a name="auto0"/></h2>
+
+<p>manhole - Connect to a Twisted Manhole service
+</p>
+
+<h2>SYNOPSIS<a name="auto1"/></h2>
+
+<p><strong>manhole</strong> </p>
+
+<h2>DESCRIPTION<a name="auto2"/></h2>
+
+<p>manhole is a GTK interface to Twisted Manhole services. You can execute python
+code as if at an interactive Python console inside a running Twisted process
+with this.</p>
+
+</div>
+    <a href="index.html">Index</a>
+    <span class="version">Version: 1.2.3</span>
+    </body>
+    </html>""")
 
 
 
@@ -920,10 +1099,7 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
     """
     Tests for L{BookBuilder}.
     """
-    if not (which("latex") and which("dvips") and which("ps2pdf13")):
-        skip = "Book Builder tests require latex."
-    if not popen4Available:
-        skip = "Book Builder requires popen2.Popen4."
+    skip = latexSkip or popen4Skip or loreSkip
 
     def setUp(self):
         """
@@ -946,7 +1122,6 @@ class BookBuilderTests(TestCase, BuilderTestsMixin):
         returned by C{self.getArbitraryLoreInput(counter)}.
         """
         path = self.howtoDir.child("%d.xhtml" % (counter,)).path
-        path = path[len(os.getcwd()) + 1:]
         return (
             r'\section{Hi! Title: %(count)s\label{%(path)s}}'
             '\n'
@@ -1274,12 +1449,382 @@ class FilePathDeltaTest(TestCase):
                           ["..", "..", "baz", "quux"])
 
 
+    def test_filePathDeltaSimilarEndElements(self):
+        """
+        L{filePathDelta} doesn't take into account final elements when
+        comparing 2 paths, but stops at the first difference.
+        """
+        self.assertEquals(filePathDelta(FilePath("/foo/bar/bar/spam"),
+                                        FilePath("/foo/bar/baz/spam")),
+                          ["..", "..", "baz", "spam"])
+
+
+
+class NewsBuilderTests(TestCase, StructureAssertingMixin):
+    """
+    Tests for L{NewsBuilder}.
+    """
+    def setUp(self):
+        """
+        Create a fake project and stuff some basic structure and content into
+        it.
+        """
+        self.builder = NewsBuilder()
+        self.project = FilePath(self.mktemp())
+        self.project.createDirectory()
+        self.existingText = 'Here is stuff which was present previously.\n'
+        self.createStructure(self.project, {
+                'NEWS': self.existingText,
+                '5.feature': 'We now support the web.\n',
+                '12.feature': 'The widget is more robust.\n',
+                '15.feature': (
+                    'A very long feature which takes many words to '
+                    'describe with any accuracy was introduced so that '
+                    'the line wrapping behavior of the news generating '
+                    'code could be verified.\n'),
+                '16.feature': (
+                    'A simpler feature\ndescribed on multiple lines\n'
+                    'was added.\n'),
+                '23.bugfix': 'Broken stuff was fixed.\n',
+                '25.removal': 'Stupid stuff was deprecated.\n',
+                '30.misc': '',
+                '35.misc': ''})
+
+
+    def test_today(self):
+        """
+        L{NewsBuilder._today} returns today's date in YYYY-MM-DD form.
+        """
+        self.assertEquals(
+            self.builder._today(), date.today().strftime('%Y-%m-%d'))
+
+
+    def test_findFeatures(self):
+        """
+        When called with L{NewsBuilder._FEATURE}, L{NewsBuilder._findChanges}
+        returns a list of bugfix ticket numbers and descriptions as a list of
+        two-tuples.
+        """
+        features = self.builder._findChanges(
+            self.project, self.builder._FEATURE)
+        self.assertEquals(
+            features,
+            [(5, "We now support the web."),
+             (12, "The widget is more robust."),
+             (15,
+              "A very long feature which takes many words to describe with "
+              "any accuracy was introduced so that the line wrapping behavior "
+              "of the news generating code could be verified."),
+             (16, "A simpler feature described on multiple lines was added.")])
+
+
+    def test_findBugfixes(self):
+        """
+        When called with L{NewsBuilder._BUGFIX}, L{NewsBuilder._findChanges}
+        returns a list of bugfix ticket numbers and descriptions as a list of
+        two-tuples.
+        """
+        bugfixes = self.builder._findChanges(
+            self.project, self.builder._BUGFIX)
+        self.assertEquals(
+            bugfixes,
+            [(23, 'Broken stuff was fixed.')])
+
+
+    def test_findRemovals(self):
+        """
+        When called with L{NewsBuilder._REMOVAL}, L{NewsBuilder._findChanges}
+        returns a list of removal/deprecation ticket numbers and descriptions
+        as a list of two-tuples.
+        """
+        removals = self.builder._findChanges(
+            self.project, self.builder._REMOVAL)
+        self.assertEquals(
+            removals,
+            [(25, 'Stupid stuff was deprecated.')])
+
+
+    def test_findMiscellaneous(self):
+        """
+        When called with L{NewsBuilder._MISC}, L{NewsBuilder._findChanges}
+        returns a list of removal/deprecation ticket numbers and descriptions
+        as a list of two-tuples.
+        """
+        misc = self.builder._findChanges(
+            self.project, self.builder._MISC)
+        self.assertEquals(
+            misc,
+            [(30, ''),
+             (35, '')])
+
+
+    def test_writeHeader(self):
+        """
+        L{NewsBuilder._writeHeader} accepts a file-like object opened for
+        writing and a header string and writes out a news file header to it.
+        """
+        output = StringIO()
+        self.builder._writeHeader(output, "Super Awesometastic 32.16")
+        self.assertEquals(
+            output.getvalue(),
+            "Super Awesometastic 32.16\n"
+            "=========================\n"
+            "\n")
+
+
+    def test_writeSection(self):
+        """
+        L{NewsBuilder._writeSection} accepts a file-like object opened for
+        writing, a section name, and a list of ticket information (as returned
+        by L{NewsBuilder._findChanges}) and writes out a section header and all
+        of the given ticket information.
+        """
+        output = StringIO()
+        self.builder._writeSection(
+            output, "Features",
+            [(3, "Great stuff."),
+             (17, "Very long line which goes on and on and on, seemingly "
+              "without end until suddenly without warning it does end.")])
+        self.assertEquals(
+            output.getvalue(),
+            "Features\n"
+            "--------\n"
+            " - Great stuff. (#3)\n"
+            " - Very long line which goes on and on and on, seemingly without end\n"
+            "   until suddenly without warning it does end. (#17)\n"
+            "\n")
+
+
+    def test_writeMisc(self):
+        """
+        L{NewsBuilder._writeMisc} accepts a file-like object opened for
+        writing, a section name, and a list of ticket information (as returned
+        by L{NewsBuilder._findChanges} and writes out a section header and all
+        of the ticket numbers, but excludes any descriptions.
+        """
+        output = StringIO()
+        self.builder._writeMisc(
+            output, "Other",
+            [(x, "") for x in range(2, 50, 3)])
+        self.assertEquals(
+            output.getvalue(),
+            "Other\n"
+            "-----\n"
+            " - #2, #5, #8, #11, #14, #17, #20, #23, #26, #29, #32, #35, #38, #41,\n"
+            "   #44, #47\n"
+            "\n")
+
+
+    def test_build(self):
+        """
+        L{NewsBuilder.build} updates a NEWS file with new features based on the
+        I{<ticket>.feature} files found in the directory specified.
+        """
+        self.builder.build(
+            self.project, self.project.child('NEWS'),
+            "Super Awesometastic 32.16")
+
+        results = self.project.child('NEWS').getContent()
+        self.assertEquals(
+            results,
+            'Super Awesometastic 32.16\n'
+            '=========================\n'
+            '\n'
+            'Features\n'
+            '--------\n'
+            ' - We now support the web. (#5)\n'
+            ' - The widget is more robust. (#12)\n'
+            ' - A very long feature which takes many words to describe with any\n'
+            '   accuracy was introduced so that the line wrapping behavior of the\n'
+            '   news generating code could be verified. (#15)\n'
+            ' - A simpler feature described on multiple lines was added. (#16)\n'
+            '\n'
+            'Bugfixes\n'
+            '--------\n'
+            ' - Broken stuff was fixed. (#23)\n'
+            '\n'
+            'Deprecations and Removals\n'
+            '-------------------------\n'
+            ' - Stupid stuff was deprecated. (#25)\n'
+            '\n'
+            'Other\n'
+            '-----\n'
+            ' - #30, #35\n'
+            '\n' + self.existingText)
+
+
+    def test_preserveTicketHint(self):
+        """
+        If a I{NEWS} file begins with the two magic lines which point readers
+        at the issue tracker, those lines are kept at the top of the new file.
+        """
+        news = self.project.child('NEWS')
+        news.setContent(
+            'Ticket numbers in this file can be looked up by visiting\n'
+            'http://twistedmatrix.com/trac/ticket/<number>\n'
+            '\n'
+            'Blah blah other stuff.\n')
+
+        self.builder.build(self.project, news, "Super Awesometastic 32.16")
+
+        self.assertEquals(
+            news.getContent(),
+            'Ticket numbers in this file can be looked up by visiting\n'
+            'http://twistedmatrix.com/trac/ticket/<number>\n'
+            '\n'
+            'Super Awesometastic 32.16\n'
+            '=========================\n'
+            '\n'
+            'Features\n'
+            '--------\n'
+            ' - We now support the web. (#5)\n'
+            ' - The widget is more robust. (#12)\n'
+            ' - A very long feature which takes many words to describe with any\n'
+            '   accuracy was introduced so that the line wrapping behavior of the\n'
+            '   news generating code could be verified. (#15)\n'
+            ' - A simpler feature described on multiple lines was added. (#16)\n'
+            '\n'
+            'Bugfixes\n'
+            '--------\n'
+            ' - Broken stuff was fixed. (#23)\n'
+            '\n'
+            'Deprecations and Removals\n'
+            '-------------------------\n'
+            ' - Stupid stuff was deprecated. (#25)\n'
+            '\n'
+            'Other\n'
+            '-----\n'
+            ' - #30, #35\n'
+            '\n'
+            'Blah blah other stuff.\n')
+
+
+    def test_emptySectionsOmitted(self):
+        """
+        If there are no changes of a particular type (feature, bugfix, etc), no
+        section for that type is written by L{NewsBuilder.build}.
+        """
+        for ticket in self.project.children():
+            basename = ticket.basename()
+            if basename.endswith('.feature') or basename.endswith('.misc'):
+                ticket.remove()
+
+        self.builder.build(
+            self.project, self.project.child('NEWS'),
+            'Some Thing 1.2')
+
+        self.assertEquals(
+            self.project.child('NEWS').getContent(),
+            'Some Thing 1.2\n'
+            '==============\n'
+            '\n'
+            'Bugfixes\n'
+            '--------\n'
+            ' - Broken stuff was fixed. (#23)\n'
+            '\n'
+            'Deprecations and Removals\n'
+            '-------------------------\n'
+            ' - Stupid stuff was deprecated. (#25)\n'
+            '\n'
+            'Here is stuff which was present previously.\n')
+
+
+    def test_duplicatesMerged(self):
+        """
+        If two change files have the same contents, they are merged in the
+        generated news entry.
+        """
+        def feature(s):
+            return self.project.child(s + '.feature')
+        feature('5').copyTo(feature('15'))
+        feature('5').copyTo(feature('16'))
+
+        self.builder.build(
+            self.project, self.project.child('NEWS'),
+            'Project Name 5.0')
+
+        self.assertEquals(
+            self.project.child('NEWS').getContent(),
+            'Project Name 5.0\n'
+            '================\n'
+            '\n'
+            'Features\n'
+            '--------\n'
+            ' - We now support the web. (#5, #15, #16)\n'
+            ' - The widget is more robust. (#12)\n'
+            '\n'
+            'Bugfixes\n'
+            '--------\n'
+            ' - Broken stuff was fixed. (#23)\n'
+            '\n'
+            'Deprecations and Removals\n'
+            '-------------------------\n'
+            ' - Stupid stuff was deprecated. (#25)\n'
+            '\n'
+            'Other\n'
+            '-----\n'
+            ' - #30, #35\n'
+            '\n'
+            'Here is stuff which was present previously.\n')
+
+
+    def test_buildAll(self):
+        """
+        L{NewsBuilder.buildAll} calls L{NewsBuilder.build} once for each
+        subproject, passing that subproject's I{topfiles} directory as C{path},
+        the I{NEWS} file in that directory as C{output}, and the subproject's
+        name as C{header}, and then again for each subproject with the
+        top-level I{NEWS} file for C{output}.
+        """
+        builds = []
+        builder = NewsBuilder()
+        builder.build = lambda path, output, header: builds.append((
+                path, output, header))
+        builder._today = lambda: '2009-12-01'
+
+        # Create a fake looking Twisted project to build from
+        project = FilePath(self.mktemp()).child("twisted")
+        project.makedirs()
+        self.createStructure(project, {
+                'NEWS': 'Old boring stuff from the past.\n',
+                '_version.py': genVersion("twisted", 1, 2, 3),
+                'topfiles': {
+                    'NEWS': 'Old core news.\n',
+                    '3.feature': 'Third feature addition.\n',
+                    '5.misc': ''},
+                'conch': {
+                    '_version.py': genVersion("twisted.conch", 3, 4, 5),
+                    'topfiles': {
+                        'NEWS': 'Old conch news.\n',
+                        '7.bugfix': 'Fixed that bug.\n'}}})
+
+        builder.buildAll(project)
+
+        coreTopfiles = project.child("topfiles")
+        coreNews = coreTopfiles.child("NEWS")
+        coreHeader = "Twisted Core 1.2.3 (2009-12-01)"
+
+        conchTopfiles = project.child("conch").child("topfiles")
+        conchNews = conchTopfiles.child("NEWS")
+        conchHeader = "Twisted Conch 3.4.5 (2009-12-01)"
+
+        aggregateNews = project.child("NEWS")
+
+        self.assertEquals(
+            builds,
+            [(conchTopfiles, conchNews, conchHeader),
+             (coreTopfiles, coreNews, coreHeader),
+             (conchTopfiles, aggregateNews, conchHeader),
+             (coreTopfiles, aggregateNews, coreHeader)])
+
+
 
 class DistributionBuilderTestBase(BuilderTestsMixin, StructureAssertingMixin,
                                    TestCase):
     """
     Base for tests of L{DistributionBuilder}.
     """
+    skip = loreSkip
 
     def setUp(self):
         BuilderTestsMixin.setUp(self)
@@ -1618,12 +2163,7 @@ class BuildAllTarballsTest(DistributionBuilderTestBase):
     """
     Tests for L{DistributionBuilder.buildAllTarballs}.
     """
-
-    if not (which("svn") and which("svnadmin")):
-        skip = "buildAllTarballs requires svn to be installed."
-    if not popen4Available:
-        skip = "buildAllTarballs requires popen2.Popen4."
-
+    skip = svnSkip or popen4Skip
 
     def setUp(self):
         self.oldHandler = signal.signal(signal.SIGCHLD, signal.SIG_DFL)
@@ -1914,9 +2454,23 @@ class ScriptTests(BuilderTestsMixin, StructureAssertingMixin, TestCase):
         self.assertRaises(SystemExit, tarballBuilder.main, [])
 
 
-if lore is None:
-    skipMessage = "Lore is not present."
-    BookBuilderTests.skip = skipMessage
-    DocBuilderTestCase.skip = skipMessage
-    ManBuilderTestCase.skip = skipMessage
-    DistributionBuilderTests.skip = skipMessage
+    def test_badNumberOfArgumentsToBuildNews(self):
+        """
+        L{NewsBuilder.main} raises L{SystemExit} when other than 1 argument is
+        passed to it.
+        """
+        newsBuilder = NewsBuilder()
+        self.assertRaises(SystemExit, newsBuilder.main, [])
+        self.assertRaises(SystemExit, newsBuilder.main, ["hello", "world"])
+
+
+    def test_buildNews(self):
+        """
+        L{NewsBuilder.main} calls L{NewsBuilder.buildAll} with a L{FilePath}
+        instance constructed from the path passed to it.
+        """
+        builds = []
+        newsBuilder = NewsBuilder()
+        newsBuilder.buildAll = builds.append
+        newsBuilder.main(["/foo/bar/baz"])
+        self.assertEquals(builds, [FilePath("/foo/bar/baz")])
