@@ -1,13 +1,17 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
+"""
+Tests for lots of functionality provided by L{twisted.internet}.
+"""
+
+import os
+import sys
+import time
 
 from twisted.trial import unittest
 from twisted.internet import reactor, protocol, error, abstract, defer
-from twisted.internet import interfaces, base, task
-from twisted.internet.tcp import Connection
-
-from twisted.test.time_helpers import Clock
+from twisted.internet import interfaces, base
 
 try:
     from twisted.internet import ssl
@@ -19,11 +23,6 @@ if ssl and not ssl.supported:
 from twisted.internet.defer import Deferred, maybeDeferred
 from twisted.python import util, runtime
 
-import os
-import sys
-import time
-import socket
-
 
 
 class ThreePhaseEventTests(unittest.TestCase):
@@ -781,49 +780,6 @@ class TimeTestCase(unittest.TestCase):
         return d
 
 
-    def testCallLaterDelayAndReset(self):
-        """
-        Test that the reactor handles DelayedCalls which have been
-        reset or delayed.
-        """
-        clock = Clock()
-        clock.install()
-        try:
-            callbackTimes = [None, None]
-
-            def resetCallback():
-                callbackTimes[0] = clock()
-
-            def delayCallback():
-                callbackTimes[1] = clock()
-
-            ireset = reactor.callLater(2, resetCallback)
-            idelay = reactor.callLater(3, delayCallback)
-
-            clock.pump(reactor, [0, 1])
-
-            self.assertIdentical(callbackTimes[0], None)
-            self.assertIdentical(callbackTimes[1], None)
-
-            ireset.reset(2) # (now)1 + 2 = 3
-            idelay.delay(3) # (orig)3 + 3 = 6
-
-            clock.pump(reactor, [0, 1])
-
-            self.assertIdentical(callbackTimes[0], None)
-            self.assertIdentical(callbackTimes[1], None)
-
-            clock.pump(reactor, [0, 1])
-
-            self.assertEquals(callbackTimes[0], 3)
-            self.assertEquals(callbackTimes[1], None)
-
-            clock.pump(reactor, [0, 3])
-            self.assertEquals(callbackTimes[1], 6)
-        finally:
-            clock.uninstall()
-
-
     def testCallLaterTime(self):
         d = reactor.callLater(10, lambda: None)
         try:
@@ -831,26 +787,6 @@ class TimeTestCase(unittest.TestCase):
         finally:
             d.cancel()
 
-    def testCallInNextIteration(self):
-        calls = []
-        def f1():
-            calls.append('f1')
-            reactor.callLater(0.0, f2)
-        def f2():
-            calls.append('f2')
-            reactor.callLater(0.0, f3)
-        def f3():
-            calls.append('f3')
-
-        reactor.callLater(0, f1)
-        self.assertEquals(calls, [])
-        reactor.iterate()
-        self.assertEquals(calls, ['f1'])
-        reactor.iterate()
-        self.assertEquals(calls, ['f1', 'f2'])
-        reactor.iterate()
-        self.assertEquals(calls, ['f1', 'f2', 'f3'])
-
     def testCallLaterOrder(self):
         l = []
         l2 = []
@@ -977,231 +913,12 @@ class CallFromThreadTests(unittest.TestCase):
         return d
 
 
-
-class DummyReactor(base.ReactorBase):
-    """
-    A reactor faking the methods needed to make it starts.
-    """
-
-    def __init__(self, clock):
-        """
-        @param clock: the clock used to fake time.
-        @type clock: C{task.Clock}.
-        """
-        base.ReactorBase.__init__(self)
-        self.clock = clock
-
-
-    def installWaker(self):
-        """
-        Called by C{self._initThreads}: no waker is needed for the tests.
-        """
-
-
-    def callLater(self, _seconds, _f, *args, **kw):
-        """
-        Override callLater using the internal clock.
-        """
-        return self.clock.callLater( _seconds, _f, *args, **kw)
-
-
-    def removeAll(self):
-        """
-        Needed during stop.
-        """
-        return []
-
-
-
-class ReactorBaseTestCase(unittest.TestCase):
-    """
-    Tests for L{base.ReactorBase} object.
-    """
-
-    def setUp(self):
-        """
-        Create a clock and a L{DummyReactor}.
-        """
-        self.clock = task.Clock()
-        self.reactor = DummyReactor(self.clock)
-
-
-    def test_stopWhenNotStarted(self):
-        """
-        Test that the reactor stop raises an error when the reactor is not
-        started.
-        """
-        self.assertRaises(RuntimeError, self.reactor.stop)
-
-
-    def test_stopWhenAlreadyStopped(self):
-        """
-        Test that the reactor stop raises an error when the reactor is already
-        stopped.
-        """
-        self.reactor.startRunning()
-        self.reactor.stop()
-        self.assertRaises(RuntimeError, self.reactor.stop)
-
-
-    def test_stopShutDownEvents(self):
-        """
-        Verify that reactor.stop fires shutdown events.
-        """
-        events = []
-        self.reactor.addSystemEventTrigger(
-            "before", "shutdown",
-            lambda: events.append(("before", "shutdown")))
-        self.reactor.addSystemEventTrigger(
-            "during", "shutdown",
-            lambda: events.append(("during", "shutdown")))
-        self.reactor.startRunning()
-        self.reactor.stop()
-
-        # Simulate the mainloop spinning a little bit.  Do this to allow
-        # reactor.stop() to schedule the shutdown event to be fired as opposed
-        # to assuming reactor.stop() will fire the shutdown event before
-        # returning.
-
-        # Generally, randomly scheduling things to happen instead of doing them
-        # synchronously is wrong.  However, this is finicky functionality which
-        # was always poorly specified and was implemented such that most times
-        # the shutdown event was fired asynchronously.  If you're implementing
-        # a new API, don't look at this advance(0) and think it's great and
-        # copy it.
-
-        # See #3168, #3146, and #3198.
-        self.reactor.clock.advance(0)
-
-        self.assertEquals(events, [("before", "shutdown"),
-                                   ("during", "shutdown")])
-
-
-    def test_multipleRun(self):
-        """
-        Verify that the reactor.startRunning raises an error when called
-        multiple times.
-        """
-        self.reactor.startRunning()
-        self.assertWarns(DeprecationWarning,
-            "Reactor already running! This behavior is deprecated since "
-            "Twisted 8.0",
-            __file__,
-            self.reactor.startRunning)
-
-
-    def test_crash(self):
-        """
-        reactor.crash should NOT fire shutdown triggers.
-        """
-        events = []
-        self.reactor.addSystemEventTrigger(
-            "before", "shutdown",
-            lambda: events.append(("before", "shutdown")))
-
-        self.reactor.callWhenRunning(
-            self.reactor.callLater, 0, self.reactor.crash)
-        self.reactor.startRunning()
-        self.clock.advance(0.1)
-        self.failIf(events, "reactor.crash invoked shutdown triggers, but it "
-                            "isn't supposed to.")
-
-
-
-class ReactorCoreTestCase(unittest.TestCase):
-    """
-    Test core functionalities of the reactor.
-    """
-
-    def test_run(self):
-        """
-        Test that reactor.crash terminates reactor.run
-        """
-        for i in xrange(3):
-            reactor.callLater(0.01, reactor.crash)
-            reactor.run()
-
-
-    def test_iterate(self):
-        """
-        Test that reactor.iterate(0) doesn't block
-        """
-        start = time.time()
-        # twisted timers are distinct from the underlying event loop's
-        # timers, so this fail-safe probably won't keep a failure from
-        # hanging the test
-        t = reactor.callLater(10, reactor.crash)
-        reactor.iterate(0) # shouldn't block
-        stop = time.time()
-        elapsed = stop - start
-        self.failUnless(elapsed < 8)
-        t.cancel()
-
-
-
-class ReactorFDTestCase(unittest.TestCase):
-    """
-    Tests for L{interfaces.IReactorFDSet}.
-    """
-
-    def test_getReaders(self):
-        """
-        Check that L{interfaces.IReactorFDSet.getReaders} reflects the actions
-        made with L{interfaces.IReactorFDSet.addReader} and
-        L{interfaces.IReactorFDSet.removeReader}.
-        """
-        s = socket.socket()
-        self.addCleanup(s.close)
-
-        c = Connection(s, protocol.Protocol())
-        self.assertNotIn(c, reactor.getReaders())
-
-        reactor.addReader(c)
-        self.assertIn(c, reactor.getReaders())
-
-        reactor.removeReader(c)
-        self.assertNotIn(c, reactor.getReaders())
-
-
-    def test_getWriters(self):
-        """
-        Check that L{interfaces.IReactorFDSet.getWriters} reflects the actions
-        made with L{interfaces.IReactorFDSet.addWriter} and
-        L{interfaces.IReactorFDSet.removeWriter}.
-        """
-        s = socket.socket()
-        self.addCleanup(s.close)
-
-        c = Connection(s, protocol.Protocol())
-        self.assertNotIn(c, reactor.getWriters())
-
-        reactor.addWriter(c)
-        self.assertIn(c, reactor.getWriters())
-
-        reactor.removeWriter(c)
-        self.assertNotIn(c, reactor.getWriters())
-
-if not interfaces.IReactorFDSet.providedBy(reactor):
-    ReactorFDTestCase.skip = "Reactor not providing IReactorFDSet"
-
-
-
 class DelayedTestCase(unittest.TestCase):
     def setUp(self):
         self.finished = 0
         self.counter = 0
         self.timers = {}
         self.deferred = defer.Deferred()
-        # ick. Sometimes there are magic timers already running:
-        # popsicle.Freezer.tick . Kill off all such timers now so they won't
-        # interfere with the test. Of course, this kind of requires that
-        # getDelayedCalls already works, so certain failure modes won't be
-        # noticed.
-        if not hasattr(reactor, "getDelayedCalls"):
-            return
-        for t in reactor.getDelayedCalls():
-            t.cancel()
-        reactor.iterate() # flush timers
 
     def tearDown(self):
         for t in self.timers.values():
@@ -1264,11 +981,23 @@ class DelayedTestCase(unittest.TestCase):
         self.deferred.addCallback(lambda x : self.checkTimers())
         return self.deferred
 
-    def testActive(self):
-        dcall = reactor.callLater(0, lambda: None)
-        self.assertEquals(dcall.active(), 1)
-        reactor.iterate()
-        self.assertEquals(dcall.active(), 0)
+
+    def test_active(self):
+        """
+        L{IDelayedCall.active} returns False once the call has run.
+        """
+        dcall = reactor.callLater(0.01, self.deferred.callback, True)
+        self.assertEquals(dcall.active(), True)
+
+        def checkDeferredCall(success):
+            self.assertEquals(dcall.active(), False)
+            return success
+
+        self.deferred.addCallback(checkDeferredCall)
+
+        return self.deferred
+
+
 
 resolve_helper = """
 import %(reactor)s
@@ -1350,72 +1079,86 @@ class Resolve(unittest.TestCase):
 if not interfaces.IReactorProcess(reactor, None):
     Resolve.skip = "cannot run test: reactor doesn't support IReactorProcess"
 
-class Counter:
-    index = 0
-
-    def add(self):
-        self.index = self.index + 1
-
-
-class Order:
-
-    stage = 0
-
-    def a(self):
-        if self.stage != 0: raise RuntimeError
-        self.stage = 1
-
-    def b(self):
-        if self.stage != 1: raise RuntimeError
-        self.stage = 2
-
-    def c(self):
-        if self.stage != 2: raise RuntimeError
-        self.stage = 3
 
 
 class CallFromThreadTestCase(unittest.TestCase):
-    """Task scheduling from threads tests."""
-
+    """
+    Task scheduling from threads tests.
+    """
     if interfaces.IReactorThreads(reactor, None) is None:
         skip = "Nothing to test without thread support"
 
+    def setUp(self):
+        self.counter = 0
+        self.deferred = Deferred()
+
+
     def schedule(self, *args, **kwargs):
-        """Override in subclasses."""
+        """
+        Override in subclasses.
+        """
         reactor.callFromThread(*args, **kwargs)
 
-    def testScheduling(self):
-        c = Counter()
-        for i in range(100):
-            self.schedule(c.add)
-        for i in range(100):
-            reactor.iterate()
-        self.assertEquals(c.index, 100)
 
-    def testCorrectOrder(self):
-        o = Order()
-        self.schedule(o.a)
-        self.schedule(o.b)
-        self.schedule(o.c)
-        reactor.iterate()
-        reactor.iterate()
-        reactor.iterate()
-        self.assertEquals(o.stage, 3)
+    def test_lotsOfThreadsAreScheduledCorrectly(self):
+        """
+        L{IReactorThreads.callFromThread} can be used to schedule a large
+        number of calls in the reactor thread.
+        """
+        def addAndMaybeFinish():
+            self.counter += 1
+            if self.counter == 100:
+                self.deferred.callback(True)
+
+        for i in xrange(100):
+            self.schedule(addAndMaybeFinish)
+
+        return self.deferred
+
+
+    def test_threadsAreRunInScheduledOrder(self):
+        """
+        Callbacks should be invoked in the order they were scheduled.
+        """
+        order = []
+
+        def check(_):
+            self.assertEquals(order, [1, 2, 3])
+
+        self.deferred.addCallback(check)
+        self.schedule(order.append, 1)
+        self.schedule(order.append, 2)
+        self.schedule(order.append, 3)
+        self.schedule(reactor.callFromThread, self.deferred.callback, None)
+
+        return self.deferred
+
+
+    def test_scheduledThreadsNotRunUntilReactorRuns(self):
+        """
+        Scheduled tasks should not be run until the reactor starts running.
+        """
+        def incAndFinish():
+            self.counter = 1
+            self.deferred.callback(True)
+        self.schedule(incAndFinish)
+
+        # Callback shouldn't have fired yet.
+        self.assertEquals(self.counter, 0)
+
+        return self.deferred
 
-    def testNotRunAtOnce(self):
-        c = Counter()
-        self.schedule(c.add)
-        # scheduled tasks should not be run at once:
-        self.assertEquals(c.index, 0)
-        reactor.iterate()
-        self.assertEquals(c.index, 1)
 
 
 class MyProtocol(protocol.Protocol):
-    """Sample protocol."""
+    """
+    Sample protocol.
+    """
 
 class MyFactory(protocol.Factory):
-    """Sample factory."""
+    """
+    Sample factory.
+    """
 
     protocol = MyProtocol
 
