@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -25,9 +25,9 @@ class LogTest(unittest.TestCase):
 
     def setUp(self):
         self.catcher = []
-        observer = self.catcher.append
-        log.addObserver(observer)
-        self.addCleanup(log.removeObserver, observer)
+        self.observer = self.catcher.append
+        log.addObserver(self.observer)
+        self.addCleanup(log.removeObserver, self.observer)
 
 
     def testObservation(self):
@@ -120,7 +120,10 @@ class LogTest(unittest.TestCase):
         L{twisted.python.log.showwarning} emits the warning as a message
         to the Twisted logging system.
         """
-        log.showwarning(
+        publisher = log.LogPublisher()
+        publisher.addObserver(self.observer)
+
+        publisher.showwarning(
             FakeWarning("unique warning message"), FakeWarning,
             "warning-filename.py", 27)
         event = self.catcher.pop()
@@ -133,7 +136,7 @@ class LogTest(unittest.TestCase):
         # Python 2.6 requires that any function used to override the
         # warnings.showwarning API accept a "line" parameter or a
         # deprecation warning is emitted.
-        log.showwarning(
+        publisher.showwarning(
             FakeWarning("unique warning message"), FakeWarning,
             "warning-filename.py", 27, line=object())
         event = self.catcher.pop()
@@ -251,7 +254,7 @@ class LogPublisherTestCase(LogPublisherTestCaseMixin, unittest.TestCase):
     def testSingleUnicode(self):
         self.lp.msg(u"Hello, \N{VULGAR FRACTION ONE HALF} world.")
         self.assertEquals(len(self.out), 1)
-        self.assertIn('with str error Traceback', self.out[0])
+        self.assertIn('with str error', self.out[0])
         self.assertIn('UnicodeEncodeError', self.out[0])
 
 
@@ -416,6 +419,33 @@ class FileObserverTestCase(LogPublisherTestCaseMixin, unittest.TestCase):
         self.assertEquals(len(self.out), 0)
 
 
+    def test_startLoggingTwice(self):
+        """
+        There are some obscure error conditions that can occur when logging is
+        started twice. See http://twistedmatrix.com/trac/ticket/3289 for more
+        information.
+        """
+        # The bug is particular to the way that the t.p.log 'global' function
+        # handle stdout. If we use our own stream, the error doesn't occur. If
+        # we use our own LogPublisher, the error doesn't occur.
+        sys.stdout = StringIO()
+        self.addCleanup(setattr, sys, 'stdout', sys.__stdout__)
+
+        def showError(eventDict):
+            if eventDict['isError']:
+                sys.__stdout__.write(eventDict['failure'].getTraceback())
+
+        log.addObserver(showError)
+        self.addCleanup(log.removeObserver, showError)
+        observer = log.startLogging(sys.stdout)
+        self.addCleanup(observer.stop)
+        # At this point, we expect that sys.stdout is a StdioOnnaStick object.
+        self.assertIsInstance(sys.stdout, log.StdioOnnaStick)
+        fakeStdout = sys.stdout
+        observer = log.startLogging(sys.stdout)
+        self.assertIdentical(sys.stdout, fakeStdout)
+
+
 class PythonLoggingObserverTestCase(unittest.TestCase):
     """
     Test the bridge with python logging module.
