@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -17,20 +17,19 @@ import stat
 try:
     import fcntl
 except ImportError:
-    fcntl = None
+    fcntl = process = None
+else:
+    from twisted.internet import process
+
 
 from zope.interface.verify import verifyObject
 
+from twisted.python.log import msg
 from twisted.internet import reactor, protocol, error, interfaces, defer
 from twisted.trial import unittest
 from twisted.python import util, runtime, procutils
 from twisted.python.compat import set
 
-try:
-    from twisted.internet import process
-except ImportError:
-    process = None
-
 
 
 class StubProcessProtocol(protocol.ProcessProtocol):
@@ -265,15 +264,36 @@ class SignalProtocol(protocol.ProcessProtocol):
 
     @ivar signal: the signal to send to the process.
     @type signal: C{str}
+
+    @ivar signaled: A flag tracking whether the signal has been sent to the
+        child or not yet.  C{False} until it is sent, then C{True}.
+    @type signaled: C{bool}
     """
 
     def __init__(self, deferred, sig):
         self.deferred = deferred
         self.signal = sig
+        self.signaled = False
 
 
     def outReceived(self, data):
-        self.transport.signalProcess(self.signal)
+        """
+        Handle the first output from the child process (which indicates it
+        is set up and ready to receive the signal) by sending the signal to
+        it.  Also log all output to help with debugging.
+        """
+        msg("Received %r from child stdout" % (data,))
+        if not self.signaled:
+            self.signaled = True
+            self.transport.signalProcess(self.signal)
+
+
+    def errReceived(self, data):
+        """
+        Log all data received from the child's stderr to help with
+        debugging.
+        """
+        msg("Received %r from child stderr" % (data,))
 
 
     def processEnded(self, reason):
@@ -284,11 +304,15 @@ class SignalProtocol(protocol.ProcessProtocol):
         of the exited process. Otherwise, errback with a C{ValueError}
         describing the problem.
         """
+        msg("Child exited: %r" % (reason.getTraceback(),))
         if not reason.check(error.ProcessTerminated):
             return self.deferred.errback(
                 ValueError("wrong termination: %s" % (reason,)))
         v = reason.value
-        signalValue = getattr(signal, 'SIG' + self.signal)
+        if isinstance(self.signal, str):
+            signalValue = getattr(signal, 'SIG' + self.signal)
+        else:
+            signalValue = self.signal
         if v.exitCode is not None:
             return self.deferred.errback(
                 ValueError("SIG%s: exitCode is %s, not None" %
@@ -488,7 +512,7 @@ class ProcessTestCase(unittest.TestCase):
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_echoer.py")
         procTrans = reactor.spawnProcess(p, exe,
-                                    [exe, "-u", scriptPath], env=None)
+                                    [exe, scriptPath], env=None)
         self.failUnless(procTrans.pid)
 
         def afterProcessEnd(ignored):
@@ -553,13 +577,19 @@ class ProcessTestCase(unittest.TestCase):
         deferredList.addCallback(_check, protocols)
         return deferredList
 
-    def testEcho(self):
+
+    def test_echo(self):
+        """
+        A spawning a subprocess which echoes its stdin to its stdout via
+        C{reactor.spawnProcess} will result in that echoed output being
+        delivered to outReceived.
+        """
         finished = defer.Deferred()
         p = EchoProtocol(finished)
 
         exe = sys.executable
         scriptPath = util.sibpath(__file__, "process_echoer.py")
-        reactor.spawnProcess(p, exe, [exe, "-u", scriptPath], env=None)
+        reactor.spawnProcess(p, exe, [exe, scriptPath], env=None)
 
         def asserts(ignored):
             self.failIf(p.failure, p.failure)
@@ -571,9 +601,6 @@ class ProcessTestCase(unittest.TestCase):
             return err
 
         return finished.addCallback(asserts).addErrback(takedownProcess)
-    testEcho.timeout = 60 # XXX This should not be.  There is already a
-                          # global timeout value.  Why do you think this
-                          # test can complete more quickly?
 
 
     def testCommandLine(self):
@@ -1071,6 +1098,33 @@ class PosixProcessBase:
         return self._testSignal('TERM')
 
 
+    def test_childSignalHandling(self):
+        """
+        The disposition of signals which are ignored in the parent
+        process is reset to the default behavior for the child
+        process.
+        """
+        # Somewhat arbitrarily select SIGUSR1 here.  It satisfies our
+        # requirements that:
+        #    - The interpreter not fiddle around with the handler
+        #      behind our backs at startup time (this disqualifies
+        #      signals like SIGINT and SIGPIPE).
+        #    - The default behavior is to exit.
+        #
+        # This lets us send the signal to the child and then verify
+        # that it exits with a status code indicating that it was
+        # indeed the signal which caused it to exit.
+        which = signal.SIGUSR1
+
+        # Ignore the signal in the parent (and make sure we clean it
+        # up).
+        handler = signal.signal(which, signal.SIG_IGN)
+        self.addCleanup(signal.signal, signal.SIGUSR1, handler)
+
+        # Now do the test.
+        return self._testSignal(signal.SIGUSR1)
+
+
     def test_executionError(self):
         """
         Raise an error during execvpe to check error management.
@@ -1149,6 +1203,17 @@ class PosixProcessBase:
 
 
 
+class MockSignal(object):
+    """
+    Neuter L{signal.signal}, but pass other attributes unscathed
+    """
+    def signal(self, sig, action):
+        return signal.getsignal(sig)
+
+    def __getattr__(self, attr):
+        return getattr(signal, attr)
+
+
 class MockOS(object):
     """
     The mock OS: overwrite L{os}, L{fcntl} and {sys} functions with fake ones.
@@ -1224,16 +1289,16 @@ class MockOS(object):
 
     def __init__(self):
         """
-        Initialiaze data structures.
+        Initialize data structures.
         """
         self.actions = []
         self.closed = []
         self.pipeCount = 0
-        self.O_RDWR = os.O_RDWR
-        self.O_NOCTTY = os.O_NOCTTY
-        self.WNOHANG = os.WNOHANG
-        self.WEXITSTATUS = os.WEXITSTATUS
-        self.WIFEXITED = os.WIFEXITED
+        self.O_RDWR = -1
+        self.O_NOCTTY = -2
+        self.WNOHANG = -4
+        self.WEXITSTATUS = lambda x: 0
+        self.WIFEXITED = lambda x: 1
         self.seteuidCalls = []
         self.setegidCalls = []
 
@@ -1529,6 +1594,9 @@ class MockProcessTestCase(unittest.TestCase):
         self.patch(process.Process, "processWriterFactory", DumbProcessWriter)
         self.patch(process, "pty", self.mockos)
 
+        self.mocksig = MockSignal()
+        self.patch(process, "signal", self.mocksig)
+
 
     def tearDown(self):
         """
@@ -1898,7 +1966,7 @@ class MockProcessTestCase(unittest.TestCase):
 
 
 class PosixProcessTestCase(unittest.TestCase, PosixProcessBase):
-    # add three non-pty test cases
+    # add two non-pty test cases
 
     def testStderr(self):
         # we assume there is no file named ZZXXX..., both in . and in /tmp
@@ -2069,6 +2137,56 @@ class Win32ProcessTestCase(unittest.TestCase):
         return self._testSignal('KILL')
 
 
+    def test_closeHandles(self):
+        """
+        The win32 handles should be properly closed when the process exits.
+        """
+        import win32api
+
+        connected = defer.Deferred()
+        ended = defer.Deferred()
+
+        class SimpleProtocol(protocol.ProcessProtocol):
+            """
+            A protocol that fires deferreds when connected and disconnected.
+            """
+            def makeConnection(self, transport):
+                connected.callback(transport)
+
+            def processEnded(self, reason):
+                ended.callback(None)
+
+        p = SimpleProtocol()
+
+        pyExe = sys.executable
+        pyArgs = [pyExe, "-u", "-c", "print 'hello'"]
+        proc = reactor.spawnProcess(p, pyExe, pyArgs)
+
+        def cbConnected(transport):
+            self.assertIdentical(transport, proc)
+            # perform a basic validity test on the handles
+            win32api.GetHandleInformation(proc.hProcess)
+            win32api.GetHandleInformation(proc.hThread)
+            # And save their values for later
+            self.hProcess = proc.hProcess
+            self.hThread = proc.hThread
+        connected.addCallback(cbConnected)
+
+        def checkTerminated(ignored):
+            # The attributes on the process object must be reset...
+            self.assertIdentical(proc.pid, None)
+            self.assertIdentical(proc.hProcess, None)
+            self.assertIdentical(proc.hThread, None)
+            # ...and the handles must be closed.
+            self.assertRaises(win32api.error,
+                              win32api.GetHandleInformation, self.hProcess)
+            self.assertRaises(win32api.error,
+                              win32api.GetHandleInformation, self.hThread)
+        ended.addCallback(checkTerminated)
+
+        return defer.gatherResults([connected, ended])
+
+
 
 class Dumbwin32procPidTest(unittest.TestCase):
     """
