@@ -1,11 +1,11 @@
-# Copyright (c) 2007-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2007-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Tests for L{twisted.application.app} and L{twisted.scripts.twistd}.
 """
 
-import signal, inspect
+import signal, inspect, errno
 
 import os, sys, cPickle, StringIO
 try:
@@ -25,6 +25,7 @@ from twisted.python.log import ILogObserver
 from twisted.python.versions import Version
 from twisted.python.components import Componentized
 from twisted.internet.defer import Deferred
+from twisted.python.fakepwd import UserDatabase
 
 try:
     from twisted.python import syslog
@@ -89,12 +90,10 @@ def patchUserDatabase(patch, user, uid, group, gid):
     pwent = pwd.getpwuid(os.getuid())
     grent = grp.getgrgid(os.getgid())
 
-    def getpwnam(name):
-        result = list(pwent)
-        result[result.index(pwent.pw_name)] = user
-        result[result.index(pwent.pw_uid)] = uid
-        result = tuple(result)
-        return {user: result}[name]
+    database = UserDatabase()
+    database.addUser(
+        user, pwent.pw_passwd, uid, pwent.pw_gid,
+        pwent.pw_gecos, pwent.pw_dir, pwent.pw_shell)
 
     def getgrnam(name):
         result = list(grent)
@@ -103,7 +102,7 @@ def patchUserDatabase(patch, user, uid, group, gid):
         result = tuple(result)
         return {group: result}[name]
 
-    patch(pwd, "getpwnam", getpwnam)
+    patch(pwd, "getpwnam", database.getpwnam)
     patch(grp, "getgrnam", getgrnam)
 
 
@@ -687,6 +686,41 @@ class UnixApplicationRunnerStartApplicationTests(unittest.TestCase):
 
 
 
+class UnixApplicationRunnerRemovePID(unittest.TestCase):
+    """
+    Tests for L{UnixApplicationRunner.removePID}.
+    """
+    if _twistd_unix is None:
+        skip = "twistd unix not available"
+
+
+    def test_removePID(self):
+        """
+        L{UnixApplicationRunner.removePID} deletes the file the name of
+        which is passed to it.
+        """
+        runner = UnixApplicationRunner({})
+        path = self.mktemp()
+        os.makedirs(path)
+        pidfile = os.path.join(path, "foo.pid")
+        file(pidfile, "w").close()
+        runner.removePID(pidfile)
+        self.assertFalse(os.path.exists(pidfile))
+
+
+    def test_removePIDErrors(self):
+        """
+        Calling L{UnixApplicationRunner.removePID} with a non-existent filename logs
+        an OSError.
+        """
+        runner = UnixApplicationRunner({})
+        runner.removePID("fakepid")
+        errors = self.flushLoggedErrors(OSError)
+        self.assertEquals(len(errors), 1)
+        self.assertEquals(errors[0].value.errno, errno.ENOENT)
+
+
+
 class DummyReactor(object):
     """
     A dummy reactor, only providing a C{run} method and checking that it
@@ -1311,23 +1345,21 @@ class UnixAppLoggerTestCase(unittest.TestCase):
         If C{syslog} is set to C{True}, L{UnixAppLogger._getLogObserver} starts
         a L{syslog.SyslogObserver} with given C{prefix}.
         """
-        class fakesyslog(object):
-            def openlog(self, prefix):
-                self.prefix = prefix
-        syslogModule = fakesyslog()
-        self.patch(syslog, "syslog", syslogModule)
+        class fakesyslogobserver(object):
+            def __init__(self, prefix):
+                fakesyslogobserver.prefix = prefix
+            def emit(self, eventDict):
+                pass
+        self.patch(syslog, "SyslogObserver", fakesyslogobserver)
         logger = UnixAppLogger({"syslog": True, "prefix": "test-prefix"})
         observer = logger._getLogObserver()
-        self.assertEquals(syslogModule.prefix, "test-prefix")
+        self.assertEquals(fakesyslogobserver.prefix, "test-prefix")
 
     if syslog is None:
         test_getLogObserverSyslog.skip = "Syslog not available"
 
 
 
-
-
-
 class DeprecationTests(unittest.TestCase):
     """
     Tests for deprecated features.
