@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.trial.test.test_runner -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -9,16 +9,16 @@ Maintainer: Jonathan Lange
 """
 
 
-import pdb, shutil
+import pdb
 import os, types, warnings, sys, inspect, imp
 import random, doctest, time
 
-from twisted.python import reflect, log, failure, modules
+from twisted.python import reflect, log, failure, modules, filepath
 from twisted.python.util import dsu
 from twisted.python.compat import set
 from twisted.python.lockfile import FilesystemLock
 
-from twisted.internet import defer, interfaces
+from twisted.internet import defer
 from twisted.trial import util, unittest
 from twisted.trial.itrial import ITestCase
 from twisted.trial.reporter import UncleanWarningsReporterWrapper
@@ -40,6 +40,16 @@ class _WorkingDirectoryBusy(Exception):
 
 
 
+class _NoTrialMarker(Exception):
+    """
+    No trial marker file could be found.
+
+    Raised when trial attempts to remove a trial temporary working directory
+    that does not contain a marker file.
+    """
+
+
+
 def isPackage(module):
     """Given an object return True if the object looks like a package"""
     if not isinstance(module, types.ModuleType):
@@ -66,6 +76,7 @@ def samefile(filename1, filename2):
     """
     return os.path.abspath(filename1) == os.path.abspath(filename2)
 
+
 def filenameToModule(fn):
     """
     Given a filename, do whatever possible to return a module object matching
@@ -277,9 +288,6 @@ class TrialSuite(TestSuite):
         reactor.addSystemEventTrigger('after', 'shutdown',
                                       lambda: d.callback(None))
         reactor.fireSystemEvent('shutdown') # radix's suggestion
-        treactor = interfaces.IReactorThreads(reactor, None)
-        if treactor is not None:
-            treactor.suggestThreadPoolSize(0)
         # As long as TestCase does crap stuff with the reactor we need to
         # manually shutdown the reactor here, and that requires util.wait
         # :(
@@ -700,36 +708,50 @@ class TrialRunner(object):
 
 
     def _removeSafely(self, path):
+        """
+        Safely remove a path, recursively.
+
+        If C{path} does not contain a node named C{"_trial_marker"}, a
+        L{_NoTrialMarker} exception is raised and the path is not removed.
+
+        @type path: L{twisted.python.filepath.FilePath}
+        @param path: The absolute path to a test directory
+        """
+        if not path.child('_trial_marker').exists():
+            raise _NoTrialMarker(
+                '%r is not a trial temporary path, refusing to remove it'
+                % (path,))
+
         try:
-            shutil.rmtree(path)
+            path.remove()
         except OSError, e:
             print ("could not remove %r, caught OSError [Errno %s]: %s"
-                   % (path, e.errno,e.strerror))
+                   % (path, e.errno, e.strerror))
             try:
-                os.rename(path,
-                          os.path.abspath("_trial_temp_old%s"
-                                          % random.randint(0, 99999999)))
+                newPath = filepath.FilePath('_trial_temp_old%s'
+                                            % random.randint(0, 99999999))
+                path.moveTo(newPath)
             except OSError, e:
                 print ("could not rename path, caught OSError [Errno %s]: %s"
-                       % (e.errno,e.strerror))
+                       % (e.errno, e.strerror))
                 raise
 
 
     def _setUpTestdir(self):
         self._tearDownLogFile()
         currentDir = os.getcwd()
-        base = os.path.normpath(os.path.abspath(self.workingDirectory))
+        base = filepath.FilePath(self.workingDirectory)
         counter = 0
         while True:
             if counter:
-                testdir = '%s-%d' % (base, counter)
+                testdir = base.sibling('%s-%d' % (base.basename(), counter))
             else:
                 testdir = base
 
-            self._testDirLock = FilesystemLock(testdir + '.lock')
+            self._testDirLock = FilesystemLock(testdir.path + '.lock')
             if self._testDirLock.lock():
                 # It is not in use
-                if os.path.exists(testdir):
+                if testdir.exists():
                     # It exists though - delete it
                     self._removeSafely(testdir)
                 break
@@ -740,8 +762,9 @@ class TrialRunner(object):
                 else:
                     raise _WorkingDirectoryBusy()
 
-        os.mkdir(testdir)
-        os.chdir(testdir)
+        testdir.makedirs()
+        os.chdir(testdir.path)
+        file('_trial_marker', 'w').close()
         return currentDir
 
 
@@ -824,7 +847,10 @@ class TrialRunner(object):
         suite = TrialSuite([test])
         startTime = time.time()
         if self.mode == self.DRY_RUN:
-            suite.visit(DryRunVisitor(result).markSuccessful)
+            for single in unittest._iterateTests(suite):
+                result.startTest(single)
+                result.addSuccess(single)
+                result.stopTest(single)
         else:
             if self.mode == self.DEBUG:
                 # open question - should this be self.debug() instead.
