@@ -1,4 +1,7 @@
 # -*- test-case-name: twisted.trial.test.test_tests -*-
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 from twisted.trial import unittest, util
 from twisted.internet import reactor, protocol, defer
 
@@ -23,22 +26,6 @@ class TestFailureInTearDown(unittest.TestCase):
         pass
 
 
-class TestFailureInSetUpClass(unittest.TestCase):
-    def setUpClass(self):
-        raise FoolishError, "I am a broken setUpClass method"
-
-    def test_noop(self):
-        pass
-
-
-class TestFailureInTearDownClass(unittest.TestCase):
-    def tearDownClass(self):
-        raise FoolishError, "I am a broken setUp method"
-
-    def test_noop(self):
-        pass
-
-
 class TestRegularFail(unittest.TestCase):
     def test_fail(self):
         self.fail("I fail")
@@ -83,15 +70,6 @@ class TestSkipTestCase(unittest.TestCase):
 TestSkipTestCase.skip = "skipping this test"
 
 
-class TestSkipTestCase2(unittest.TestCase):
-
-    def setUpClass(self):
-        raise unittest.SkipTest, "thi stest is fukct"
-
-    def test_thisTestWillBeSkipped(self):
-        pass
-
-
 class DelayedCall(unittest.TestCase):
     hiddenExceptionMsg = "something blew up"
 
