@@ -1,16 +1,16 @@
-# Copyright (c) 2008 Twisted Matrix Laboratories.
+# Copyright (c) 2008-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
 Tests for loading tests by name.
 """
 
-import md5
 import os
 import shutil
 import sys
 
 from twisted.python import util
+from twisted.python.hashlib import md5
 from twisted.trial.test import packages
 from twisted.trial import runner, reporter, unittest
 from twisted.trial.itrial import ITestCase
@@ -76,6 +76,9 @@ class FinderTest(packages.PackageTest):
 
 
 class FileTest(packages.SysPathManglingTest):
+    """
+    Tests for L{runner.filenameToModule}.
+    """
     def test_notFile(self):
         self.failUnlessRaises(ValueError,
                               runner.filenameToModule, 'doesntexist')
@@ -85,31 +88,60 @@ class FileTest(packages.SysPathManglingTest):
         import sample as sample2
         self.failUnlessEqual(sample2, sample1)
 
+
     def test_moduleNotInPath(self):
+        """
+        If passed the path to a file containing the implementation of a
+        module within a package which is not on the import path,
+        L{runner.filenameToModule} returns a module object loosely
+        resembling the module defined by that file anyway.
+        """
+        # "test_sample" isn't actually the name of this module.  However,
+        # filenameToModule can't seem to figure that out.  So clean up this
+        # mis-named module.  It would be better if this weren't necessary
+        # and filenameToModule either didn't exist or added a correctly
+        # named module to sys.modules.
+        self.addCleanup(sys.modules.pop, 'test_sample', None)
+
         self.mangleSysPath(self.oldPath)
-        sample1 = runner.filenameToModule(os.path.join(self.parent,
-                                                       'goodpackage',
-                                                       'test_sample.py'))
+        sample1 = runner.filenameToModule(
+            os.path.join(self.parent, 'goodpackage', 'test_sample.py'))
         self.mangleSysPath(self.newPath)
         from goodpackage import test_sample as sample2
         self.failUnlessEqual(os.path.splitext(sample2.__file__)[0],
                              os.path.splitext(sample1.__file__)[0])
 
+
     def test_packageInPath(self):
         package1 = runner.filenameToModule(os.path.join(self.parent,
                                                         'goodpackage'))
         import goodpackage
         self.failUnlessEqual(goodpackage, package1)
 
+
     def test_packageNotInPath(self):
+        """
+        If passed the path to a directory which represents a package which
+        is not on the import path, L{runner.filenameToModule} returns a
+        module object loosely resembling the package defined by that
+        directory anyway.
+        """
+        # "__init__" isn't actually the name of the package!  However,
+        # filenameToModule is pretty stupid and decides that is its name
+        # after all.  Make sure it gets cleaned up.  See the comment in
+        # test_moduleNotInPath for possible courses of action related to
+        # this.
+        self.addCleanup(sys.modules.pop, "__init__")
+
         self.mangleSysPath(self.oldPath)
-        package1 = runner.filenameToModule(os.path.join(self.parent,
-                                                        'goodpackage'))
+        package1 = runner.filenameToModule(
+            os.path.join(self.parent, 'goodpackage'))
         self.mangleSysPath(self.newPath)
         import goodpackage
         self.failUnlessEqual(os.path.splitext(goodpackage.__file__)[0],
                              os.path.splitext(package1.__file__)[0])
 
+
     def test_directoryNotPackage(self):
         self.failUnlessRaises(ValueError, runner.filenameToModule,
                               util.sibpath(__file__, 'directory'))
@@ -299,6 +331,25 @@ class LoaderTest(packages.SysPathManglingTest):
                                       'package.test_import_module'])
 
 
+    def test_differentInstances(self):
+        """
+        L{TestLoader.loadClass} returns a suite with each test method
+        represented by a different instances of the L{TestCase} they are
+        defined on.
+        """
+        class DistinctInstances(unittest.TestCase):
+            def test_1(self):
+                self.first = 'test1Run'
+
+            def test_2(self):
+                self.assertFalse(hasattr(self, 'first'))
+
+        suite = self.loader.loadClass(DistinctInstances)
+        result = reporter.Reporter()
+        suite.run(result)
+        self.assertTrue(result.wasSuccessful())
+
+
     def test_loadModuleWith_test_suite(self):
         """
         Check that C{test_suite} is used when present and other L{TestCase}s are
@@ -350,7 +401,6 @@ class LoaderTest(packages.SysPathManglingTest):
 
     # XXX - duplicated and modified from test_script
     def assertSuitesEqual(self, test1, test2):
-        loader = runner.TestLoader()
         names1 = testNames(test1)
         names2 = testNames(test2)
         names1.sort()
@@ -484,7 +534,7 @@ class PackageOrderingTest(packages.SysPathManglingTest):
 #             if isinstance(s, type) or isinstance(s, types.ClassType):
 #                 return s.__module__+'.'+s.__name__
             n = runner.name(s)
-            d = md5.new(n).hexdigest()
+            d = md5(n).hexdigest()
             return d
         self.loadSortedPackages(sillySorter)
 
