@@ -1,16 +1,16 @@
-# Copyright (c) 2005-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2005-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 #
-# Maintainer: Jonathan Lange <jml@twistedmatrix.com>
-# Author: Robert Collins <robertc@robertcollins.net>
+# Maintainer: Jonathan Lange
+# Author: Robert Collins
 
 
-import StringIO
+import StringIO, os, sys
 from zope.interface import implements
 
 from twisted.trial.itrial import IReporter, ITestCase
 from twisted.trial import unittest, runner, reporter, util
-from twisted.python import failure, log, reflect
+from twisted.python import failure, log, reflect, filepath
 from twisted.scripts import trial
 from twisted.plugins import twisted_trial
 from twisted import plugin
@@ -44,12 +44,17 @@ class CapturingReporter(object):
     separator = None
     testsRun = None
 
-    def __init__(self, *a, **kw):
+    def __init__(self, stream=None, tbformat=None, rterrors=None,
+                 publisher=None):
         """
         Create a capturing reporter.
         """
         self._calls = []
         self.shouldStop = False
+        self._stream = stream
+        self._tbformat = tbformat
+        self._rterrors = rterrors
+        self._publisher = publisher
 
 
     def startTest(self, method):
@@ -86,72 +91,90 @@ class CapturingReporter(object):
 
 
 
-class TestTrialRunner(unittest.TestCase):
+class TrialRunnerTestsMixin:
+    """
+    Mixin defining tests for L{runner.TrialRunner}.
+    """
+    def tearDown(self):
+        self.runner._tearDownLogFile()
 
-    def setUp(self):
-        self.stream = StringIO.StringIO()
-        self.runner = runner.TrialRunner(CapturingReporter, stream=self.stream)
-        self.test = TestTrialRunner('test_empty')
 
     def test_empty(self):
         """
         Empty test method, used by the other tests.
         """
 
-    def tearDown(self):
-        self.runner._tearDownLogFile()
 
     def _getObservers(self):
         return log.theLogPublisher.observers
 
+
     def test_addObservers(self):
         """
-        Tests that the runner add logging observers during the run.
+        Any log system observers L{TrialRunner.run} adds are removed by the
+        time it returns.
         """
         originalCount = len(self._getObservers())
         self.runner.run(self.test)
         newCount = len(self._getObservers())
-        self.failUnlessEqual(originalCount + 1, newCount)
+        self.assertEqual(newCount, originalCount)
 
-    def test_addObservers_repeat(self):
-        self.runner.run(self.test)
-        count = len(self._getObservers())
-        self.runner.run(self.test)
-        newCount = len(self._getObservers())
-        self.failUnlessEqual(count, newCount)
 
     def test_logFileAlwaysActive(self):
         """
         Test that a new file is opened on each run.
         """
-        oldSetUpLogging = self.runner._setUpLogging
+        oldSetUpLogFile = self.runner._setUpLogFile
         l = []
-        def setUpLogging():
-            oldSetUpLogging()
+        def setUpLogFile():
+            oldSetUpLogFile()
             l.append(self.runner._logFileObserver)
-        self.runner._setUpLogging = setUpLogging
+        self.runner._setUpLogFile = setUpLogFile
         self.runner.run(self.test)
         self.runner.run(self.test)
         self.failUnlessEqual(len(l), 2)
         self.failIf(l[0] is l[1], "Should have created a new file observer")
 
+
     def test_logFileGetsClosed(self):
         """
         Test that file created is closed during the run.
         """
-        oldSetUpLogging = self.runner._setUpLogging
+        oldSetUpLogFile = self.runner._setUpLogFile
         l = []
-        def setUpLogging():
-            oldSetUpLogging()
+        def setUpLogFile():
+            oldSetUpLogFile()
             l.append(self.runner._logFileObject)
-        self.runner._setUpLogging = setUpLogging
+        self.runner._setUpLogFile = setUpLogFile
         self.runner.run(self.test)
         self.failUnlessEqual(len(l), 1)
         self.failUnless(l[0].closed)
 
 
 
-class TrialRunnerWithUncleanWarningsReporter(TestTrialRunner):
+class TestTrialRunner(TrialRunnerTestsMixin, unittest.TestCase):
+    """
+    Tests for L{runner.TrialRunner} with the feature to turn unclean errors
+    into warnings disabled.
+    """
+    def setUp(self):
+        self.stream = StringIO.StringIO()
+        self.runner = runner.TrialRunner(CapturingReporter, stream=self.stream)
+        self.test = TestTrialRunner('test_empty')
+
+
+    def test_publisher(self):
+        """
+        The reporter constructed by L{runner.TrialRunner} is passed
+        L{twisted.python.log} as the value for the C{publisher} parameter.
+        """
+        result = self.runner._makeResult()
+        self.assertIdentical(result._publisher, log)
+
+
+
+class TrialRunnerWithUncleanWarningsReporter(TrialRunnerTestsMixin,
+                                             unittest.TestCase):
     """
     Tests for the TrialRunner's interaction with an unclean-error suppressing
     reporter.
@@ -288,24 +311,31 @@ class TestRunner(unittest.TestCase):
     def getRunner(self):
         r = trial._makeRunner(self.config)
         r.stream = StringIO.StringIO()
+        # XXX The runner should always take care of cleaning this up itself.
+        # It's not clear why this is necessary.  The runner always tears down
+        # its log file.
         self.addCleanup(r._tearDownLogFile)
+        # XXX The runner should always take care of cleaning this up itself as
+        # well.  It's necessary because TrialRunner._setUpTestdir might raise
+        # an exception preventing Reporter.done from being run, leaving the
+        # observer added by Reporter.__init__ still present in the system.
+        # Something better needs to happen inside
+        # TrialRunner._runWithoutDecoration to remove the need for this cludge.
+        r._log = log.LogPublisher()
         return r
 
 
     def test_runner_can_get_reporter(self):
         self.parseOptions([])
         result = self.config['reporter']
-        my_runner = self.getRunner()
-        try:
-            self.assertEqual(result, my_runner._makeResult().__class__)
-        finally:
-            my_runner._tearDownLogFile()
+        runner = self.getRunner()
+        self.assertEqual(result, runner._makeResult().__class__)
 
 
     def test_runner_get_result(self):
         self.parseOptions([])
-        my_runner = self.getRunner()
-        result = my_runner._makeResult()
+        runner = self.getRunner()
+        result = runner._makeResult()
         self.assertEqual(result.__class__, self.config['reporter'])
 
 
@@ -336,10 +366,89 @@ class TestRunner(unittest.TestCase):
     def test_runner_working_directory(self):
         self.parseOptions(['--temp-directory', 'some_path'])
         runner = self.getRunner()
-        try:
-            self.assertEquals(runner.workingDirectory, 'some_path')
-        finally:
-            runner._tearDownLogFile()
+        self.assertEquals(runner.workingDirectory, 'some_path')
+
+
+    def test_concurrentImplicitWorkingDirectory(self):
+        """
+        If no working directory is explicitly specified and the default
+        working directory is in use by another runner, L{TrialRunner.run}
+        selects a different default working directory to use.
+        """
+        self.parseOptions([])
+
+        initialDirectory = os.getcwd()
+        self.addCleanup(os.chdir, initialDirectory)
+
+        firstRunner = self.getRunner()
+        secondRunner = self.getRunner()
+
+        where = {}
+
+        class ConcurrentCase(unittest.TestCase):
+            def test_first(self):
+                """
+                Start a second test run which will have a default working
+                directory which is the same as the working directory of the
+                test run already in progress.
+                """
+                # Change the working directory to the value it had before this
+                # test suite was started.
+                where['concurrent'] = subsequentDirectory = os.getcwd()
+                os.chdir(initialDirectory)
+                self.addCleanup(os.chdir, subsequentDirectory)
+
+                secondRunner.run(ConcurrentCase('test_second'))
+
+            def test_second(self):
+                """
+                Record the working directory for later analysis.
+                """
+                where['record'] = os.getcwd()
+
+        result = firstRunner.run(ConcurrentCase('test_first'))
+        bad = result.errors + result.failures
+        if bad:
+            self.fail(bad[0][1])
+        self.assertEqual(
+            where, {
+                'concurrent': os.path.join(initialDirectory, '_trial_temp'),
+                'record': os.path.join(initialDirectory, '_trial_temp-1')})
+
+
+    def test_concurrentExplicitWorkingDirectory(self):
+        """
+        If a working directory which is already in use is explicitly specified,
+        L{TrialRunner.run} raises L{_WorkingDirectoryBusy}.
+        """
+        self.parseOptions(['--temp-directory', os.path.abspath(self.mktemp())])
+
+        initialDirectory = os.getcwd()
+        self.addCleanup(os.chdir, initialDirectory)
+
+        firstRunner = self.getRunner()
+        secondRunner = self.getRunner()
+
+        class ConcurrentCase(unittest.TestCase):
+            def test_concurrent(self):
+                """
+                Try to start another runner in the same working directory and
+                assert that it raises L{_WorkingDirectoryBusy}.
+                """
+                self.assertRaises(
+                    runner._WorkingDirectoryBusy,
+                    secondRunner.run, ConcurrentCase('test_failure'))
+
+            def test_failure(self):
+                """
+                Should not be called, always fails.
+                """
+                self.fail("test_failure should never be called.")
+
+        result = firstRunner.run(ConcurrentCase('test_concurrent'))
+        bad = result.errors + result.failures
+        if bad:
+            self.fail(bad[0][1])
 
 
     def test_runner_normal(self):
@@ -368,6 +477,94 @@ class TestRunner(unittest.TestCase):
         self.assertEqual(['runcall'], debugger._calls)
 
 
+    def test_removeSafelyNoTrialMarker(self):
+        """
+        If a path doesn't contain a node named C{"_trial_marker"}, that path is
+        not removed by L{runner._removeSafely} and a L{runner._NoTrialMarker}
+        exception is raised instead.
+        """
+        directory = self.mktemp()
+        os.mkdir(directory)
+        dirPath = filepath.FilePath(directory)
+
+        self.parseOptions([])
+        myRunner = self.getRunner()
+        self.assertRaises(runner._NoTrialMarker,
+                          myRunner._removeSafely, dirPath)
+
+
+    def test_removeSafelyRemoveFailsMoveSucceeds(self):
+        """
+        If an L{OSError} is raised while removing a path in
+        L{runner._removeSafely}, an attempt is made to move the path to a new
+        name.
+        """
+        def dummyRemove():
+            """
+            Raise an C{OSError} to emulate the branch of L{runner._removeSafely}
+            in which path removal fails.
+            """
+            raise OSError()
+
+        # Patch stdout so we can check the print statements in _removeSafely
+        out = StringIO.StringIO()
+        stdout = self.patch(sys, 'stdout', out)
+
+        # Set up a trial directory with a _trial_marker
+        directory = self.mktemp()
+        os.mkdir(directory)
+        dirPath = filepath.FilePath(directory)
+        dirPath.child('_trial_marker').touch()
+        # Ensure that path.remove() raises an OSError
+        dirPath.remove = dummyRemove
+
+        self.parseOptions([])
+        myRunner = self.getRunner()
+        myRunner._removeSafely(dirPath)
+        self.assertIn("could not remove FilePath", out.getvalue())
+
+
+    def test_removeSafelyRemoveFailsMoveFails(self):
+        """
+        If an L{OSError} is raised while removing a path in
+        L{runner._removeSafely}, an attempt is made to move the path to a new
+        name. If that attempt fails, the L{OSError} is re-raised.
+        """
+        def dummyRemove():
+            """
+            Raise an C{OSError} to emulate the branch of L{runner._removeSafely}
+            in which path removal fails.
+            """
+            raise OSError("path removal failed")
+
+        def dummyMoveTo(path):
+            """
+            Raise an C{OSError} to emulate the branch of L{runner._removeSafely}
+            in which path movement fails.
+            """
+            raise OSError("path movement failed")
+
+        # Patch stdout so we can check the print statements in _removeSafely
+        out = StringIO.StringIO()
+        stdout = self.patch(sys, 'stdout', out)
+
+        # Set up a trial directory with a _trial_marker
+        directory = self.mktemp()
+        os.mkdir(directory)
+        dirPath = filepath.FilePath(directory)
+        dirPath.child('_trial_marker').touch()
+
+        # Ensure that path.remove() and path.moveTo() both raise OSErrors
+        dirPath.remove = dummyRemove
+        dirPath.moveTo = dummyMoveTo
+
+        self.parseOptions([])
+        myRunner = self.getRunner()
+        error = self.assertRaises(OSError, myRunner._removeSafely, dirPath)
+        self.assertEquals(str(error), "path movement failed")
+        self.assertIn("could not remove FilePath", out.getvalue())
+
+
 
 class TestTrialSuite(unittest.TestCase):
 
@@ -379,6 +576,7 @@ class TestTrialSuite(unittest.TestCase):
 
 
 
+
 class TestUntilFailure(unittest.TestCase):
     class FailAfter(unittest.TestCase):
         """
@@ -390,12 +588,14 @@ class TestUntilFailure(unittest.TestCase):
             if len(self.count) == 3:
                 self.fail('Count reached 3')
 
+
     def setUp(self):
         TestUntilFailure.FailAfter.count = []
         self.test = TestUntilFailure.FailAfter('test_foo')
         self.stream = StringIO.StringIO()
         self.runner = runner.TrialRunner(reporter.Reporter, stream=self.stream)
 
+
     def test_runUntilFailure(self):
         """
         Test that the runUntilFailure method of the runner actually fail after
@@ -406,6 +606,7 @@ class TestUntilFailure(unittest.TestCase):
         self.failIf(result.wasSuccessful())
         self.assertEquals(self._getFailures(result), 1)
 
+
     def _getFailures(self, result):
         """
         Get the number of failures that were reported to a result.
@@ -413,6 +614,43 @@ class TestUntilFailure(unittest.TestCase):
         return len(result.failures)
 
 
+    def test_runUntilFailureDecorate(self):
+        """
+        C{runUntilFailure} doesn't decorate the tests uselessly: it does it one
+        time when run starts, but not at each turn.
+        """
+        decorated = []
+        def decorate(test, interface):
+            decorated.append((test, interface))
+            return test
+        self.patch(unittest, "decorate", decorate)
+        result = self.runner.runUntilFailure(self.test)
+        self.failUnlessEqual(result.testsRun, 1)
+
+        self.assertEquals(len(decorated), 1)
+        self.assertEquals(decorated, [(self.test, ITestCase)])
+
+
+    def test_runUntilFailureForceGCDecorate(self):
+        """
+        C{runUntilFailure} applies the force-gc decoration after the standard
+        L{ITestCase} decoration, but only one time.
+        """
+        decorated = []
+        def decorate(test, interface):
+            decorated.append((test, interface))
+            return test
+        self.patch(unittest, "decorate", decorate)
+        self.runner._forceGarbageCollection = True
+        result = self.runner.runUntilFailure(self.test)
+        self.failUnlessEqual(result.testsRun, 1)
+
+        self.assertEquals(len(decorated), 2)
+        self.assertEquals(decorated,
+            [(self.test, ITestCase),
+             (self.test, unittest._ForceGarbageCollectionDecorator)])
+
+
 
 class UncleanUntilFailureTests(TestUntilFailure):
     """
