@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.trial.test.test_util -*-
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 #
 
@@ -188,15 +188,11 @@ class _Janitor(object):
     def _cleanThreads(self):
         reactor = self._getReactor()
         if interfaces.IReactorThreads.providedBy(reactor):
-            reactor.suggestThreadPoolSize(0)
-            if getattr(reactor, 'threadpool', None) is not None:
-                try:
-                    reactor.removeSystemEventTrigger(
-                        reactor.threadpoolShutdownID)
-                except KeyError:
-                    pass
-                # Remove the threadpool, and let the reactor put it back again
-                # later like a good boy
+            if reactor.threadpool is not None:
+                # Stop the threadpool now so that a new one is created. 
+                # This improves test isolation somewhat (although this is a
+                # post class cleanup hook, so it's only isolating classes
+                # from each other, not methods from each other).
                 reactor._stopThreadPool()
 
     def _cleanReactor(self):
@@ -213,6 +209,21 @@ class _Janitor(object):
         return selectableStrings
 
 
+def excInfoOrFailureToExcInfo(err):
+    """
+    Coerce a Failure to an _exc_info, if err is a Failure.
+
+    @param err: Either a tuple such as returned by L{sys.exc_info} or a
+        L{Failure} object.
+    @return: A tuple like the one returned by L{sys.exc_info}. e.g.
+        C{exception_type, exception_object, traceback_object}.
+    """
+    if isinstance(err, Failure):
+        # Unwrap the Failure into a exc_info tuple.
+        err = (err.type, err.value, err.getTracebackObject())
+    return err
+
+
 def suppress(action='ignore', **kwarg):
     """
     Sets up the .suppress tuple properly, pass options to this method as you
@@ -364,4 +375,4 @@ _runSequentially = defer.deferredGenerator(_runSequentially)
 
 
 __all__ = ['FailureError', 'DirtyReactorWarning', 'DirtyReactorError',
-           'PendingTimedCallsError', 'runSequentially']
+           'PendingTimedCallsError', 'excInfoOrFailureToExcInfo']
