@@ -1,12 +1,9 @@
 # -*- test-case-name: twisted.test.test_internet -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Very basic functionality for a Reactor implementation.
-
-Maintainer: Itamar Shtull-Trauring
 """
 
 import socket # needed only for sync-dns
@@ -14,19 +11,16 @@ from zope.interface import implements, classImplements
 
 import sys
 import warnings
-import operator
 from heapq import heappush, heappop, heapify
 
-try:
-    import fcntl
-except ImportError:
-    fcntl = None
 import traceback
 
+from twisted.python.compat import set
+from twisted.python.util import unsignedID
 from twisted.internet.interfaces import IReactorCore, IReactorTime, IReactorThreads
 from twisted.internet.interfaces import IResolverSimple, IReactorPluggableResolver
 from twisted.internet.interfaces import IConnector, IDelayedCall
-from twisted.internet import main, error, abstract, defer, threads
+from twisted.internet import fdesc, main, error, abstract, defer, threads
 from twisted.python import log, failure, reflect
 from twisted.python.runtime import seconds as runtimeSeconds, platform, platformType
 from twisted.internet.defer import Deferred, DeferredList
@@ -159,6 +153,7 @@ class DelayedCall(styles.Ephemeral):
     def __le__(self, other):
         return self.time <= other.time
 
+
     def __str__(self):
         if self._str is not None:
             return self._str
@@ -173,8 +168,9 @@ class DelayedCall(styles.Ephemeral):
             func = None
 
         now = self.seconds()
-        L = ["<DelayedCall %s [%ss] called=%s cancelled=%s" % (
-                id(self), self.time - now, self.called, self.cancelled)]
+        L = ["<DelayedCall 0x%x [%ss] called=%s cancelled=%s" % (
+                unsignedID(self), self.time - now, self.called,
+                self.cancelled)]
         if func is not None:
             L.extend((" ", func, "("))
             if self.args:
@@ -192,22 +188,36 @@ class DelayedCall(styles.Ephemeral):
         return "".join(L)
 
 
-class ThreadedResolver:
+
+class ThreadedResolver(object):
+    """
+    L{ThreadedResolver} uses a reactor, a threadpool, and
+    L{socket.gethostbyname} to perform name lookups without blocking the
+    reactor thread.  It also supports timeouts indepedently from whatever
+    timeout logic L{socket.gethostbyname} might have.
+
+    @ivar reactor: The reactor the threadpool of which will be used to call
+        L{socket.gethostbyname} and the I/O thread of which the result will be
+        delivered.
+    """
     implements(IResolverSimple)
 
     def __init__(self, reactor):
         self.reactor = reactor
         self._runningQueries = {}
 
+
     def _fail(self, name, err):
         err = error.DNSLookupError("address %r not found: %s" % (name, err))
         return failure.Failure(err)
 
+
     def _cleanup(self, name, lookupDeferred):
         userDeferred, cancelCall = self._runningQueries[lookupDeferred]
         del self._runningQueries[lookupDeferred]
         userDeferred.errback(self._fail(name, "timeout error"))
 
+
     def _checkTimeout(self, result, name, lookupDeferred):
         try:
             userDeferred, cancelCall = self._runningQueries[lookupDeferred]
@@ -222,19 +232,31 @@ class ThreadedResolver:
             else:
                 userDeferred.callback(result)
 
+
     def getHostByName(self, name, timeout = (1, 3, 11, 45)):
+        """
+        See L{twisted.internet.interfaces.IResolverSimple.getHostByName}.
+
+        Note that the elements of C{timeout} are summed and the result is used
+        as a timeout for the lookup.  Any intermediate timeout or retry logic
+        is left up to the platform via L{socket.gethostbyname}.
+        """
         if timeout:
-            timeoutDelay = reduce(operator.add, timeout)
+            timeoutDelay = sum(timeout)
         else:
             timeoutDelay = 60
         userDeferred = defer.Deferred()
-        lookupDeferred = threads.deferToThread(socket.gethostbyname, name)
+        lookupDeferred = threads.deferToThreadPool(
+            self.reactor, self.reactor.getThreadPool(),
+            socket.gethostbyname, name)
         cancelCall = self.reactor.callLater(
             timeoutDelay, self._cleanup, name, lookupDeferred)
         self._runningQueries[lookupDeferred] = (userDeferred, cancelCall)
         lookupDeferred.addBoth(self._checkTimeout, name, lookupDeferred)
         return userDeferred
 
+
+
 class BlockingResolver:
     implements(IResolverSimple)
 
@@ -432,6 +454,8 @@ class ReactorBase(object):
         self.running = False
         self._started = False
         self._justStopped = False
+        # reactor internal readers, e.g. the waker.
+        self._internalReaders = set()
         self.waker = None
 
         # Arrange for the running attribute to change to True at the right time
@@ -460,12 +484,13 @@ class ReactorBase(object):
         return oldResolver
 
     def wakeUp(self):
-        """Wake up the event loop."""
-        if not threadable.isInIOThread():
-            if self.waker:
-                self.waker.wakeUp()
-            # if the waker isn't installed, the reactor isn't running, and
-            # therefore doesn't need to be woken up
+        """
+        Wake up the event loop.
+        """
+        if self.waker:
+            self.waker.wakeUp()
+        # if the waker isn't installed, the reactor isn't running, and
+        # therefore doesn't need to be woken up
 
     def doIteration(self, delay):
         """
@@ -624,10 +649,7 @@ class ReactorBase(object):
         guarantee is that it will be on its way to the running state.
         """
         if self._started:
-            warnings.warn(
-                    "Reactor already running! This behavior is deprecated "
-                    "since Twisted 8.0",
-                    category=DeprecationWarning, stacklevel=4)
+            raise error.ReactorAlreadyRunning()
         self._started = True
         self._stopped = False
         threadable.registerAsIOThread()
@@ -734,8 +756,7 @@ class ReactorBase(object):
                     break
             del self.threadCallQueue[:count]
             if self.threadCallQueue:
-                if self.waker:
-                    self.waker.wakeUp()
+                self.wakeUp()
 
         # insert new delayed calls now
         self._insertNewDelayedCalls()
@@ -860,6 +881,8 @@ class ReactorBase(object):
     # IReactorThreads
     if platform.supportsThreads():
         threadpool = None
+        # ID of the trigger starting the threadpool
+        _threadpoolStartupID = None
         # ID of the trigger stopping the threadpool
         threadpoolShutdownID = None
 
@@ -884,36 +907,52 @@ class ReactorBase(object):
             Create the threadpool accessible with callFromThread.
             """
             from twisted.python import threadpool
-            self.threadpool = threadpool.ThreadPool(0, 10, 'twisted.internet.reactor')
-            self.callWhenRunning(self.threadpool.start)
+            self.threadpool = threadpool.ThreadPool(
+                0, 10, 'twisted.internet.reactor')
+            self._threadpoolStartupID = self.callWhenRunning(
+                self.threadpool.start)
             self.threadpoolShutdownID = self.addSystemEventTrigger(
                 'during', 'shutdown', self._stopThreadPool)
 
         def _stopThreadPool(self):
             """
-            Stop the reactor threadpool.
+            Stop the reactor threadpool.  This method is only valid if there
+            is currently a threadpool (created by L{_initThreadPool}).  It
+            is not intended to be called directly; instead, it will be
+            called by a shutdown trigger created in L{_initThreadPool}.
             """
+            triggers = [self._threadpoolStartupID, self.threadpoolShutdownID]
+            for trigger in filter(None, triggers):
+                try:
+                    self.removeSystemEventTrigger(trigger)
+                except ValueError:
+                    pass
+            self._threadpoolStartupID = None
             self.threadpoolShutdownID = None
             self.threadpool.stop()
             self.threadpool = None
 
+
+        def getThreadPool(self):
+            """
+            See L{twisted.internet.interfaces.IReactorThreads.getThreadPool}.
+            """
+            if self.threadpool is None:
+                self._initThreadPool()
+            return self.threadpool
+
+
         def callInThread(self, _callable, *args, **kwargs):
             """
             See L{twisted.internet.interfaces.IReactorThreads.callInThread}.
             """
-            if self.threadpool is None:
-                self._initThreadPool()
-            self.threadpool.callInThread(_callable, *args, **kwargs)
+            self.getThreadPool().callInThread(_callable, *args, **kwargs)
 
         def suggestThreadPoolSize(self, size):
             """
             See L{twisted.internet.interfaces.IReactorThreads.suggestThreadPoolSize}.
             """
-            if size == 0 and self.threadpool is None:
-                return
-            if self.threadpool is None:
-                self._initThreadPool()
-            self.threadpool.adjustPoolsize(maxthreads=size)
+            self.getThreadPool().adjustPoolsize(maxthreads=size)
     else:
         # This is for signal handlers.
         def callFromThread(self, f, *args, **kw):
@@ -1022,9 +1061,7 @@ class BasePort(abstract.FileDescriptor):
     def createInternetSocket(self):
         s = socket.socket(self.addressFamily, self.socketType)
         s.setblocking(0)
-        if fcntl and hasattr(fcntl, 'FD_CLOEXEC'):
-            old = fcntl.fcntl(s.fileno(), fcntl.F_GETFD)
-            fcntl.fcntl(s.fileno(), fcntl.F_SETFD, old | fcntl.FD_CLOEXEC)
+        fdesc._setCloseOnExec(s.fileno())
         return s
 
 
@@ -1074,6 +1111,11 @@ class _SignalReactorMixin:
 
         if platformType == 'posix':
             signal.signal(signal.SIGCHLD, self._handleSigchld)
+            # Also call the signal handler right now, in case we missed any
+            # signals before we installed it.  This should only happen if
+            # someone used spawnProcess before calling reactor.run (and the
+            # process also exited already).
+            self._handleSigchld(signal.SIGCHLD, None)
 
 
     def _handleSigchld(self, signum, frame, _threadSupport=platform.supportsThreads()):
