@@ -1,7 +1,6 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Exceptions and errors for use in twisted.internet modules.
 
@@ -10,6 +9,10 @@ Maintainer: Itamar Shtull-Trauring
 
 import socket
 
+from twisted.python import deprecate
+from twisted.python.versions import Version
+
+
 
 class BindError(Exception):
     """An error occurred binding to an interface"""
@@ -233,6 +236,8 @@ class PotentialZombieWarning(Warning):
     """
     Emitted when L{IReactorProcess.spawnProcess} is called in a way which may
     result in termination of the created child process not being reported.
+
+    Deprecated in Twisted 10.0.
     """
     MESSAGE = (
         "spawnProcess called, but the SIGCHLD handler is not "
@@ -242,6 +247,12 @@ class PotentialZombieWarning(Warning):
         "never see this process finish, and it may become a "
         "zombie process.")
 
+deprecate.deprecatedModuleAttribute(
+    Version("Twisted", 10, 0, 0),
+    "There is no longer any potential for zombie process.",
+    __name__,
+    "PotentialZombieWarning")
+
 
 
 class ProcessDone(ConnectionDone):
