@@ -1,7 +1,6 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 Interface documentation.
 
@@ -54,7 +53,7 @@ class IConnector(Interface):
 
 
 class IResolverSimple(Interface):
-    
+
     def getHostByName(name, timeout = (1, 3, 11, 45)):
         """
         Resolve the domain name C{name} into an IP address.
@@ -194,7 +193,7 @@ class IResolver(IResolverSimple):
 
 
 class IReactorArbitrary(Interface):
-    
+
     def listenWith(portType, *args, **kw):
         """
         Start an instance of the given C{portType} listening.
@@ -392,9 +391,6 @@ class IReactorUNIXDatagram(Interface):
 class IReactorUDP(Interface):
     """
     UDP socket methods.
-
-    IMPORTANT: This is an experimental new interface. It may change
-    without backwards compatability. Suggestions are welcome.
     """
 
     def listenUDP(port, protocol, interface='', maxPacketSize=8192):
@@ -404,14 +400,6 @@ class IReactorUDP(Interface):
         @return: object which provides L{IListeningPort}.
         """
 
-    def connectUDP(remotehost, remoteport, protocol, localport=0,
-                  interface='', maxPacketSize=8192):
-        """
-        DEPRECATED.
-
-        Connects a L{twisted.internet.protocol.ConnectedDatagramProtocol}
-        instance to a UDP port.
-        """
 
 
 class IReactorMulticast(Interface):
@@ -454,8 +442,24 @@ class IReactorProcess(Interface):
                      sequence of strings. The first string should be the
                      executable's name.
 
-        @param env: the environment variables to pass to the processs; a
-                    dictionary of strings. If 'None', use os.environ.
+        @type env: a C{dict} mapping C{str} to C{str}, or C{None}.
+        @param env: the environment variables to pass to the child process. The
+                    resulting behavior varies between platforms. If
+                      - C{env} is not set:
+                        - On POSIX: pass an empty environment.
+                        - On Windows: pass C{os.environ}.
+                      - C{env} is C{None}:
+                        - On POSIX: pass C{os.environ}.
+                        - On Windows: pass C{os.environ}.
+                      - C{env} is a C{dict}:
+                        - On POSIX: pass the key/value pairs in C{env} as the
+                          complete environment.
+                        - On Windows: update C{os.environ} with the key/value
+                          pairs in the C{dict} before passing it. As a
+                          consequence of U{bug #1640
+                          <http://twistedmatrix.com/trac/ticket/1640>}, passing
+                          keys with empty values in an effort to unset
+                          environment variables I{won't} unset them.
 
         @param path: the path to run the subprocess in - defaults to the
                      current directory.
@@ -624,11 +628,21 @@ class IReactorThreads(Interface):
     Internally, this should use a thread pool and dispatch methods to them.
     """
 
+    def getThreadPool():
+        """
+        Return the threadpool used by L{callInThread}.  Create it first if
+        necessary.
+
+        @rtype: L{twisted.python.threadpool.ThreadPool}
+        """
+
+
     def callInThread(callable, *args, **kwargs):
         """
         Run the callable object in a separate thread.
         """
 
+
     def callFromThread(callable, *args, **kw):
         """
         Cause a function to be executed by the reactor thread.
@@ -644,6 +658,7 @@ class IReactorThreads(Interface):
         the same thread, use L{callLater} with a delay of 0.
         """
 
+
     def suggestThreadPoolSize(size):
         """
         Suggest the size of the internal threadpool used to dispatch functions
@@ -786,7 +801,7 @@ class IReactorPluggableResolver(Interface):
     """
     A reactor with a pluggable name resolver interface.
     """
-    
+
     def installResolver(resolver):
         """
         Set the internal resolver to use to for name lookups.
@@ -800,8 +815,7 @@ class IReactorPluggableResolver(Interface):
 
 class IReactorFDSet(Interface):
     """
-    Implement me to be able to use
-    L{FileDescriptor<twisted.internet.abstract.FileDescriptor>} type resources.
+    Implement me to be able to use L{IFileDescriptor} type resources.
 
     This assumes that your main-loop uses UNIX-style numeric file descriptors
     (or at least similarly opaque IDs returned from a .fileno() method)
@@ -823,7 +837,7 @@ class IReactorFDSet(Interface):
         I add writer to the set of file descriptors to get write events for.
 
         @param writer: An L{IWriteDescriptor} provider that will be checked for
-                       read events until it is removed from the reactor with
+                       write events until it is removed from the reactor with
                        L{removeWriter}.
 
         @return: C{None}.
@@ -907,7 +921,7 @@ class ILoggingContext(Interface):
     Give context information that will be used to log events generated by
     this item.
     """
-    
+
     def logPrefix():
         """
         @return: Prefix used during log formatting to indicate context.
@@ -917,12 +931,12 @@ class ILoggingContext(Interface):
 
 class IFileDescriptor(ILoggingContext):
     """
-    A file descriptor.
+    An interface representing a UNIX-style numeric file descriptor.
     """
 
     def fileno():
         """
-        @return: The platform-specified representation of a file-descriptor
+        @return: The platform-specified representation of a file descriptor
                  number.
         """
 
@@ -944,7 +958,13 @@ class IFileDescriptor(ILoggingContext):
                        failure may be of other classes as well.
         """
 
+
 class IReadDescriptor(IFileDescriptor):
+    """
+    An L{IFileDescriptor} that can read.
+
+    This interface is generally used in conjunction with L{IReactorFDSet}.
+    """
 
     def doRead():
         """
@@ -953,6 +973,11 @@ class IReadDescriptor(IFileDescriptor):
 
 
 class IWriteDescriptor(IFileDescriptor):
+    """
+    An L{IFileDescriptor} that can write.
+
+    This interface is generally used in conjunction with L{IReactorFDSet}.
+    """
 
     def doWrite():
         """
@@ -962,8 +987,7 @@ class IWriteDescriptor(IFileDescriptor):
 
 class IReadWriteDescriptor(IReadDescriptor, IWriteDescriptor):
     """
-    I am a L{FileDescriptor<twisted.internet.abstract.FileDescriptor>} that
-    can both read and write.
+    An L{IFileDescriptor} that can both read and write.
     """
 
 
@@ -1028,6 +1052,8 @@ class IConsumer(Interface):
         @param streaming: C{True} if C{producer} provides L{IPushProducer},
         C{False} if C{producer} provides L{IPullProducer}.
 
+        @raise RuntimeError: If a producer is already registered.
+
         @return: C{None}
         """
 
@@ -1045,7 +1071,7 @@ class IFinishableConsumer(IConsumer):
     """
     A Consumer for producers that finish.
     """
-    
+
     def finish():
         """
         The producer has finished producing.
