@@ -7,8 +7,12 @@ Twisted's automated release system.
 
 This module is only for use within Twisted's release system. If you are anyone
 else, do not use it. The interface and behaviour will change without notice.
+
+Only Linux is supported by this code.  It should not be used by any tools
+which must run on multiple platforms (eg the setup.py script).
 """
 
+import textwrap
 from datetime import date
 import sys
 import os
@@ -122,9 +126,19 @@ def changeAllProjectVersions(root, versionTemplate):
         version = Version(packageName, versionTemplate.major,
                           versionTemplate.minor, versionTemplate.micro,
                           prerelease=versionTemplate.prerelease)
+
+        # The placement of the top-level README with respect to other files (eg
+        # _version.py) is sufficiently different from the others that we just
+        # have to handle it specially.
+        if packageName == "twisted":
+            _changeVersionInFile(
+                project.getVersion(), version, root.child('README').path)
+
         project.updateVersion(version)
 
 
+
+
 class Project(object):
     """
     A representation of a project that has a version.
@@ -392,7 +406,8 @@ class APIBuilder(object):
     pydoctor to be installed and usable (which means you won't be able to
     use it with Python 2.3).
     """
-    def build(self, projectName, projectURL, sourceURL, packagePath, outputPath):
+    def build(self, projectName, projectURL, sourceURL, packagePath,
+              outputPath):
         """
         Call pydoctor's entry point with options which will generate HTML
         documentation for the specified package's API.
@@ -426,7 +441,7 @@ class APIBuilder(object):
              "--html-viewsource-base", sourceURL,
              "--add-package", packagePath.path,
              "--html-output", outputPath.path,
-             "--quiet", "--make-html"])
+             "--html-write-function-pages", "--quiet", "--make-html"])
 
 
 
@@ -569,6 +584,226 @@ class BookBuilder(LoreBuilderMixin):
 
 
 
+class NewsBuilder(object):
+    """
+    Generate the new section of a NEWS file.
+
+    The C{_FEATURE}, C{_BUGFIX}, C{_REMOVAL}, and C{_MISC} attributes of
+    this class are symbolic names for the news entry types which are
+    supported.  Conveniently, they each also take on the value of the file
+    name extension which indicates a news entry of that type.
+
+    @cvar _headings: A C{dict} mapping one of the news entry types to the
+        heading to write out for that type of news entry.
+
+    @cvar _TICKET_HINT: A C{str} giving the text which appears at the top of
+        each news file and which should be kept at the top, not shifted down
+        with all the other content.  Put another way, this is the text after
+        which the new news text is inserted.
+    """
+    _FEATURE = ".feature"
+    _BUGFIX = ".bugfix"
+    _REMOVAL = ".removal"
+    _MISC = ".misc"
+
+    _headings = {
+        _FEATURE: "Features",
+        _BUGFIX: "Bugfixes",
+        _REMOVAL: "Deprecations and Removals",
+        _MISC: "Other",
+        }
+
+    _TICKET_HINT = (
+        'Ticket numbers in this file can be looked up by visiting\n'
+        'http://twistedmatrix.com/trac/ticket/<number>\n'
+        '\n')
+
+    def _today(self):
+        """
+        Return today's date as a string in YYYY-MM-DD format.
+        """
+        return date.today().strftime('%Y-%m-%d')
+
+
+    def _findChanges(self, path, ticketType):
+        """
+        Load all the feature ticket summaries.
+
+        @param path: A L{FilePath} the direct children of which to search
+            for news entries.
+
+        @param ticketType: The type of news entries to search for.  One of
+            L{NewsBuilder._FEATURE}, L{NewsBuilder._BUGFIX},
+            L{NewsBuilder._REMOVAL}, or L{NewsBuilder._MISC}.
+
+        @return: A C{list} of two-tuples.  The first element is the ticket
+            number as an C{int}.  The second element of each tuple is the
+            description of the feature.
+        """
+        results = []
+        for child in path.children():
+            base, ext = os.path.splitext(child.basename())
+            if ext == ticketType:
+                results.append((
+                        int(base),
+                        ' '.join(child.getContent().splitlines())))
+        results.sort()
+        return results
+
+
+    def _writeHeader(self, fileObj, header):
+        """
+        Write a version header to the given file.
+
+        @param fileObj: A file-like object to which to write the header.
+        @param header: The header to write to the file.
+        @type header: C{str}
+        """
+        fileObj.write(header + '\n' + '=' * len(header) + '\n\n')
+
+
+    def _writeSection(self, fileObj, header, tickets):
+        """
+        Write out one section (features, bug fixes, etc) to the given file.
+
+        @param fileObj: A file-like object to which to write the news section.
+
+        @param header: The header for the section to write.
+        @type header: C{str}
+
+        @param tickets: A C{list} of ticket information of the sort returned
+            by L{NewsBuilder._findChanges}.
+        """
+        if not tickets:
+            return
+
+        reverse = {}
+        for (ticket, description) in tickets:
+            reverse.setdefault(description, []).append(ticket)
+        for description in reverse:
+            reverse[description].sort()
+        reverse = reverse.items()
+        reverse.sort(key=lambda (descr, tickets): tickets[0])
+
+        fileObj.write(header + '\n' + '-' * len(header) + '\n')
+        for (description, relatedTickets) in reverse:
+            ticketList = ', '.join([
+                    '#' + str(ticket) for ticket in relatedTickets])
+            entry = ' - %s (%s)' % (description, ticketList)
+            entry = textwrap.fill(entry, subsequent_indent='   ')
+            fileObj.write(entry + '\n')
+        fileObj.write('\n')
+
+
+    def _writeMisc(self, fileObj, header, tickets):
+        """
+        Write out a miscellaneous-changes section to the given file.
+
+        @param fileObj: A file-like object to which to write the news section.
+
+        @param header: The header for the section to write.
+        @type header: C{str}
+
+        @param tickets: A C{list} of ticket information of the sort returned
+            by L{NewsBuilder._findChanges}.
+        """
+        if not tickets:
+            return
+
+        fileObj.write(header + '\n' + '-' * len(header) + '\n')
+        formattedTickets = []
+        for (ticket, ignored) in tickets:
+            formattedTickets.append('#' + str(ticket))
+        entry = ' - ' + ', '.join(formattedTickets)
+        entry = textwrap.fill(entry, subsequent_indent='   ')
+        fileObj.write(entry + '\n\n')
+
+
+    def build(self, path, output, header):
+        """
+        Load all of the change information from the given directory and write
+        it out to the given output file.
+
+        @param path: A directory (probably a I{topfiles} directory) containing
+            change information in the form of <ticket>.<change type> files.
+        @type path: L{FilePath}
+
+        @param output: The NEWS file to which the results will be prepended.
+        @type output: L{FilePath}
+
+        @param header: The top-level header to use when writing the news.
+        @type header: L{str}
+        """
+        oldNews = output.getContent()
+        newNews = output.sibling('NEWS.new').open('w')
+        if oldNews.startswith(self._TICKET_HINT):
+            newNews.write(self._TICKET_HINT)
+            oldNews = oldNews[len(self._TICKET_HINT):]
+
+        self._writeHeader(newNews, header)
+        for part in (self._FEATURE, self._BUGFIX, self._REMOVAL):
+            tickets = self._findChanges(path, part)
+            self._writeSection(newNews, self._headings.get(part), tickets)
+        self._writeMisc(
+            newNews,
+            self._headings.get(self._MISC),
+            self._findChanges(path, self._MISC))
+        newNews.write(oldNews)
+        newNews.close()
+        output.sibling('NEWS.new').moveTo(output)
+
+
+    def buildAll(self, baseDirectory):
+        """
+        Find all of the Twisted subprojects beneath C{baseDirectory} and update
+        their news files from the ticket change description files in their
+        I{topfiles} directories and update the news file in C{baseDirectory}
+        with all of the news.
+
+        @param baseDirectory: A L{FilePath} representing the root directory
+            beneath which to find Twisted projects for which to generate
+            news (see L{findTwistedProjects}).
+        """
+        today = self._today()
+
+        # Get all the subprojects to generate news for
+        projects = findTwistedProjects(baseDirectory)
+        # And order them alphabetically for ease of reading
+        projects.sort(key=lambda proj: proj.directory.path)
+        # And generate them backwards since we write news by prepending to
+        # files.
+        projects.reverse()
+
+        for aggregateNews in [False, True]:
+            for project in projects:
+                topfiles = project.directory.child("topfiles")
+                if aggregateNews:
+                    news = baseDirectory.child("NEWS")
+                else:
+                    news = topfiles.child("NEWS")
+                name = project.directory.basename().title()
+                if name == 'Twisted':
+                    name = 'Core'
+                version = project.getVersion().base()
+                self.build(
+                    topfiles, news, "Twisted %s %s (%s)" % (name, version, today))
+
+
+    def main(self, args):
+        """
+        Build all news files.
+
+        @param args: The command line arguments to process.  This must contain
+            one string, the path to the base of the Twisted checkout for which
+            to build the news.
+        @type args: C{list} of C{str}
+        """
+        if len(args) != 1:
+            sys.exit("Must specify one argument: the path to the Twisted checkout")
+        self.buildAll(FilePath(args[0]))
+
+
+
 def filePathDelta(origin, destination):
     """
     Return a list of strings that represent C{destination} as a path relative
@@ -591,6 +826,8 @@ def filePathDelta(origin, destination):
     for elem1, elem2 in zip(path1, path2):
         if elem1 == elem2:
             commonItems += 1
+        else:
+            break
     path = [".."] * (len(path1) - commonItems)
     return path + path2[commonItems:]
 
@@ -984,3 +1221,45 @@ class BuildTarballsScript(object):
             sys.exit("Must specify two arguments: "
                      "Twisted checkout and destination path")
         self.buildAllTarballs(FilePath(args[0]), FilePath(args[1]))
+
+
+
+class BuildAPIDocsScript(object):
+    """
+    A thing for building API documentation. See L{main}.
+    """
+
+    def buildAPIDocs(self, projectRoot, output):
+        """
+        Build the API documentation of Twisted, with our project policy.
+
+        @param projectRoot: A L{FilePath} representing the root of the Twisted
+            checkout.
+        @param output: A L{FilePath} pointing to the desired output directory.
+        """
+        version = Project(projectRoot.child("twisted")).getVersion()
+        versionString = version.base()
+        sourceURL = ("http://twistedmatrix.com/trac/browser/tags/releases/"
+                     "twisted-%s" % (versionString,))
+        apiBuilder = APIBuilder()
+        apiBuilder.build(
+            "Twisted",
+            "http://twistedmatrix.com/",
+            sourceURL,
+            projectRoot.child("twisted"),
+            output)
+
+
+    def main(self, args):
+        """
+        Build API documentation.
+
+        @type args: list of str
+        @param args: The command line arguments to process.  This must contain
+            two strings: the path to the root of the Twisted checkout, and a
+            path to an output directory.
+        """
+        if len(args) != 2:
+            sys.exit("Must specify two arguments: "
+                     "Twisted checkout and distination path")
+        self.buildAPIDocs(FilePath(args[0]), FilePath(args[1]))
