@@ -1,11 +1,8 @@
 # Copyright (c) 2001-2004 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-# 
-
-"""Utilities for dealing with processes.
-
-API Stability: Unstable
+"""
+Utilities for dealing with processes.
 """
 
 import os
@@ -33,7 +30,10 @@ def which(name, flags=os.X_OK):
     """
     result = []
     exts = filter(None, os.environ.get('PATHEXT', '').split(os.pathsep))
-    for p in os.environ['PATH'].split(os.pathsep):
+    path = os.environ.get('PATH', None)
+    if path is None:
+        return []
+    for p in os.environ.get('PATH', '').split(os.pathsep):
         p = os.path.join(p, name)
         if os.access(p, flags):
             result.append(p)
