@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.python.test.test_util -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 import os, sys, hmac, errno, new, inspect, warnings
@@ -332,8 +332,9 @@ def str_xor(s, b):
     return ''.join([chr(ord(c) ^ b) for c in s])
 
 def keyed_md5(secret, challenge):
-    """Create the keyed MD5 string for the given secret and challenge."""
-    import warnings
+    """
+    Create the keyed MD5 string for the given secret and challenge.
+    """
     warnings.warn(
         "keyed_md5() is deprecated.  Use the stdlib module hmac instead.",
         DeprecationWarning, stacklevel=2
@@ -568,6 +569,13 @@ def dsu(list, key):
     L2.sort()
     return [e[2] for e in L2]
 
+try:
+    from twisted.python._initgroups import initgroups as _c_initgroups
+except ImportError:
+    _c_initgroups = None
+
+
+
 if pwd is None or grp is None or setgroups is None or getgroups is None:
     def initgroups(uid, primaryGid):
         """
@@ -576,6 +584,7 @@ if pwd is None or grp is None or setgroups is None or getgroups is None:
         Underlying platform support require to manipulate groups is missing.
         """
 else:
+    # Fallback to the inefficient Python version
     def _setgroups_until_success(l):
         while(1):
             # NASTY NASTY HACK (but glibc does it so it must be okay):
@@ -605,8 +614,11 @@ else:
         """
         Initializes the group access list.
 
-        This is done by reading the group database /etc/group and using all
-        groups of which C{uid} is a member.  The additional group
+        If the C extension is present, we're calling it, which in turn calls
+        initgroups(3).
+        
+        If not, this is done by reading the group database /etc/group and using
+        all groups of which C{uid} is a member.  The additional group
         C{primaryGid} is also added to the list.
 
         If the given user is a member of more than C{NGROUPS}, arbitrary
@@ -620,6 +632,8 @@ else:
         @param primaryGid: If provided, an additional GID to include when
             setting the groups.
         """
+        if _c_initgroups is not None:
+            return _c_initgroups(pwd.getpwuid(uid)[0], primaryGid)
         try:
             # Try to get the maximum number of groups
             max_groups = os.sysconf("SC_NGROUPS_MAX")
@@ -716,38 +730,13 @@ class SubclassableCStringIO(object):
 
 def moduleMovedForSplit(origModuleName, newModuleName, moduleDesc,
                         projectName, projectURL, globDict):
-    from twisted.python import reflect
-    modoc = """
-%(moduleDesc)s
-
-This module is DEPRECATED. It has been split off into a third party
-package, Twisted %(projectName)s. Please see %(projectURL)s.
-
-This is just a place-holder that imports from the third-party %(projectName)s
-package for backwards compatibility. To use it, you need to install
-that package.
-""" % {'moduleDesc': moduleDesc,
-       'projectName': projectName,
-       'projectURL': projectURL}
-
-    #origModule = reflect.namedModule(origModuleName)
-    try:
-        newModule = reflect.namedModule(newModuleName)
-    except ImportError:
-        raise ImportError("You need to have the Twisted %s "
-                          "package installed to use %s. "
-                          "See %s."
-                          % (projectName, origModuleName, projectURL))
-
-    # Populate the old module with the new module's contents
-    for k,v in vars(newModule).items():
-        globDict[k] = v
-    globDict['__doc__'] = modoc
-    import warnings
-    warnings.warn("%s has moved to %s. See %s." % (origModuleName, newModuleName,
-                                                   projectURL),
-                  DeprecationWarning, stacklevel=3)
-    return
+    """
+    No-op function; only present for backwards compatibility.  There is no
+    reason to call this function.
+    """
+    warnings.warn(
+        "moduleMovedForSplit is deprecated since Twisted 9.0.",
+        DeprecationWarning, stacklevel=2)
 
 
 def untilConcludes(f, *a, **kw):
@@ -759,6 +748,23 @@ def untilConcludes(f, *a, **kw):
                 continue
             raise
 
+_idFunction = id
+
+def setIDFunction(idFunction):
+    """
+    Change the function used by L{unsignedID} to determine the integer id value
+    of an object.  This is largely useful for testing to give L{unsignedID}
+    deterministic, easily-controlled behavior.
+
+    @param idFunction: A function with the signature of L{id}.
+    @return: The previous function being used by L{unsignedID}.
+    """
+    global _idFunction
+    oldIDFunction = _idFunction
+    _idFunction = idFunction
+    return oldIDFunction
+
+
 # A value about twice as large as any Python int, to which negative values
 # from id() will be added, moving them into a range which should begin just
 # above where positive values from id() leave off.
@@ -766,13 +772,20 @@ _HUGEINT = (sys.maxint + 1L) * 2L
 def unsignedID(obj):
     """
     Return the id of an object as an unsigned number so that its hex
-    representation makes sense
+    representation makes sense.
+
+    This is mostly necessary in Python 2.4 which implements L{id} to sometimes
+    return a negative value.  Python 2.3 shares this behavior, but also
+    implements hex and the %x format specifier to represent negative values as
+    though they were positive ones, obscuring the behavior of L{id}.  Python
+    2.5's implementation of L{id} always returns positive values.
     """
-    rval = id(obj)
+    rval = _idFunction(obj)
     if rval < 0:
         rval += _HUGEINT
     return rval
 
+
 def mergeFunctionMetadata(f, g):
     """
     Overwrite C{g}'s name and docstring with values from C{f}.  Update
@@ -951,5 +964,5 @@ __all__ = [
     "raises", "IntervalDifferential", "FancyStrMixin", "FancyEqMixin",
     "dsu", "switchUID", "SubclassableCStringIO", "moduleMovedForSplit",
     "unsignedID", "mergeFunctionMetadata", "nameToLabel", "uidFromString",
-    "gidFromString", "runAsEffectiveUser",
+    "gidFromString", "runAsEffectiveUser", "moduleMovedForSplit",
 ]
