@@ -1,23 +1,20 @@
-# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
 """
 FTP tests.
 
-Maintainer: U{Andrew Bennetts<mailto:spiv@twistedmatrix.com>}
+Maintainer: Andrew Bennetts
 """
 
-import os.path
-from StringIO import StringIO
-import shutil
+import os
 import errno
 
 from zope.interface import implements
 
-from twisted.trial import unittest
+from twisted.trial import unittest, util
 from twisted.protocols import basic
-from twisted.internet import reactor, protocol, defer, error
+from twisted.internet import reactor, task, protocol, defer, error
 from twisted.internet.interfaces import IConsumer
 from twisted.cred import portal, checkers, credentials
 from twisted.python import failure, filepath
@@ -25,12 +22,12 @@ from twisted.test import proto_helpers
 
 from twisted.protocols import ftp, loopback
 
-class NonClosingStringIO(StringIO):
-    def close(self):
-        pass
-
-StringIOWithoutClosing = NonClosingStringIO
 
+_changeDirectorySuppression = util.suppress(
+    category=DeprecationWarning,
+    message=(
+        r"FTPClient\.changeDirectory is deprecated in Twisted 8\.2 and "
+        r"newer\.  Use FTPClient\.cwd instead\."))
 
 
 class Dummy(basic.LineReceiver):
@@ -58,6 +55,7 @@ class _BufferingProtocol(protocol.Protocol):
         self.d.callback(self)
 
 
+
 class FTPServerTestCase(unittest.TestCase):
     """
     Simple tests for an FTP server with the default settings.
@@ -65,51 +63,51 @@ class FTPServerTestCase(unittest.TestCase):
     @ivar clientFactory: class used as ftp client.
     """
     clientFactory = ftp.FTPClientBasic
+    userAnonymous = "anonymous"
 
     def setUp(self):
         # Create a directory
         self.directory = self.mktemp()
         os.mkdir(self.directory)
+        self.dirPath = filepath.FilePath(self.directory)
 
         # Start the server
         p = portal.Portal(ftp.FTPRealm(self.directory))
         p.registerChecker(checkers.AllowAnonymousAccess(),
                           credentials.IAnonymous)
-        self.factory = ftp.FTPFactory(portal=p)
-        self.port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
+        self.factory = ftp.FTPFactory(portal=p,
+                                      userAnonymous=self.userAnonymous)
+        port = reactor.listenTCP(0, self.factory, interface="127.0.0.1")
+        self.addCleanup(port.stopListening)
 
         # Hook the server's buildProtocol to make the protocol instance
         # accessible to tests.
         buildProtocol = self.factory.buildProtocol
         d1 = defer.Deferred()
         def _rememberProtocolInstance(addr):
+            # Done hooking this.
+            del self.factory.buildProtocol
+
             protocol = buildProtocol(addr)
             self.serverProtocol = protocol.wrappedProtocol
+            def cleanupServer():
+                if self.serverProtocol.transport is not None:
+                    self.serverProtocol.transport.loseConnection()
+            self.addCleanup(cleanupServer)
             d1.callback(None)
             return protocol
         self.factory.buildProtocol = _rememberProtocolInstance
 
         # Connect a client to it
-        portNum = self.port.getHost().port
+        portNum = port.getHost().port
         clientCreator = protocol.ClientCreator(reactor, self.clientFactory)
         d2 = clientCreator.connectTCP("127.0.0.1", portNum)
         def gotClient(client):
             self.client = client
+            self.addCleanup(self.client.transport.loseConnection)
         d2.addCallback(gotClient)
         return defer.gatherResults([d1, d2])
 
-    def tearDown(self):
-        # Clean up sockets
-        self.client.transport.loseConnection()
-        d = defer.maybeDeferred(self.port.stopListening)
-        d.addCallback(self.ebTearDown)
-        return d
-
-    def ebTearDown(self, ignore):
-        del self.serverProtocol
-        # Clean up temporary directory
-        shutil.rmtree(self.directory)
-
     def assertCommandResponse(self, command, expectedResponseLines,
                               chainDeferred=None):
         """Asserts that a sending an FTP command receives the expected
@@ -153,6 +151,31 @@ class FTPServerTestCase(unittest.TestCase):
             chainDeferred=d)
 
 
+
+class FTPAnonymousTestCase(FTPServerTestCase):
+    """
+    Simple tests for an FTP server with different anonymous username.
+    The new anonymous username used in this test case is "guest"
+    """
+    userAnonymous = "guest"
+
+    def test_anonymousLogin(self):
+        """
+        Tests whether the changing of the anonymous username is working or not.
+        The FTP server should not comply about the need of password for the
+        username 'guest', letting it login as anonymous asking just an email
+        address as password.
+        """
+        d = self.assertCommandResponse(
+            'USER guest',
+            ['331 Guest login ok, type your email address as password.'])
+        return self.assertCommandResponse(
+            'PASS test@twistedmatrix.com',
+            ['230 Anonymous login ok, access restrictions apply.'],
+            chainDeferred=d)
+
+
+
 class BasicFTPServerTestCase(FTPServerTestCase):
     def testNotLoggedInReply(self):
         """When not logged in, all commands other than USER and PASS should
@@ -343,6 +366,18 @@ class BasicFTPServerTestCase(FTPServerTestCase):
                           protocol.Factory())
 
 
+    def test_portRangeInheritedFromFactory(self):
+        """
+        The L{FTP} instances created by L{ftp.FTPFactory.buildProtocol} have
+        their C{passivePortRange} attribute set to the same object the
+        factory's C{passivePortRange} attribute is set to.
+        """
+        portRange = xrange(2017, 2031)
+        self.factory.passivePortRange = portRange
+        protocol = self.factory.buildProtocol(None)
+        self.assertEqual(portRange, protocol.wrappedProtocol.passivePortRange)
+
+
 
 class FTPServerTestCaseAdvancedClient(FTPServerTestCase):
     """
@@ -460,6 +495,58 @@ class FTPServerPasvDataConnectionTestCase(FTPServerTestCase):
         return d
 
 
+    def test_NLSTEmpty(self):
+        """
+        NLST with no argument returns the directory listing for the current
+        working directory.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        # Touch a file in the current working directory
+        self.dirPath.child('test.txt').touch()
+        # Make a directory in the current working directory
+        self.dirPath.child('foo').createDirectory()
+
+        self._download('NLST ', chainDeferred=d)
+        def checkDownload(download):
+            filenames = download[:-2].split('\r\n')
+            filenames.sort()
+            self.assertEquals(['foo', 'test.txt'], filenames)
+        return d.addCallback(checkDownload)
+
+
+    def test_NLSTNonexistent(self):
+        """
+        NLST on a non-existent file/directory returns nothing.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        self._download('NLST nonexistent.txt', chainDeferred=d)
+        def checkDownload(download):
+            self.assertEquals('', download)
+        return d.addCallback(checkDownload)
+
+
+    def test_NLSTOnPathToFile(self):
+        """
+        NLST on an existent file returns only the path to that file.
+        """
+        # Login
+        d = self._anonymousLogin()
+
+        # Touch a file in the current working directory
+        self.dirPath.child('test.txt').touch()
+
+        self._download('NLST test.txt', chainDeferred=d)
+        def checkDownload(download):
+            filenames = download[:-2].split('\r\n')
+            self.assertEquals(['test.txt'], filenames)
+        return d.addCallback(checkDownload)
+
+
+
 class FTPServerPortDataConnectionTestCase(FTPServerPasvDataConnectionTestCase):
     def setUp(self):
         self.dataPorts = []
@@ -512,6 +599,143 @@ class FTPServerPortDataConnectionTestCase(FTPServerPasvDataConnectionTestCase):
         return d.addCallback(gotPortNum)
 
 
+
+class DTPFactoryTests(unittest.TestCase):
+    """
+    Tests for L{ftp.DTPFactory}.
+    """
+    def setUp(self):
+        """
+        Create a fake protocol interpreter and a L{ftp.DTPFactory} instance to
+        test.
+        """
+        self.reactor = task.Clock()
+
+        class ProtocolInterpreter(object):
+            dtpInstance = None
+
+        self.protocolInterpreter = ProtocolInterpreter()
+        self.factory = ftp.DTPFactory(
+            self.protocolInterpreter, None, self.reactor)
+
+
+    def test_setTimeout(self):
+        """
+        L{ftp.DTPFactory.setTimeout} uses the reactor passed to its initializer
+        to set up a timed event to time out the DTP setup after the specified
+        number of seconds.
+        """
+        # Make sure the factory's deferred fails with the right exception, and
+        # make it so we can tell exactly when it fires.
+        finished = []
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+        d.addCallback(finished.append)
+
+        self.factory.setTimeout(6)
+
+        # Advance the clock almost to the timeout
+        self.reactor.advance(5)
+
+        # Nothing should have happened yet.
+        self.assertFalse(finished)
+
+        # Advance it to the configured timeout.
+        self.reactor.advance(1)
+
+        # Now the Deferred should have failed with TimeoutError.
+        self.assertTrue(finished)
+
+        # There should also be no calls left in the reactor.
+        self.assertFalse(self.reactor.calls)
+
+
+    def test_buildProtocolOnce(self):
+        """
+        A L{ftp.DTPFactory} instance's C{buildProtocol} method can be used once
+        to create a L{ftp.DTP} instance.
+        """
+        protocol = self.factory.buildProtocol(None)
+        self.assertIsInstance(protocol, ftp.DTP)
+
+        # A subsequent call returns None.
+        self.assertIdentical(self.factory.buildProtocol(None), None)
+
+
+    def test_timeoutAfterConnection(self):
+        """
+        If a timeout has been set up using L{ftp.DTPFactory.setTimeout}, it is
+        cancelled by L{ftp.DTPFactory.buildProtocol}.
+        """
+        self.factory.setTimeout(10)
+        protocol = self.factory.buildProtocol(None)
+        # Make sure the call is no longer active.
+        self.assertFalse(self.reactor.calls)
+
+
+    def test_connectionAfterTimeout(self):
+        """
+        If L{ftp.DTPFactory.buildProtocol} is called after the timeout
+        specified by L{ftp.DTPFactory.setTimeout} has elapsed, C{None} is
+        returned.
+        """
+        # Handle the error so it doesn't get logged.
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+
+        # Set up the timeout and then cause it to elapse so the Deferred does
+        # fail.
+        self.factory.setTimeout(10)
+        self.reactor.advance(10)
+
+        # Try to get a protocol - we should not be able to.
+        self.assertIdentical(self.factory.buildProtocol(None), None)
+
+        # Make sure the Deferred is doing the right thing.
+        return d
+
+
+    def test_timeoutAfterConnectionFailed(self):
+        """
+        L{ftp.DTPFactory.deferred} fails with L{PortConnectionError} when
+        L{ftp.DTPFactory.clientConnectionFailed} is called.  If the timeout
+        specified with L{ftp.DTPFactory.setTimeout} expires after that, nothing
+        additional happens.
+        """
+        finished = []
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+        d.addCallback(finished.append)
+
+        self.factory.setTimeout(10)
+        self.assertFalse(finished)
+        self.factory.clientConnectionFailed(None, None)
+        self.assertTrue(finished)
+        self.reactor.advance(10)
+        return d
+
+
+    def test_connectionFailedAfterTimeout(self):
+        """
+        If L{ftp.DTPFactory.clientConnectionFailed} is called after the timeout
+        specified by L{ftp.DTPFactory.setTimeout} has elapsed, nothing beyond
+        the normal timeout before happens.
+        """
+        # Handle the error so it doesn't get logged.
+        d = self.assertFailure(self.factory.deferred, ftp.PortConnectionError)
+
+        # Set up the timeout and then cause it to elapse so the Deferred does
+        # fail.
+        self.factory.setTimeout(10)
+        self.reactor.advance(10)
+
+        # Now fail the connection attempt.  This should do nothing.  In
+        # particular, it should not raise an exception.
+        self.factory.clientConnectionFailed(None, defer.TimeoutError("foo"))
+
+        # Give the Deferred to trial so it can make sure it did what we
+        # expected.
+        return d
+
+
+
 # -- Client Tests -----------------------------------------------------------
 
 class PrintLines(protocol.Protocol):
@@ -614,17 +838,13 @@ class FTPFileListingTests(unittest.TestCase):
 
 
 class FTPClientTests(unittest.TestCase):
-    def tearDown(self):
-        # Clean up self.port, if any.
-        port = getattr(self, 'port', None)
-        if port is not None:
-            return port.stopListening()
 
     def testFailedRETR(self):
         f = protocol.Factory()
         f.noisy = 0
-        self.port = reactor.listenTCP(0, f, interface="127.0.0.1")
-        portNum = self.port.getHost().port
+        port = reactor.listenTCP(0, f, interface="127.0.0.1")
+        self.addCleanup(port.stopListening)
+        portNum = port.getHost().port
         # This test data derived from a bug report by ranty on #twisted
         responses = ['220 ready, dude (vsFTPd 1.0.0: beat me, break me)',
                      # USER anonymous
@@ -1322,6 +1542,25 @@ class FTPClientTestCase(unittest.TestCase):
         return d
 
 
+    def test_changeDirectoryDeprecated(self):
+        """
+        L{ftp.FTPClient.changeDirectory} is deprecated and the direct caller of
+        it is warned of this.
+        """
+        self._testLogin()
+        d = self.assertWarns(
+            DeprecationWarning,
+            "FTPClient.changeDirectory is deprecated in Twisted 8.2 and "
+            "newer.  Use FTPClient.cwd instead.",
+            __file__,
+            lambda: self.client.changeDirectory('.'))
+        # This is necessary to make the Deferred fire.  The Deferred needs
+        # to fire so that tearDown doesn't cause it to errback and fail this
+        # or (more likely) a later test.
+        self.client.lineReceived('250 success')
+        return d
+
+
     def test_changeDirectory(self):
         """
         Test the changeDirectory method.
@@ -1337,6 +1576,7 @@ class FTPClientTestCase(unittest.TestCase):
         self.assertEquals(self.transport.value(), 'CWD bar/foo\r\n')
         self.client.lineReceived('250 Requested File Action Completed OK')
         return d
+    test_changeDirectory.suppress = [_changeDirectorySuppression]
 
 
     def test_failedChangeDirectory(self):
@@ -1351,6 +1591,7 @@ class FTPClientTestCase(unittest.TestCase):
         self.assertEquals(self.transport.value(), 'CWD bar/foo\r\n')
         self.client.lineReceived('550 bar/foo: No such file or directory')
         return d
+    test_failedChangeDirectory.suppress = [_changeDirectorySuppression]
 
 
     def test_strangeFailedChangeDirectory(self):
@@ -1366,6 +1607,127 @@ class FTPClientTestCase(unittest.TestCase):
         self.assertEquals(self.transport.value(), 'CWD bar/foo\r\n')
         self.client.lineReceived('252 I do what I want !')
         return d
+    test_strangeFailedChangeDirectory.suppress = [_changeDirectorySuppression]
+
+
+    def test_renameFromTo(self):
+        """
+        L{ftp.FTPClient.rename} issues I{RNTO} and I{RNFR} commands and returns
+        a L{Deferred} which fires when a file has successfully been renamed.
+        """
+        self._testLogin()
+
+        d = self.client.rename("/spam", "/ham")
+        self.assertEqual(self.transport.value(), 'RNFR /spam\r\n')
+        self.transport.clear()
+
+        fromResponse = (
+            '350 Requested file action pending further information.\r\n')
+        self.client.lineReceived(fromResponse)
+        self.assertEqual(self.transport.value(), 'RNTO /ham\r\n')
+        toResponse = (
+            '250 Requested File Action Completed OK')
+        self.client.lineReceived(toResponse)
+
+        d.addCallback(self.assertEqual, ([fromResponse], [toResponse]))
+        return d
+
+
+    def test_renameFromToEscapesPaths(self):
+        """
+        L{ftp.FTPClient.rename} issues I{RNTO} and I{RNFR} commands with paths
+        escaped according to U{http://cr.yp.to/ftp/filesystem.html}.
+        """
+        self._testLogin()
+
+        fromFile = "/foo/ba\nr/baz"
+        toFile = "/qu\nux"
+        self.client.rename(fromFile, toFile)
+        self.client.lineReceived("350 ")
+        self.client.lineReceived("250 ")
+        self.assertEqual(
+            self.transport.value(),
+            "RNFR /foo/ba\x00r/baz\r\n"
+            "RNTO /qu\x00ux\r\n")
+
+
+    def test_renameFromToFailingOnFirstError(self):
+        """
+        The L{Deferred} returned by L{ftp.FTPClient.rename} is errbacked with
+        L{CommandFailed} if the I{RNFR} command receives an error response code
+        (for example, because the file does not exist).
+        """
+        self._testLogin()
+
+        d = self.client.rename("/spam", "/ham")
+        self.assertEqual(self.transport.value(), 'RNFR /spam\r\n')
+        self.transport.clear()
+
+        self.client.lineReceived('550 Requested file unavailable.\r\n')
+        # The RNTO should not execute since the RNFR failed.
+        self.assertEqual(self.transport.value(), '')
+
+        return self.assertFailure(d, ftp.CommandFailed)
+
+
+    def test_renameFromToFailingOnRenameTo(self):
+        """
+        The L{Deferred} returned by L{ftp.FTPClient.rename} is errbacked with
+        L{CommandFailed} if the I{RNTO} command receives an error response code
+        (for example, because the destination directory does not exist).
+        """
+        self._testLogin()
+
+        d = self.client.rename("/spam", "/ham")
+        self.assertEqual(self.transport.value(), 'RNFR /spam\r\n')
+        self.transport.clear()
+
+        self.client.lineReceived('350 Requested file action pending further information.\r\n')
+        self.assertEqual(self.transport.value(), 'RNTO /ham\r\n')
+        self.client.lineReceived('550 Requested file unavailable.\r\n')
+        return self.assertFailure(d, ftp.CommandFailed)
+
+
+    def test_makeDirectory(self):
+        """
+        L{ftp.FTPClient.makeDirectory} issues a I{MKD} command and returns a
+        L{Deferred} which is called back with the server's response if the
+        directory is created.
+        """
+        self._testLogin()
+
+        d = self.client.makeDirectory("/spam")
+        self.assertEquals(self.transport.value(), 'MKD /spam\r\n')
+        self.client.lineReceived('257 "/spam" created.')
+        return d.addCallback(self.assertEqual, ['257 "/spam" created.'])
+
+
+    def test_makeDirectoryPathEscape(self):
+        """
+        L{ftp.FTPClient.makeDirectory} escapes the path name it sends according
+        to U{http://cr.yp.to/ftp/filesystem.html}.
+        """
+        self._testLogin()
+        d = self.client.makeDirectory("/sp\nam")
+        self.assertEqual(self.transport.value(), 'MKD /sp\x00am\r\n')
+        # This is necessary to make the Deferred fire.  The Deferred needs
+        # to fire so that tearDown doesn't cause it to errback and fail this
+        # or (more likely) a later test.
+        self.client.lineReceived('257 win')
+        return d
+
+
+    def test_failedMakeDirectory(self):
+        """
+        L{ftp.FTPClient.makeDirectory} returns a L{Deferred} which is errbacked
+        with L{CommandFailed} if the server returns an error response code.
+        """
+        self._testLogin()
+
+        d = self.client.makeDirectory("/spam")
+        self.assertEquals(self.transport.value(), 'MKD /spam\r\n')
+        self.client.lineReceived('550 PERMISSION DENIED')
+        return self.assertFailure(d, ftp.CommandFailed)
 
 
     def test_getDirectory(self):
@@ -1414,15 +1776,65 @@ class FTPClientTestCase(unittest.TestCase):
         return d
 
 
+    def test_removeFile(self):
+        """
+        L{ftp.FTPClient.removeFile} sends a I{DELE} command to the server for
+        the indicated file and returns a Deferred which fires after the server
+        sends a 250 response code.
+        """
+        self._testLogin()
+        d = self.client.removeFile("/tmp/test")
+        self.assertEquals(self.transport.value(), 'DELE /tmp/test\r\n')
+        response = '250 Requested file action okay, completed.'
+        self.client.lineReceived(response)
+        return d.addCallback(self.assertEqual, [response])
 
-class DummyTransport:
-    def write(self, bytes):
-        pass
 
-class BufferingTransport:
-    buffer = ''
-    def write(self, bytes):
-        self.buffer += bytes
+    def test_failedRemoveFile(self):
+        """
+        If the server returns a response code other than 250 in response to a
+        I{DELE} sent by L{ftp.FTPClient.removeFile}, the L{Deferred} returned
+        by C{removeFile} is errbacked with a L{Failure} wrapping a
+        L{CommandFailed}.
+        """
+        self._testLogin()
+        d = self.client.removeFile("/tmp/test")
+        self.assertEquals(self.transport.value(), 'DELE /tmp/test\r\n')
+        response = '501 Syntax error in parameters or arguments.'
+        self.client.lineReceived(response)
+        d = self.assertFailure(d, ftp.CommandFailed)
+        d.addCallback(lambda exc: self.assertEqual(exc.args, ([response],)))
+        return d
+
+
+    def test_unparsableRemoveFileResponse(self):
+        """
+        If the server returns a response line which cannot be parsed, the
+        L{Deferred} returned by L{ftp.FTPClient.removeFile} is errbacked with a
+        L{BadResponse} containing the response.
+        """
+        self._testLogin()
+        d = self.client.removeFile("/tmp/test")
+        response = '765 blah blah blah'
+        self.client.lineReceived(response)
+        d = self.assertFailure(d, ftp.BadResponse)
+        d.addCallback(lambda exc: self.assertEqual(exc.args, ([response],)))
+        return d
+
+
+    def test_multilineRemoveFileResponse(self):
+        """
+        If the server returns multiple response lines, the L{Deferred} returned
+        by L{ftp.FTPClient.removeFile} is still fired with a true value if the
+        ultimate response code is 250.
+        """
+        self._testLogin()
+        d = self.client.removeFile("/tmp/test")
+        response = ['250-perhaps a progress report',
+                    '250 okay']
+        map(self.client.lineReceived, response)
+        return d.addCallback(self.assertTrue)
+
 
 
 class FTPClientBasicTests(unittest.TestCase):
@@ -1442,7 +1854,7 @@ class FTPClientBasicTests(unittest.TestCase):
 
     def testMultilineResponse(self):
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = DummyTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Imaginary FTP.')
 
         # Queue (and send) a dummy command, and set up a callback to capture the
@@ -1487,44 +1899,51 @@ class FTPClientBasicTests(unittest.TestCase):
              '321',
              '210 Done.'], result[0])
 
-    def testNoPasswordGiven(self):
-        """Passing None as the password avoids sending the PASS command."""
+
+    def test_noPasswordGiven(self):
+        """
+        Passing None as the password avoids sending the PASS command.
+        """
         # Create a client, and give it a greeting.
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = BufferingTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Welcome to Imaginary FTP.')
 
         # Queue a login with no password
         ftpClient.queueLogin('bob', None)
-        self.failUnlessEqual('USER bob\r\n', ftpClient.transport.buffer)
+        self.assertEquals('USER bob\r\n', ftpClient.transport.value())
 
         # Clear the test buffer, acknowledge the USER command.
-        ftpClient.transport.buffer = ''
+        ftpClient.transport.clear()
         ftpClient.lineReceived('200 Hello bob.')
 
         # The client shouldn't have sent anything more (i.e. it shouldn't have
         # sent a PASS command).
-        self.failUnlessEqual('', ftpClient.transport.buffer)
+        self.assertEquals('', ftpClient.transport.value())
 
-    def testNoPasswordNeeded(self):
-        """Receiving a 230 response to USER prevents PASS from being sent."""
+
+    def test_noPasswordNeeded(self):
+        """
+        Receiving a 230 response to USER prevents PASS from being sent.
+        """
         # Create a client, and give it a greeting.
         ftpClient = ftp.FTPClientBasic()
-        ftpClient.transport = BufferingTransport()
+        ftpClient.transport = proto_helpers.StringTransport()
         ftpClient.lineReceived('220 Welcome to Imaginary FTP.')
 
         # Queue a login with no password
         ftpClient.queueLogin('bob', 'secret')
-        self.failUnlessEqual('USER bob\r\n', ftpClient.transport.buffer)
+        self.assertEquals('USER bob\r\n', ftpClient.transport.value())
 
         # Clear the test buffer, acknowledge the USER command with a 230
         # response code.
-        ftpClient.transport.buffer = ''
+        ftpClient.transport.clear()
         ftpClient.lineReceived('230 Hello bob.  No password needed.')
 
         # The client shouldn't have sent anything more (i.e. it shouldn't have
         # sent a PASS command).
-        self.failUnlessEqual('', ftpClient.transport.buffer)
+        self.assertEquals('', ftpClient.transport.value())
+
 
 
 class PathHandling(unittest.TestCase):
@@ -2250,4 +2669,3 @@ class FTPReadWriteTestCase(unittest.TestCase, IReadWriteTestsMixin):
         Return the content of the temporary file.
         """
         return self.root.child(self.filename).getContent()
-
