@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -55,6 +55,7 @@ class AbstractFilePathTestCase(unittest.TestCase):
         f = self.subfile("sub3", "file3.ext3")
         f.close()
         self.path = filepath.FilePath(cmn)
+        self.root = filepath.FilePath("/")
 
 
     def test_segmentsFromPositive(self):
@@ -83,6 +84,23 @@ class AbstractFilePathTestCase(unittest.TestCase):
         self.assertEquals(set(x), set(self.all))
 
 
+    def test_parents(self):
+        """
+        L{FilePath.parents()} should return an iterator of every ancestor of
+        the L{FilePath} in question.
+        """
+        L = []
+        pathobj = self.path.child("a").child("b").child("c")
+        fullpath = pathobj.path
+        lastpath = fullpath
+        thispath = os.path.dirname(fullpath)
+        while lastpath != self.root.path:
+            L.append(thispath)
+            lastpath = thispath
+            thispath = os.path.dirname(thispath)
+        self.assertEquals([x.path for x in pathobj.parents()], L)
+
+
     def test_validSubdir(self):
         """Verify that a valid subdirectory will show up as a directory, but not as a
         file, not as a symlink, and be listable.
@@ -256,13 +274,59 @@ def zipit(dirname, zfname):
             zf.write(fspath, arcpath)
     zf.close()
 
-class ZipFilePathTestCase(AbstractFilePathTestCase):
 
+
+class ZipFilePathTestCase(AbstractFilePathTestCase):
+    """
+    Test various L{ZipPath} path manipulations as well as reprs for L{ZipPath}
+    and L{ZipArchive}.
+    """
     def setUp(self):
         AbstractFilePathTestCase.setUp(self)
-        zipit(self.cmn, self.cmn+'.zip')
-        self.path = ZipArchive(self.cmn+'.zip')
-        self.all = [x.replace(self.cmn, self.cmn+'.zip') for x in self.all]
+        zipit(self.cmn, self.cmn + '.zip')
+        self.path = ZipArchive(self.cmn + '.zip')
+        self.root = self.path
+        self.all = [x.replace(self.cmn, self.cmn + '.zip') for x in self.all]
+
+
+    def test_zipPathRepr(self):
+        """
+        Make sure that invoking ZipPath's repr prints the correct class name
+        and an absolute path to the zip file.
+        """
+        child = self.path.child("foo")
+        pathRepr = "ZipPath(%r)" % (
+            os.path.abspath(self.cmn + ".zip" + os.sep + 'foo'),)
+
+        # Check for an absolute path
+        self.assertEquals(repr(child), pathRepr)
+
+        # Create a path to the file rooted in the current working directory
+        relativeCommon = self.cmn.replace(os.getcwd() + os.sep, "", 1) + ".zip"
+        relpath = ZipArchive(relativeCommon)
+        child = relpath.child("foo")
+
+        # Check using a path without the cwd prepended
+        self.assertEquals(repr(child), pathRepr)
+
+
+    def test_zipArchiveRepr(self):
+        """
+        Make sure that invoking ZipArchive's repr prints the correct class
+        name and an absolute path to the zip file.
+        """
+        pathRepr = 'ZipArchive(%r)' % (os.path.abspath(self.cmn + '.zip'),)
+
+        # Check for an absolute path
+        self.assertEquals(repr(self.path), pathRepr)
+
+        # Create a path to the file rooted in the current working directory
+        relativeCommon = self.cmn.replace(os.getcwd() + os.sep, "", 1) + ".zip"
+        relpath = ZipArchive(relativeCommon)
+
+        # Check using a path without the cwd prepended
+        self.assertEquals(repr(relpath), pathRepr)
+
 
 
 class FilePathTestCase(AbstractFilePathTestCase):
@@ -350,6 +414,23 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertRaises(filepath.LinkError, iterateOverPath)
 
 
+    def test_walkObeysDescendWithCyclicalSymlinks(self):
+        """
+        Verify that, after making a path with cyclical symlinks, when the
+        supplied C{descend} predicate returns C{False}, the target is not
+        traversed, as if it was a simple symlink.
+        """
+        self.createLinks()
+        # we create cyclical symlinks
+        self.symlink(self.path.child("sub1").path,
+                     self.path.child("sub1").child("sub1.loopylink").path)
+        def noSymLinks(path):
+            return not path.islink()
+        def iterateOverPath():
+            return [foo.path for foo in self.path.walk(descend=noSymLinks)]
+        self.assertTrue(iterateOverPath())
+
+
     def test_walkObeysDescend(self):
         """
         Verify that when the supplied C{descend} predicate returns C{False},
