@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.trial.test.test_tests -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -12,12 +12,37 @@ Maintainer: Jonathan Lange
 import doctest, inspect
 import os, warnings, sys, tempfile, gc, types
 from pprint import pformat
+try:
+    from dis import findlinestarts as _findlinestarts
+except ImportError:
+    # Definition copied from Python's Lib/dis.py - findlinestarts was not
+    # available in Python 2.3.  This function is copyright Python Software
+    # Foundation, released under the Python license:
+    # http://www.python.org/psf/license/
+    def _findlinestarts(code):
+        """Find the offsets in a byte code which are start of lines in the source.
+
+        Generate pairs (offset, lineno) as described in Python/compile.c.
+
+        """
+        byte_increments = [ord(c) for c in code.co_lnotab[0::2]]
+        line_increments = [ord(c) for c in code.co_lnotab[1::2]]
+
+        lastlineno = None
+        lineno = code.co_firstlineno
+        addr = 0
+        for byte_incr, line_incr in zip(byte_increments, line_increments):
+            if byte_incr:
+                if lineno != lastlineno:
+                    yield (addr, lineno)
+                    lastlineno = lineno
+                addr += byte_incr
+            lineno += line_incr
+        if lineno != lastlineno:
+            yield (addr, lineno)
 
 from twisted.internet import defer, utils
 from twisted.python import components, failure, log, monkey
-from twisted.python.reflect import qual
-from twisted.python.compat import set
-from twisted.python import deprecate
 from twisted.python.deprecate import getDeprecationWarningString
 
 from twisted.trial import itrial, reporter, util
@@ -595,10 +620,6 @@ _logObserver = _LogObserver()
 
 _wait_is_running = []
 
-_classFixturesDeprecationMessage = (
-    "%(method)s, deprecated since Twisted 8.2.0, was overridden by "
-    "%(class)s.  Use %(replace)s instead.")
-
 class TestCase(_Assertions):
     """
     A unit test. The atom of the unit testing universe.
@@ -634,20 +655,14 @@ class TestCase(_Assertions):
     raised in a test. Useful for testing deprecated code. See also
     L{util.suppress}.
 
-    @ivar timeout: C{None} or a real number of seconds. If set, the test will
+    @ivar timeout: A real number of seconds. If set, the test will
     raise an error if it takes longer than C{timeout} seconds.
+    If not set, util.DEFAULT_TIMEOUT_DURATION is used.
 
     @ivar todo: C{None}, a string or a tuple of C{(errors, reason)} where
     C{errors} is either an exception class or an iterable of exception
     classes, and C{reason} is a string. See L{Todo} or L{makeTodo} for more
     information.
-
-    @ivar _suppressUpDownWarning: Private flag used by tests for C{setUpClass}
-        and C{tearDownClass} to suppress the deprecation warnings for these
-        methods.  This is necessary since the normal warning suppression
-        mechanism does not work for these warnings.  No code should use this
-        flag aside from tests for these methods.  When support for the methods
-        is removed altogether, so should this flag be removed.
     """
 
     implements(itrial.ITestCase)
@@ -667,21 +682,9 @@ class TestCase(_Assertions):
         testMethod = getattr(self, methodName)
         self._parents = [testMethod, self]
         self._parents.extend(util.getPythonContainers(testMethod))
-        self._shared = (hasattr(self, 'setUpClass') or
-                        hasattr(self, 'tearDownClass'))
-        if self._shared:
-            self._prepareClassFixture()
-            if not hasattr(self.__class__, '_instances'):
-                self._initInstances()
-            self.__class__._instances.add(self)
         self._passed = False
         self._cleanups = []
 
-    def _initInstances(cls):
-        cls._instances = set()
-        cls._instancesRun = set()
-    _initInstances = classmethod(_initInstances)
-
     if sys.version_info >= (2, 6):
         # Override the comparison defined by the base TestCase which considers
         # instances of the same class with the same _testMethodName to be
@@ -700,24 +703,6 @@ class TestCase(_Assertions):
             return self is not other
 
 
-
-    def _isFirst(self):
-        return len(self.__class__._instancesRun) == 0
-
-    def _isLast(self):
-        return self.__class__._instancesRun == self.__class__._instances
-
-    def _prepareClassFixture(self):
-        """Lots of tests assume that test methods all run in the same instance
-        of TestCase.  This isn't true. Calling this method ensures that
-        self.__class__._testCaseInstance contains an instance of this class
-        that will remain the same for all tests from this class.
-        """
-        if not hasattr(self.__class__, '_testCaseInstance'):
-            self.__class__._testCaseInstance = self
-        if self.__class__._testCaseInstance.__class__ != self.__class__:
-            self.__class__._testCaseInstance = self
-
     def _run(self, methodName, result):
         from twisted.internet import reactor
         timeout = self.getTimeout()
@@ -743,11 +728,7 @@ class TestCase(_Assertions):
                     result.addError(self, f)
         onTimeout = utils.suppressWarnings(
             onTimeout, util.suppress(category=DeprecationWarning))
-        if self._shared:
-            test = self.__class__._testCaseInstance
-        else:
-            test = self
-        method = getattr(test, methodName)
+        method = getattr(self, methodName)
         d = defer.maybeDeferred(utils.runWithWarningsSuppressed,
                                 self.getSuppress(), method)
         call = reactor.callLater(timeout, onTimeout, d)
@@ -763,49 +744,6 @@ class TestCase(_Assertions):
     def __call__(self, *args, **kwargs):
         return self.run(*args, **kwargs)
 
-    def deferSetUpClass(self, result):
-        """
-        Run the per-class set up fixture, C{setUpClass}, for this test case.
-
-        This must be called only once per TestCase subclass, since it will
-        run the fixture unconditionally.
-
-        @type result: L{IReporter} provider
-        @param result: The result which will be used to report any problems
-            encountered in C{setUpClass}.
-
-        @return: A L{Deferred} which will fire with the result of
-            C{setUpClass} or with C{None} if there is no C{setUpClass}
-            defined.
-        """
-        if not hasattr(self, 'setUpClass'):
-            d = defer.succeed(None)
-            d.addCallback(self.deferSetUp, result)
-            return d
-        if not getattr(self, '_suppressUpDownWarning', None):
-            warn = deprecate.getWarningMethod()
-            warn(_classFixturesDeprecationMessage % {
-                    'method': 'setUpClass',
-                    'class': qual(self.__class__),
-                    'replace': 'setUp'},
-                category=DeprecationWarning,
-                stacklevel=0)
-        d = self._run('setUpClass', result)
-        d.addCallbacks(self.deferSetUp, self._ebDeferSetUpClass,
-                       callbackArgs=(result,),
-                       errbackArgs=(result,))
-        return d
-
-    def _ebDeferSetUpClass(self, error, result):
-        if error.check(SkipTest):
-            result.addSkip(self, self._getReason(error))
-            self.__class__._instancesRun.remove(self)
-        elif error.check(KeyboardInterrupt):
-            result.stop()
-        else:
-            result.addError(self, error)
-            self.__class__._instancesRun.remove(self)
-
     def deferSetUp(self, ignored, result):
         d = self._run('setUp', result)
         d.addCallbacks(self.deferTestMethod, self._ebDeferSetUp,
@@ -829,8 +767,6 @@ class TestCase(_Assertions):
                        errbackArgs=(result,))
         d.addBoth(self.deferRunCleanups, result)
         d.addBoth(self.deferTearDown, result)
-        if self._shared and hasattr(self, 'tearDownClass') and self._isLast():
-            d.addBoth(self.deferTearDownClass, result)
         return d
 
     def _cbDeferTestMethod(self, ignored, result):
@@ -882,41 +818,6 @@ class TestCase(_Assertions):
                     result.stop()
                 self._passed = False
 
-    def deferTearDownClass(self, ignored, result):
-        """
-        Run the per-class tear down fixture, C{tearDownClass}, for this test
-        case.
-
-        This must be called only once per TestCase subclass, since it will
-        run the fixture unconditionally.  This must not be called if there
-        is no C{tearDownClass} method.
-
-        @param ignored: An ignored parameter.
-
-        @type result: L{IReporter} provider
-        @param result: The result which will be used to report any problems
-            encountered in C{tearDownClass}.
-
-        @return: A L{Deferred} which will fire with the result of
-            C{tearDownClass}.
-        """
-        if not getattr(self, '_suppressUpDownWarning', None):
-            warn = deprecate.getWarningMethod()
-            warn(_classFixturesDeprecationMessage % {
-                    'method': 'tearDownClass',
-                    'class': qual(self.__class__),
-                    'replace': 'tearDown'},
-                category=DeprecationWarning,
-                stacklevel=1)
-        d = self._run('tearDownClass', result)
-        d.addErrback(self._ebTearDownClass, result)
-        return d
-
-    def _ebTearDownClass(self, error, result):
-        if error.check(KeyboardInterrupt):
-            result.stop()
-        result.addError(self, error)
-
     def _cleanUp(self, result):
         try:
             clean = util._Janitor(self, result).postCaseCleanup()
@@ -1015,12 +916,6 @@ class TestCase(_Assertions):
         @raise ValueError: If C{offendingFunctions} is not C{None} and includes
             an object which is not a L{FunctionType} or L{MethodType} instance.
 
-        @raise IOError: If the source file (.py) for one of the functions in
-            C{offendingFunctions} is not available, its lines cannot be
-            determined and L{IOError} will be raised.  (It may be possible to
-            implement this function without requiring source files by using the
-            co_lnotab attribute of code objects.)
-
         @return: A C{list}, each element of which is a C{dict} giving
             information about one warning which was flushed by this call.  The
             keys of each C{dict} are:
@@ -1047,21 +942,35 @@ class TestCase(_Assertions):
             self._warnings[:] = []
         else:
             toFlush = []
-            for w in self._warnings:
+            for aWarning in self._warnings:
                 for aFunction in offendingFunctions:
                     if not isinstance(aFunction, (
                             types.FunctionType, types.MethodType)):
                         raise ValueError("%r is not a function or method" % (
                                 aFunction,))
-                    filename = inspect.getabsfile(aFunction)
-                    if os.path.normcase(filename) != os.path.normcase(w.filename):
+
+                    # inspect.getabsfile(aFunction) sometimes returns a
+                    # filename which disagrees with the filename the warning
+                    # system generates.  This seems to be because a
+                    # function's code object doesn't deal with source files
+                    # being renamed.  inspect.getabsfile(module) seems
+                    # better (or at least agrees with the warning system
+                    # more often), and does some normalization for us which
+                    # is desirable.  inspect.getmodule() is attractive, but
+                    # somewhat broken in Python 2.3.  See Python bug 4845.
+                    aModule = sys.modules[aFunction.__module__]
+                    filename = inspect.getabsfile(aModule)
+
+                    if filename != os.path.normcase(aWarning.filename):
                         continue
-                    lines, start = inspect.getsourcelines(aFunction)
-                    if not (start <= w.lineno <= start + len(lines)):
+                    lineStarts = list(_findlinestarts(aFunction.func_code))
+                    first = lineStarts[0][1]
+                    last = lineStarts[-1][1]
+                    if not (first <= aWarning.lineno <= last):
                         continue
                     # The warning points to this function, flush it and move on
                     # to the next warning.
-                    toFlush.append(w)
+                    toFlush.append(aWarning)
                     break
             # Remove everything which is being flushed.
             map(self._warnings.remove, toFlush)
@@ -1168,8 +1077,6 @@ class TestCase(_Assertions):
         else:
             result = new_result
         self._timedOut = False
-        if self._shared and self not in self.__class__._instances:
-            self.__class__._instances.add(self)
         result.startTest(self)
         if self.getSkip(): # don't run test methods that are marked as .skip
             result.addSkip(self, self.getSkip())
@@ -1181,25 +1088,14 @@ class TestCase(_Assertions):
         # by it will be collected and retrievable by flushWarnings.
         def runThunk():
             self._passed = False
-            first = False
-            if self._shared:
-                first = self._isFirst()
-                self.__class__._instancesRun.add(self)
             self._deprecateReactor(reactor)
             try:
-                if first:
-                    d = self.deferSetUpClass(result)
-                else:
-                    d = self.deferSetUp(None, result)
+                d = self.deferSetUp(None, result)
                 try:
                     self._wait(d)
                 finally:
                     self._cleanUp(result)
-                    if self._shared and self._isLast():
-                        self._initInstances()
-                        self._classCleanUp(result)
-                    if not self._shared:
-                        self._classCleanUp(result)
+                    self._classCleanUp(result)
             finally:
                 self._undeprecateReactor(reactor)
 
@@ -1381,6 +1277,7 @@ class UnsupportedTrialFeature(Exception):
     """A feature of twisted.trial was used that pyunit cannot support."""
 
 
+
 class PyUnitResultAdapter(object):
     """
     Wrap a C{TestResult} from the standard library's C{unittest} so that it
@@ -1396,10 +1293,7 @@ class PyUnitResultAdapter(object):
         self.original = original
 
     def _exc_info(self, err):
-        if isinstance(err, failure.Failure):
-            # Unwrap the Failure into a exc_info tuple.
-            err = (err.type, err.value, err.getTracebackObject())
-        return err
+        return util.excInfoOrFailureToExcInfo(err)
 
     def startTest(self, method):
         self.original.startTest(method)
@@ -1700,4 +1594,4 @@ for methodName in _assertions:
     globals()[methodName] = _deprecate(methodName)
 
 
-__all__ = ['TestCase', 'wait', 'FailTest', 'SkipTest']
+__all__ = ['TestCase', 'FailTest', 'SkipTest']
